/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "centralwidget.h"
#include "ui_attachdialog.h"

#include "stracerunner.h"

#include "kdebug.h"

#include <QSortFilterProxyModel>
#include <QHeaderView>
#include <QInputDialog>
#include <QTreeView>
#include <ksysguard/ProcessModel.h>
#include <ksysguard/process.h>
#include "customitemdelegate.h"

CentralWidget::CentralWidget(QWidget* parent) : QWidget(parent)
{
    setupUi(this);

    m_proxyModel = new QSortFilterProxyModel(this);
    sysCallView->setModel(m_proxyModel);

    sysCallView->setCornerButtonEnabled(false);
    sysCallView->setShowGrid(false);
    sysCallView->verticalHeader()->hide();
    QItemDelegate *customDelegate = new CustomItemDelegate();
    sysCallView->setItemDelegateForColumn(2,customDelegate);
    sysCallView->setItemDelegateForColumn(3,customDelegate);

    startButton->setEnabled(true);
    stopButton->setEnabled(false);
    commandLine->setEnabled(true);

    connect(filterLine, SIGNAL(textChanged(const QString&)),
            this, SLOT(filterTextChanged(const QString&)));
    connect(startButton, SIGNAL(clicked()), this, SLOT(startButtonClicked()));
    connect(stopButton, SIGNAL(clicked()), this, SLOT(stopButtonClicked()));
    connect(attachButton, SIGNAL(clicked()), this, SLOT(attachButtonClicked()));
}

void CentralWidget::setSysCallModel(QAbstractItemModel* model)
{
    if (!model) return;

    m_proxyModel->setSourceModel(model);
    sysCallView->hideColumn(0);
    sysCallView->horizontalHeader()->setResizeMode(2, QHeaderView::Stretch);
}

void CentralWidget::setStraceRunner(StraceRunner* straceRunner)
{
    m_straceRunner = straceRunner;
    QString command = straceRunner->tracedCommand();
    if (command.length() > 0) {
        commandLine->setText(command);
    }
    connect(m_straceRunner,
            SIGNAL(traceFinished(int)),
            this,
            SLOT(traceFinished(int)));
    connect(m_straceRunner,
            SIGNAL(tracing()),
            this,
            SLOT(tracing()));
    connect(m_straceRunner,
            SIGNAL(tracing()),
            m_proxyModel->sourceModel(),
            SLOT(restart()));
}

void CentralWidget::filterTextChanged(const QString& text)
{
    kDebug() << "Filter text: " << text;
    QRegExp filter(text);
    m_proxyModel->setFilterRegExp(filter);
}

void CentralWidget::traceFinished(int exitCode)
{
    Q_UNUSED(exitCode);
    kDebug() << "CentralWidget::traceFinished()";
    startButton->setEnabled(true);
    attachButton->setEnabled(true);
    stopButton->setEnabled(false);
    commandLine->setEnabled(true);
}

void CentralWidget::tracing()
{
    kDebug() << "CentralWidget::tracing()";
    startButton->setEnabled(false);
    attachButton->setEnabled(false);
    stopButton->setEnabled(true);
    commandLine->setEnabled(false);
}

void CentralWidget::startButtonClicked()
{
    kDebug() << "CentralWidget::startButtonClicked()";

    QStringList cmdArgs = commandLine->text().split(QRegExp("\\s+"));
    kDebug() << "cmdArgs: " << cmdArgs;
    if (! cmdArgs.count() > 0) return;

    QString exec = cmdArgs.takeFirst();
    m_straceRunner->setTraceCommand(exec, cmdArgs);
    m_straceRunner->traceCommand();
}

void CentralWidget::attachButtonClicked()
{
    kDebug() << "CentralWidget::startButtonClicked()";

    QDialog attachDialog(this);
    Ui::AttachDialog ui;
    ui.setupUi(&attachDialog);
    ui.ksysguardprocesslist->treeView()->setContextMenuPolicy(Qt::PreventContextMenu);
    ui.ksysguardprocesslist->treeView()->setSelectionMode(QAbstractItemView::SingleSelection);
    if (!attachDialog.exec()) return;
    QList<KSysGuard::Process*> selected=ui.ksysguardprocesslist->selectedProcesses();
    if (selected.empty()) return;
    
    m_straceRunner->setPID(selected[0]->pid);
    m_straceRunner->traceCommand();
}


void CentralWidget::stopButtonClicked()
{
    kDebug() << "CentralWidget::stopButtonClicked()";
    m_straceRunner->stopTrace();
}

#include "centralwidget.moc"
