#include <QFileInfo>
#include <QDir>
#include <sys/socket.h>

#include "filetracker.h"
#include "addressdecoder.h"
#include "kdebug.h"

FileTracker *FileTracker::m_instance = 0;

FileOpenInfo::FileOpenInfo(double time, int fd, const QString& fileName)
        : m_time(time), m_fd(fd), m_listening(false), m_info(fileName), m_type(File)
{
    kDebug() << "FileOpenInfo created" << endl;
}

FileOpenInfo::FileOpenInfo() : m_time(-1), m_fd(-1), m_listening(false), m_info("Unknown"), m_type(Unknown)
{
}

FileOpenInfo::FileOpenInfo(double time, const FileOpenInfo& parent, int fd, const QString& remote) :
    m_time(time), m_fd(fd), m_local(parent.m_local), m_remote(remote), m_listening(false), m_info(parent.m_info), m_type(Socket)
{
}


FileOpenInfo::FileOpenInfo(double time, int fd, int family, int type, int protocol)
    : m_time(time), m_fd(fd), m_listening(false), m_type(Socket)
{
    m_info= "Family: " + AddressDecoder::family(family);
    m_info+="\nType: "+AddressDecoder::type(type);
    m_info+="\nProtocol: "+AddressDecoder::protocol(protocol);
}

FileOpenInfo::FileOpenInfo(double time, int fd, const QString& family, const QString& type, const QString& protocol)
    : m_time(time), m_fd(fd), m_listening(false),  m_type(Socket)
{
    m_info= "Family: " + family;
    m_info+="\nType: "+type;
    m_info+="\nProtocol: "+protocol;
}
    
void FileOpenInfo::setListening(double time)
{
    m_time=time;
    m_listening=true;
}

double FileOpenInfo::time() const 
{
    return m_time;
}

int FileOpenInfo::fd() const
{
    return m_fd;
}

QString FileOpenInfo::text() const
{
    switch (m_type) {
    case File:  return m_info;
    case Socket: return "Socket  ["+m_local+(m_remote.isEmpty() ? "]" : " -> "+m_remote+" ]");
    default: return m_info;
    }
}

QString FileOpenInfo::extraInfo() const
{
    if (m_listening) return m_info+"\nState: listening";
    else return m_info;
}


void FileTracker::fileOpened(double time, int fd, const QString& fileName) 
{
    kDebug() << "Time: " << time;
    kDebug() << "FD: " << fd;
    kDebug() << "Filename: " << fileName;

    FileOpenInfo fileOpen(time, fd, fileName);
    addFileOpenInfo(fileOpen);
}

void FileTracker::socketCreated(double time, int fd, const QString& family, const QString& type, const QString& protocol)
{
    FileOpenInfo newSocket(time, fd, family, type, protocol);
    addFileOpenInfo(newSocket);
}

void FileTracker::addFileOpenInfo(const FileOpenInfo& fileOpenInfo)
{
    m_fileDescriptors[fileOpenInfo.fd()].append(fileOpenInfo);
}

FileOpenInfo FileTracker::lookupFileInfo(double time, int fd)
{
    FileOpenInfo fileInfo;

    kDebug() << "Looking up fd " << fd << " time " << time << endl;
    if (m_fileDescriptors.contains(fd)) {
        // TODO: Use a proper search algorithm
        QVectorIterator<FileOpenInfo> i(m_fileDescriptors[fd]);
        i.toBack();
        while (i.hasPrevious()) {
            FileOpenInfo fileInfoElement = i.previous();
            if (fileInfoElement.time() <= time) {
                return fileInfoElement;
            }
        }
    } else {
        kDebug() << "File descriptor is unknown" << endl;
    }

    return fileInfo;
}

void FileTracker::socketListen(double time, int fd) 
{
    FileOpenInfo socket=lookupFileInfo(time,fd);
    socket.setListening(time);
    addFileOpenInfo(socket);
}

void FileTracker::socketAddress(double time, int fd, const QString& addr, bool local) 
{
    FileOpenInfo socket=lookupFileInfo(time,fd);
    if (local) socket.setLocalAddress(addr, time);
    else socket.setRemoteAddress(addr, time);
    addFileOpenInfo(socket);
}

void FileTracker::socketAccepted(double time, int parentfd, int fd, const QString& addr) 
{
    FileOpenInfo socket=lookupFileInfo(time,parentfd);
    FileOpenInfo newsocket=FileOpenInfo(time, socket, fd, addr);
    addFileOpenInfo(newsocket);
}

void FileTracker::populateFromPID(pid_t pid) 
{
    QString path("/proc/");
    path+=QString::number(pid);
    path+="/fd";
    QDir fddir(path);
    if (!fddir.isReadable()) return;
    QFileInfoList fds=fddir.entryInfoList(QDir::Files);
    Q_FOREACH(QFileInfo fi,fds) {
	bool ok;
	int fd=fi.fileName().toInt(&ok);
	if (!ok || !fi.isSymLink()) continue;
	fileOpened(0,fd,fi.symLinkTarget());
    }
}
