/*  This file is part of the KDE project
    Copyright (C) 2007 Isaac Clerencia <isaac@warp.es>
    Copyright (C) 2007 Javier Uruen <juruen@warp.es>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    */

#include "stracerunner.h"
#include "filetracker.h"

#include <QFileSystemWatcher>

#include <kdebug.h>
#include <kapplication.h>
#include <ktemporaryfile.h>
#include <kprocess.h>

const char* STRACE_CMD = "/usr/bin/strace";

StraceRunner::StraceRunner() : m_strace(0), m_fileWatcher(0)
{
   m_straceArgs << "-ttt" << "-xx" << "-T";
   m_running = false;
   m_pid=0;
}

StraceRunner::~StraceRunner() 
{
}

QString StraceRunner::tracedCommand() const
{
    return m_exec;
}

void StraceRunner::setPID(pid_t pid)
{
    m_pid=pid;
    m_exec=QString::null;
}

pid_t StraceRunner::getPID() const
{
    return m_pid;
}

void StraceRunner::setTraceCommand(const QString & command,
                                const QStringList & args)
{
    m_exec = command;
    m_args = args;
    m_pid=0;
}

void StraceRunner::traceCommand()
{

    if (m_running) {
        kDebug() << "I'm already tracing a command...";
        return;
    }

    tracing();
    prepareTrace();
    if (m_pid) kdDebug() << "Tracing PID " << m_pid;
    else kDebug() << "Tracing cmd " << m_exec << " with args " << m_args;

    m_running = true;
    m_strace->setOutputChannelMode(KProcess::MergedChannels);
    m_strace->start();
    if (m_pid) FT->populateFromPID(m_pid);

}

void StraceRunner::stopTrace()
{
   m_strace->kill();
}

QStringList StraceRunner::straceArgs()
{
   return m_straceArgs;
}

void StraceRunner::processFinished(int exitCode)
{
    Q_UNUSED(exitCode);
    /* TODO Evaluate exit code */
    m_running = false;
    kDebug () << "Traced process finished with exit code " <<  exitCode;
    traceFinished(exitCode);
    m_strace->deleteLater();
    m_fileWatcher->deleteLater();
    m_strace=0;
    m_fileWatcher=0;
}

void StraceRunner::readyRead(){
    QStringList lines;

    QByteArray line;
    while (!(line = m_outFile->readLine()).isEmpty()) {
        //kDebug() << "Line at stracerunner " << line;
        lines += (m_partialLine + QString(line));
        m_partialLine = QString("");
    }

    if (lines.count() > 0) {
        //kDebug() << "Lines read " << lines.count();
        if (!lines.last().contains("\n")) {
            m_partialLine = lines.takeLast();
        }
        outputRead(lines);
    }
}

void StraceRunner::prepareTrace() 
{
    kDebug() << "Starting tracing cmd " << m_exec 
             << " with args " << m_args << endl;
 
    /* Create temporary file and open it with QFile */
    KTemporaryFile tempFile;
    tempFile.setAutoRemove(false);
    QString tempFileName;
    if (tempFile.open()) {
        tempFileName = tempFile.fileName();
        tempFile.close();
        m_outFile = new QFile(tempFileName);
        m_outFile->open(QIODevice::ReadOnly | QIODevice::Text);
    } else {
        kDebug() << "Couldn't open temporay file";
        return;
    }

    QStringList kprocArgs;	
    /* Add custom strace args */
    kprocArgs << m_straceArgs;
    /* Add strace arg to trace output to our temporary file */
    kprocArgs << "-o" << tempFileName ;
    /* Add program to trace and its arguments */
    if (!m_pid)  kprocArgs << m_exec << m_args;
    else kprocArgs << "-p" << QString::number(m_pid);

    kDebug() << "output in " <<  tempFileName;

    m_strace = new KProcess(this);
    m_strace->setProgram(QString(STRACE_CMD), kprocArgs);
     
    /* Create watcher for temporary file as that's where strace is going
     * to dump its output */
    m_fileWatcher = new QFileSystemWatcher(QStringList(tempFileName), this);

    connect(m_strace, 
            SIGNAL(finished (int)), 
            this, 
            SLOT(processFinished(int ))
           );

    connect(m_fileWatcher, 
            SIGNAL(fileChanged(QString)), 
            this, 
            SLOT(readyRead())
           );
}

#include "stracerunner.moc"
