set(SHAMAN_BACKEND "" CACHE STRING "Specifies the Shaman backend to build. Current available options are 
                                   Aqpm, Apt, QPackageKit, Fake, TestScript. Not setting this variable will build the most
                                   appropriate backend for your system")
## Check if the user did not specify a backend to be built. If that is the case,
## we check what is the best backend to build on this system.
## 4.4: We leave polkit-0.9 having more priority over polkit-1. This will change from 4.5 on
if(NOT SHAMAN_BACKEND)
    macro_optional_find_package(Aqpm)
    macro_optional_find_package(QApt)
    macro_optional_find_package(AptPkg)
    macro_optional_find_package(QPackageKit)
    macro_optional_find_package(AkabeiClient)

    if (AQPM_FOUND)
        set (SHAMAN_BACKEND "Aqpm")
    elseif (QAPT_FOUND)
        set (SHAMAN_BACKEND "QApt")
    elseif (APTPKG_FOUND)
        set (SHAMAN_BACKEND "Apt")
    elseif (QPACKAGEKIT_FOUND)
        set (SHAMAN_BACKEND "QPackageKit")
    elseif (AKABEICLIENT_FOUND)
        set (SHAMAN_BACKEND "Akabei")
    else ()
        macro_log_feature(AQPM_FOUND "Aqpm" "QT wrapper library around alpm, Arch Linux Package Management Library."
                          "http://chakra-project.org" FALSE "" "Provides a backend for Shaman.")
        macro_log_feature(QAPT_FOUND "QApt" "High-level Qt wrapper/Apt implementation around the Apt-pkg library"
                          "http://kubuntu.org" FALSE "" "Provides a backend for Shaman.")
        macro_log_feature(APTPKG_FOUND "Apt-pkg" "Libraries for developing with APT's libapt-pkg Debian package manipulation library"
                          "http://wiki.debian.org/Apt" FALSE "" "Provides a backend for Shaman.")
        macro_log_feature(QPACKAGEKIT_FOUND "PackageKit" "An universal package management system." "http://www.packagekit.org"
                          FALSE "" "Provides a backend for Shaman.")
        macro_log_feature(AKABEICLIENT_FOUND "Akabei" "Package Management System of the Chakra Project." "http://www.chakra-project.org"
                          FALSE "" "Provides a backend for Shaman.")
        message ("WARNING: No additional packages for building one of the Shaman Backends were found. Falling back to Fake backend")
        set (SHAMAN_BACKEND "Fake")
    endif ()

    # Case-insensitive
    string(TOUPPER ${SHAMAN_BACKEND} SHAMAN_BACKEND)
else(NOT SHAMAN_BACKEND)
    # Case-insensitive
    string(TOUPPER ${SHAMAN_BACKEND} SHAMAN_BACKEND)

    # Check if the specified backend is valid. If it is not, we fall back to the Fake one
    if (NOT SHAMAN_BACKEND STREQUAL "AQPM" AND NOT SHAMAN_BACKEND STREQUAL "APT" AND NOT SHAMAN_BACKEND STREQUAL "QAPT" AND NOT SHAMAN_BACKEND STREQUAL "QPACKAGEKIT" AND NOT SHAMAN_BACKEND STREQUAL "AKABEI" AND NOT SHAMAN_BACKEND STREQUAL "TESTSCRIPT" AND NOT SHAMAN_BACKEND STREQUAL "FAKE")
        message ("WARNING: The Shaman Backend ${SHAMAN_BACKEND} you specified does not exist. Falling back to Fake backend")
        set (SHAMAN_BACKEND "Fake")
    endif (NOT SHAMAN_BACKEND STREQUAL "AQPM" AND NOT SHAMAN_BACKEND STREQUAL "APT" AND NOT SHAMAN_BACKEND STREQUAL "QAPT" AND NOT SHAMAN_BACKEND STREQUAL "QPACKAGEKIT" AND NOT SHAMAN_BACKEND STREQUAL "AKABEI" AND NOT SHAMAN_BACKEND STREQUAL "TESTSCRIPT" AND NOT SHAMAN_BACKEND STREQUAL "FAKE")

    # Check requirements for each backend. If not, fall back to the fake one
    if (SHAMAN_BACKEND STREQUAL "AQPM")
        macro_optional_find_package(Aqpm)

        if (NOT AQPM_FOUND)
            message ("WARNING: You chose the Aqpm Shaman backend but you don't have Aqpm installed.
                      Falling back to Fake backend")
            set (SHAMAN_BACKEND "FAKE")
        endif (NOT AQPM_FOUND)
    elseif (SHAMAN_BACKEND STREQUAL "QAPT")
        macro_optional_find_package(AptPkg)
        macro_optional_find_package(QApt)

        if (NOT APTPKG_FOUND)
            message ("WARNING: You chose the QApt Shaman backend but you don't have Aptpkg installed.
                      Falling back to Fake backend")
                    set (SHAMAN_BACKEND "FAKE")
        endif (NOT APTPKG_FOUND)
        if (NOT QAPT_FOUND)
            message ("WARNING: You chose the QApt Shaman backend but you don't have QApt installed.
                      Falling back to Fake backend")
                    set (SHAMAN_BACKEND "FAKE")
        endif (NOT QAPT_FOUND)
    elseif (SHAMAN_BACKEND STREQUAL "APT")
        macro_optional_find_package(AptPkg)

        if (NOT APTPKG_FOUND)
            message ("WARNING: You chose the Apt Shaman backend but you don't have Apt installed.
                      Falling back to Fake backend")
                    set (SHAMAN_BACKEND "FAKE")
        endif (NOT APTPKG_FOUND)
    elseif (SHAMAN_BACKEND STREQUAL "QPACKAGEKIT")
        macro_optional_find_package(QPackageKit)

        if (NOT QPACKAGEKIT_FOUND)
            message ("WARNING: You chose the QPackageKit Shaman backend but you don't have QPackageKit installed.
                      Falling back to Fake backend")
            set (SHAMAN_BACKEND "FAKE")
        endif (NOT QPACKAGEKIT_FOUND)
    elseif (SHAMAN_BACKEND STREQUAL "AKABEI")
        macro_optional_find_package(AkabeiClient)

        if (NOT AKABEICLIENT_FOUND)
            message ("WARNING: You chose the Akabei Shaman backend but you don't have AkabeiClient installed.
                      Falling back to Fake backend")
            set (SHAMAN_BACKEND "FAKE")
        endif (NOT AKABEICLIENT_FOUND)
    endif ()
endif(NOT SHAMAN_BACKEND)

# Add the correct libraries and variables depending on the backend
if(SHAMAN_BACKEND STREQUAL "AQPM")
    message(STATUS "Building Shaman Aqpm Backend")
    macro_log_feature(AQPM_FOUND "Aqpm" "QT wrapper library around alpm, Arch Linux Package Management Library." "http://chakra-project.org" FALSE "" "Provides a backend for Shaman.")

    set_source_files_properties( backends/aqpm/AqpmPlugin.cpp
                             PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )

    include_directories(${AQPM_INCLUDE_DIR})

    set(BACKEND_SRCS
        backends/aqpm/AqpmPlugin.cpp
        backends/aqpm/AqpmPackage.cpp
        backends/aqpm/AqpmLocalPackage.cpp
        backends/aqpm/KdemodDistUpgrade.cpp
    )

    set(BACKEND_LIBS
        ${AQPM_LIBRARY}
    )

    find_package(PolkitQt)
    macro_log_feature(POLKITQT_FOUND "PolkitQt" "A Qt wrapper around PolicyKit." "http://www.kde.org" FALSE ""
                      "Required to build aqpm kcmodules")

    if (POLKITQT_FOUND)
        # Add configuration modules for aqpm
        include_directories(${POLKITQT_INCLUDE_DIR})
        add_subdirectory(backends/aqpm)
    endif (POLKITQT_FOUND)
elseif(SHAMAN_BACKEND STREQUAL "QAPT")
    message(STATUS "Building Shaman QApt Backend")
    macro_log_feature(APTPKG_FOUND "Apt-pkg" "Libraries for developing with APT's libapt-pkg Debian package manipulation library" "http://wiki.debian.org/Apt" FALSE "" "Provides a backend for Shaman.")
    macro_log_feature(QAPT_FOUND "QApt" "High-level Qt wrapper/Apt implementation around the Apt-pkg library" "http://kubuntu.org" FALSE "" "Provides a backend for Shaman.")
    set_source_files_properties( backends/qapt/AptBackend.cpp
                             PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )
    include_directories(${APTPKG_INCLUDE_DIR})
    set(BACKEND_SRCS
        backends/qapt/AptBackend.cpp
        backends/qapt/DebPackage.cpp
    )
    set(BACKEND_LIBS
        ${APTPKG_LIBRARIES}
        ${QAPT_LIBRARY}
    )
elseif(SHAMAN_BACKEND STREQUAL "APT")
    message(STATUS "Building Shaman Apt Backend")
    macro_log_feature(APTPKG_FOUND "Apt-pkg" "Libraries for developing with APT's libapt-pkg Debian package manipulation library" "http://wiki.debian.org/Apt" FALSE "" "Provides a backend for Shaman.")
    set_source_files_properties( backends/apt/AptBackend.cpp
                             PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )
    include_directories(${APTPKG_INCLUDE_DIR})
    set(BACKEND_SRCS
        backends/apt/AptBackend.cpp
        backends/apt/DebPackage.cpp
	backends/apt/OperationProgress.cpp
    )
    add_subdirectory(backends/apt)
    set(BACKEND_LIBS
        ${APTPKG_LIBRARIES}
    )
elseif(SHAMAN_BACKEND STREQUAL "QPACKAGEKIT")
    message(STATUS "Building Shaman QPackageKit Backend")
    macro_log_feature(QPACKAGEKIT_FOUND "PackageKit" "A common package management system." "http://www.packagekit.org" FALSE "" "Provides a backend for Shaman.")

    set_source_files_properties( backends/packagekit/PKPlugin.cpp
                             PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )

    find_package(KDE4Workspace 4.2.0 REQUIRED)

    include_directories(${QPACKAGEKIT_INCLUDE_DIR} ${KDE4WORKSPACE_INCLUDE_DIR})

    set(BACKEND_SRCS
        backends/packagekit/PKPlugin.cpp
        backends/packagekit/PKPackage.cpp
        backends/packagekit/PKPackageListener.cpp
    )

    set(BACKEND_LIBS
        ${QPACKAGEKIT_LIB}
        ${KDE4WORKSPACE_KWORKSPACE_LIBRARY}
    )
elseif(SHAMAN_BACKEND STREQUAL "AKABEI")
    message(STATUS "Building Shaman Akabei Backend")
    macro_log_feature(AKABEICLIENT_FOUND "Akabei" "Package Management System of the Chakra Project." "http://www.chakra-project.org"
                          FALSE "" "Provides a backend for Shaman.")
    find_package(AkabeiCore REQUIRED)
    
    set_source_files_properties( backends/akabei/AkabeiBackend.cpp
                             PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )

    include_directories(
        ${AKABEICORE_INCLUDE_DIR}
        ${AKABEICLIENT_INCLUDE_DIR}
    )

    set(BACKEND_SRCS
        backends/akabei/AkabeiBackend.cpp
        backends/akabei/AkabeiPackage.cpp
    )

    set(BACKEND_LIBS
        ${AKABEICORE_LIBRARIES}
        ${AKABEICLIENT_LIBRARIES}
    )
elseif(SHAMAN_BACKEND STREQUAL "TESTSCRIPT")
    message(STATUS "WARNING: This is a backend which is meant only for test purposes")

    set (testscript_SRCS
        backends/testscript/test.py
    )
    INSTALL(FILES ${testscript_SRCS} DESTINATION ${KDE4_SERVICES_INSTALL_DIR})
    set (SHAMAN_BACKEND_PATH "${KDE4_SERVICES_INSTALL_DIR}/test.py")
    set (USE_SCRIPTED_BACKEND TRUE)
else()
    message(STATUS "WARNING: Building Shaman Fake Backend")
    set_source_files_properties( backends/fake/FakeBackend.cpp
                                 PROPERTIES COMPILE_FLAGS -DQT_STATICPLUGIN )

    set(BACKEND_SRCS
        backends/fake/FakeBackend.cpp
        backends/fake/FakePackage.cpp
    )
endif()
