/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AptBackend.h"
#include "DebPackage.h"
#include "OperationProgress.h"

#include <kicon.h>
#include <kdebug.h>

#include <apt-pkg/sourcelist.h>
#include <apt-pkg/depcache.h>
#include <apt-pkg/pkgcache.h>
#include <apt-pkg/pkgcachegen.h>
#include <apt-pkg/policy.h>
#include <apt-pkg/init.h>


#include "shamantransactioninterface.h"
#include <QTimer>

using namespace KAuth;

AptBackend::AptBackend()
    : Shaman::BackendInterface(), mHelperID("org.kde.shaman.apt")
    , mList(0), mCache(0), mPolicy(0)
{
    
}

AptBackend::~AptBackend()
{
    delete mPolicy;
    delete mCache;
    delete mList;
}

void AptBackend::transactionQuestionAnswered(Shaman::TransactionQuestion* question)
{

}

void AptBackend::init()
{
    setCapabilities(/*Transaction capabilities*/
                    Shaman::BackendCapabilities::InstallPackagesCapability |
                    Shaman::BackendCapabilities::RemovePackagesCapability |
                    Shaman::BackendCapabilities::UpgradePackagesCapability |
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability |
                    Shaman::BackendCapabilities::UpgradeSystemCapability |
                    Shaman::BackendCapabilities::DistUpgradeCapability |
                    Shaman::BackendCapabilities::InstallPackageFromFileCapability,
                    /*Search capabilities*/
                    Shaman::BackendCapabilities::BasicSearchFromViewCapability,
                    /*Package Grouping capabilities*/
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability,
                    //Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    /*Information capabilities*/
                    Shaman::BackendCapabilities::ShowFileListCapability |
                    Shaman::BackendCapabilities::ShowConflictsWithCapability);

    Shaman::ProcessQueueOption::List options;
    options << new Shaman::ProcessQueueOption(i18n("Purge"),
                                              i18n("Purge configuration files when removing packages. "
                                                   "Valuable data might be lost."),
                                              QVariant("purge"));
    setAvailableProcessQueueOptions(options);
    KAboutPerson person(ki18n("David Palacio"), ki18n("Maintainer"), "dpalacio@uninorte.edu.co", "http://bakalacio.blogspot.com/");
    // TODO: Change package icon
    setProperties(i18n("Apt backend (not working)"), i18n("A backend that does almost nothing"), KIcon("applications-other"),
                  KIcon("application-x-deb"), QList<KAboutPerson>() << person, QStringList());
    pkgInitConfig(*_config) && pkgInitSystem(*_config,_system);
    setState(Shaman::Backend::Idling);
}

bool AptBackend::aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate)
{
    if (((oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||
         (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) &&
        backend()->operationMode() == Shaman::Base::CachedMode)
    {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
        // If we're initializing, just don't reject the state transition
        if (oldstate != Shaman::Backend::Initializing)
        {
            // Otherwise do.
            return false;
        }
    }

    // Else we're fine with defaults
    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

bool AptBackend::cancelQueueProcessing()
{
    return false; //todo?
    Action cancelProcess(mHelperID+".process");
    cancelProcess.setHelperID(mHelperID);
    
    QVariantMap args;
    args.insert("command", "stop");
    
    ActionReply reply = cancelProcess.execute();
    if(reply.succeeded())
    {
        setState(Shaman::Backend::Idling);
        return true;
    }
    else
    {
        //Should be Broken?
        kdDebug() << "Warning: Failed to stop queue processing";
        setState(Shaman::Backend::Idling);
        return false;
    }
}

bool AptBackend::cancelRepositoriesUpdate()
{
    Action cancelUpdate(mHelperID+".update");
    cancelUpdate.setHelperID(mHelperID);
    
    QVariantMap args;
    args.insert("command", "stop");
    cancelUpdate.setArguments(args);
    
    if(cancelUpdate.execute()!=ActionReply::SuccessReply)
    {
        kdDebug() << "Failed to cancel update";
        return false;
    }
    setState(Shaman::Backend::Idling);
    return true;
}

void AptBackend::updateRepositories()
{
    kdDebug() << "Interface: update repositories";
    Action update(mHelperID+".update");
    update.setHelperID(mHelperID);
    connect(update.watcher(), SIGNAL(actionPerformed(ActionReply)),
            this, SLOT(slotUpdateFinished()));
    update.setExecutesAsync(true);
    ActionReply reply = update.execute();
    if(reply!=ActionReply::SuccessReply)
    {
        kdDebug() << "Failed to update indices";
        setState(Shaman::Backend::Idling);
        //do something
    }
}

void AptBackend::reload()
{
    setState(Shaman::Backend::Reloading);
    if(!mList)
        mList = new pkgSourceList;
    mList->ReadMainList();
    MMap *m = 0;
    
    OperationProgress *op = new OperationProgress;
    connect(op, SIGNAL(updated(OperationProgress*)), this, SLOT(reloadingSources(OperationProgress*)));
    connect(op, SIGNAL(finished(OperationProgress*)), this, SLOT(reloadingSources(OperationProgress*)));
    bool res = pkgMakeStatusCache(*mList, *op, &m, true);
    op->deleteLater();
    
    Shaman::Repository::List repos;
    Shaman::AbstractPackage::List packages;
    Shaman::PackageGroup::List groups;
    
    if(!mCache)
        mCache = new pkgCache(m);
    if(!mPolicy)
        mPolicy = new pkgPolicy(mCache);
    int processedPackages = 0;
    
    QHash<QString,Shaman::PackageGroup*> sectionCollection;
    
    QString packageName, sectionName;
    Shaman::BackendPackage *package;
    Shaman::PackageGroup *section;
    pkgCache::PkgIterator it = mCache->PkgBegin();
    for(;it!=mCache->PkgEnd();++it)
    {
        packageName = it.Name();
        pkgCache::VerIterator vit = it.VersionList();
        pkgCache::VerIterator cndVer = mPolicy->GetCandidateVer(it);
        for(;!vit.end();++vit)
        {
            package = new DebPackage(this, packageName, vit, cndVer);
            sectionName = vit.Section();
            section = sectionCollection[sectionName];
            if(!section)
            {
                section = new Shaman::PackageGroup(sectionName, this);
                sectionCollection[sectionName] = section;
            }
            section->append(package);
            packages << package;
        }
        showOperationProgress(i18n("Reloading cache"), (++processedPackages)*100/mCache->HeaderP->PackageCount);
    }
    groups.append(sectionCollection.values());

    setPackages(packages);
    setRepositories(repos);
    setPackageGroups(groups);

    setState(Shaman::Backend::Idling);
}

void AptBackend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    kdDebug() << "markAdditionalTargetsForQueue. Options:" << options.size();
    pkgDepCache dep(mCache);
    OpTextProgress op;
    dep.Init(&op);
    {
        DebPackage *quePkg;
        foreach(Shaman::AbstractPackage *package, backend()->queuedPackages())
        {
            quePkg = static_cast<DebPackage*>(package);
            pkgCache::PkgIterator pkgIter = quePkg->version.ParentPkg();
            //pkgCache::Package *package
            switch(package->action())
            {
                case Shaman::AbstractPackage::Install:
                case Shaman::AbstractPackage::Upgrade:
                    if(mPolicy->GetCandidateVer(pkgIter)!=quePkg->version)
                        dep.SetCandidateVersion(quePkg->version);
                    dep.MarkInstall(pkgIter);
                    break;
                case Shaman::AbstractPackage::Uninstall:
                    dep.MarkDelete(pkgIter);
                    break;
                case Shaman::AbstractPackage::NoAction:
                    kdDebug() << "Warning: Queued package with no action";
                    break;
                case Shaman::AbstractPackage::TransactionDependencyInstall:
                case Shaman::AbstractPackage::TransactionDependencyUpgrade:
                case Shaman::AbstractPackage::TransactionDependencyUninstall:
                    break;
            }
        }
    }
    {
        DebPackage *addPkg;
        for(pkgCache::PkgIterator it = dep.PkgBegin(); !it.end(); ++it)
        {
            pkgDepCache::StateCache st = dep[it];
            //new installs and upgrades are installs
            if(!st.Install())
                continue;
            addPkg = findPackageByVersion(st.InstVerIter(*mCache));
            //package is queued (has an action)
            if(addPkg->action()!=Shaman::AbstractPackage::NoAction)
            {
                continue;
            }
            if(st.NewInstall()) //greater status in depcache.h
            {
                addPkg->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);
                kdDebug() << "oo install install" << it.Name();
            }
            else if(st.Upgrade())
            {
                addPkg->setAction(Shaman::AbstractPackage::TransactionDependencyUpgrade);
                kdDebug() << "oo Upgrade Upgrade" << it.Name();
            }
        }
    }
    kdDebug() << "base size:" << backend()->queuedPackages().size() << "dep size:" << dep.InstCount();
    setState(Shaman::Backend::Idling);
}

void AptBackend::processQueue(const Shaman::AbstractPackage::List& packages, const Shaman::ProcessQueueOption::List &options)
{
    kdDebug() << "Process queue." << packages.size() << "packages." << options.size() <<"options.";
    Action process(mHelperID+".process");
    process.setHelperID(mHelperID);
    process.setExecutesAsync(true);
    connect(process.watcher(), SIGNAL(actionPerformed(ActionReply)),
            this, SLOT(slotProcessFinished(ActionReply)));
    connect(process.watcher(), SIGNAL(progressStep(const QVariantMap&)),
            this, SLOT(slotProcessRequest(const QVariantMap&)));
    QVariantMap args;
    //probemos... es install upgrade?
    QVariantMap installList; //Map: package - version(nil if candidate)
    //QVariantMap upgradeList;
    QVariantList removeList;
    QVariantMap optionList;
    
    DebPackage *dPkg;
    foreach(Shaman::AbstractPackage *package, packages)
    {
        dPkg = static_cast<DebPackage*>(package);
        switch(package->action())
        {
            case Shaman::AbstractPackage::Install:
            case Shaman::AbstractPackage::Upgrade:
                if(dPkg->isCandidate())
                {
                    installList.insert(dPkg->name(),QVariant());
                }
                else
                {
                    installList.insert(dPkg->name(),dPkg->version.VerStr());
                }
                break;
            case Shaman::AbstractPackage::Uninstall:
                removeList.append(dPkg->name());
                break;
            //case Upgrade:
            //    break;
            //off warning
            case Shaman::AbstractPackage::TransactionDependencyInstall:
            case Shaman::AbstractPackage::TransactionDependencyUninstall:
            case Shaman::AbstractPackage::TransactionDependencyUpgrade:
            case Shaman::AbstractPackage::NoAction:
                break;
        }
    }
    
    foreach(const Shaman::ProcessQueueOption *option, options)
    {
        optionList.insert(option->data().toString(),true);
    }
    args.insert("install", installList);
    //args.insert("upgrade", upgradeList);
    args.insert("remove", removeList);
    args.insert("options", optionList);
    process.setArguments(args);
    ActionReply reply = process.execute();
    if(reply!=ActionReply::SuccessReply)
    {
        kdDebug() << "Failed to process queue";
        setState(Shaman::Backend::Idling);
        //do something
    }
}

void AptBackend::reloadingSources(OperationProgress *op)
{
    showOperationProgress(i18n("Reloading sources"), (int)op->percent());
}

void AptBackend::slotUpdateFinished()
{
    setState(Shaman::Backend::Idling);
}

void AptBackend::slotProcessRequest(const QVariantMap &args)
{
    QString command = args.value("command").toString();
    if(command=="operation")
    {
        addTransactionMessage(args.value("operation").toString());
    }
    else if(command=="mediachange")
    {
        //TODO: Request media change
        kdDebug() << "TODO: media change request";
        Action mediaChange(mHelperID+".helperdata.mediachangereply");
        mediaChange.setHelperID(mHelperID+".helperdata");
        QVariantMap args;
        args.insert("command", "mediachanged");
        args.insert("mediachanged", false); //no media change
        mediaChange.setArguments(args);
        ActionReply reply = mediaChange.execute();
        if(!reply.succeeded())
        {
            kdDebug() << "algo fallo ;)" << reply.errorCode() << reply.errorDescription();
            kdDebug() << "es auth error:" << (reply.type() == ActionReply::KAuthError)
                      << "auth den:" << (reply.errorCode() == ActionReply::AuthorizationDenied)
                      << "dbus:" << (reply.errorCode() == ActionReply::DBusError);
        }
    }
    else if(command=="debconfquestion")
    {
        //TODO: DebConf reply
        kdDebug() << "TODO: debconf reply";
        Action reply(mHelperID+".process");
        reply.setHelperID(mHelperID);
        QVariantMap args;
        args.insert("command", "debconfreply");
        args.insert("debconfreply", false); //no media change
        reply.setArguments(args);
        reply.execute();
    }
}

void AptBackend::slotProcessFinished(ActionReply reply)
{
    setState(Shaman::Backend::Idling);
}

void AptBackend::slotOperationReply(ActionReply reply)
{
    if(reply.succeeded())
    {
        setState(Shaman::Backend::Idling);
    }
    else
    {
        setTransactionError(Shaman::TransactionInterface::UnknownError, "ups");
    }
}

DebPackage *AptBackend::findPackageByVersion(const pkgCache::VerIterator &version)
{
    foreach(Shaman::AbstractPackage *pkg, backend()->packages())
    {
        DebPackage *package = static_cast<DebPackage*>(pkg);
        if(package->version == version)
        {
            return package;
        }
    }
    return 0;
}

Q_EXPORT_PLUGIN2(shaman_backend, AptBackend)
