/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "Helper.h"
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string>

#include <QtCore/QCoreApplication>
#include <QtCore/QMetaObject>
#include <QtCore/QDebug>

#include <QtDBus/QDBusConnection>

#include <kdebug.h>

#include <apt-pkg/sourcelist.h>
#include <apt-pkg/algorithms.h>
#include <apt-pkg/pkgcachegen.h>
#include <apt-pkg/error.h>
#include <apt-pkg/init.h>

#include "HelperAcquire.h"
#include "OperationProgress.h"
#include "HelperAdaptor.h"

AptHelper::AptHelper()
{
    initOk = pkgInitConfig(*_config) && pkgInitSystem(*_config,_system);
    qDebug() << "delay extra dbus interface init";
    qDebug() << "invoke:" << QMetaObject::invokeMethod(this, "init", Qt::QueuedConnection);
}

void AptHelper::init()
{
    system("touch /tmp/heo");
    new HelperAdaptor(this);
    qDebug() << "register service" << QDBusConnection::systemBus().registerService("org.kde.shaman.data");
    qDebug() << "register object" << QDBusConnection::systemBus().registerObject("/", this);
}

ActionReply AptHelper::initError() const
{
    ActionReply fail = ActionReply::HelperErrorReply;
    fail.setErrorCode(1);
    fail.setErrorDescription("Could not initialize Apt");
    return fail;
}

ActionReply AptHelper::update(QVariantMap args)
{
    kdDebug() << "Update indices";
    if(!initOk)
        return initError();
    
    HelperAcquire acquireStatus;
    connect(this, SIGNAL(stopUpdate()), &acquireStatus, SLOT(stop()));
    pkgSourceList list;
    list.ReadMainList();
    bool reply = ListUpdate(acquireStatus, list, 1000);
    if(reply)
    {
        kdDebug() << "Update sucessful";
        return ActionReply::SuccessReply;
    }
    kdDebug() << "Update error";
    return ActionReply::HelperErrorReply;
}

void AptHelper::stopupdate()
{
    kdDebug() << "Stop update";
    emit(stopUpdate());
}

ActionReply AptHelper::process(QVariantMap args)
{
    kdDebug() << "Process packages";
    if(!initOk)
        return initError();
    
    QVariantMap options = args.value("options").toMap();
    bool purge = options.value("purge").toBool();
    
    pkgSourceList mList;
    mList.ReadMainList();

    MMap *m = 0;
    OperationProgress progress;
    connect(&progress, SIGNAL(updated(OperationProgress*)),
            this, SLOT(operationReport(OperationProgress*)));
    bool res = pkgMakeStatusCache(mList, progress, &m, true);
    
    pkgCache mCache(m);
    //pkgPolicy mPolicy(mCache);
    pkgDepCache dep(&mCache);
    dep.Init(&progress);

    QVariantMap installList = args.value("install").toMap();
    for(QVariantMap::const_iterator in = installList.constBegin(); in != installList.constEnd(); ++in)
    {
        std::string packageName = in.key().toStdString();
        kdDebug() << "Install" << in.key() << QString::fromStdString(packageName);
        pkgCache::PkgIterator pkg = mCache.FindPkg(packageName);
        kdDebug() << "pkg:" << pkg.Name();
        dep.MarkInstall(pkg);
        if(!in.value().isNull())
        {
            std::string packageVersion = in.value().toString().toStdString();
            for(pkgCache::VerIterator ver = pkg.VersionList(); !ver.end(); ++ver)
            {
                if(in.key().toLatin1()==ver.VerStr())
                {
                    dep.SetCandidateVersion(ver);
                }
            }
        }
    }
    QVariantList removeList = args.value("remove").toList();
    for(QVariantList::const_iterator in = removeList.constBegin(); in != removeList.constEnd(); ++in)
    {
        std::string packageName = (*in).toString().toStdString();
        kdDebug() << "Delete" << (*in).toString();
        pkgCache::PkgIterator pkg = mCache.FindPkg(packageName);
        dep.MarkDelete(pkg, purge);
    }
    dep.MarkAndSweep();
    kdDebug() << "marked and swept";
    HelperAcquire acProgress;
    connect(&acProgress, SIGNAL(mediaChangeRequest(const QString &, const QString &)),
            this, SIGNAL(mediaChangeRequest(const QString &, const QString &)));
    connect(this, SIGNAL(mediaChangeReply(bool)), &acProgress, SLOT(mediaChangeReply(bool)));
    pkgRecords recs(dep);
    pkgAcquire fetcher(&acProgress);
    pkgPackageManager *pm = _system->CreatePM(&dep);
    if(pm->GetArchives(&fetcher, &mList, &recs) == false ||
        _error->PendingError() == true)
    {
        kdDebug() << "Error in GetArchives";
        return ActionReply(ActionReply::HelperError);
    }
    kdDebug() << "Run fetcher";
    if(fetcher.Run() == pkgAcquire::Failed)
    {
        return ActionReply(ActionReply::HelperError);
    }
    kdDebug() << "Do install -1";
    kdDebug() << "PATH:" << getenv("PATH");
    //Set environment for DPKG. Fixed or...?
    setenv("PATH", "/bin:/sbin:/usr/sbin/:/usr/bin",1);
    pkgPackageManager::OrderResult result = pm->DoInstall();
    if(result!=pkgPackageManager::Completed)
    {
        kdDebug() << "Install did not complete";
        while(_error->PendingError())
        {
            std::string error;
            _error->PopMessage(error);
            kdDebug() << "Error message:" << QString::fromStdString(error);
        }
        return ActionReply::HelperErrorReply;
    }
    return ActionReply::SuccessReply;
}

void AptHelper::stopprocess()
{
    kdDebug() << "Stop progress";
    emit stopProcess();
}

ActionReply AptHelper::distupgrade(QVariantMap /*args*/)
{
    return ActionReply::HelperErrorReply;
}

void AptHelper::mediachanged(bool reply)
{
    emit mediaChangeReply(reply);
    kdDebug() << "Should change media" << reply;
}

void AptHelper::debconfreply(QVariantMap /*args*/)
{
}

// Handle as signal at the DBus adaptor
//void AptHelper::mediaChangeRequest(const QString &media, const QString &drive)
//{
//    kdDebug() << "Media change request," << media << drive;
//    QVariantMap request;
//    request.insert("command", "mediachange");
//    request.insert("media", media);
//    request.insert("drive", drive);
//    HelperSupport::progressStep(request);
//}

void AptHelper::operationReport(OperationProgress *op)
{
    QVariantMap request;
    request.insert("command", "operation");
    request.insert("operation", op->operation());
    HelperSupport::progressStep(request);
}

KDE4_AUTH_HELPER_MAIN("org.kde.shaman.apt", AptHelper)

