/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AqpmPlugin.h"

#include "shamantransactioninterface.h"

#include <shamanmacros.h>

#include <KDebug>
#include <KUrl>
#include <KAboutData>
#include <klocalizedstring.h>

#include "AqpmPackage.h"
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <QTimer>
#include <KIcon>
#include "AqpmIcons.h"
#include "KdemodDistUpgrade.h"
#include "AqpmLocalPackage.h"
#include <shamanmetapackage.h>

using namespace Aqpm;

AqpmPlugin::AqpmPlugin()
  : Shaman::BackendInterface(0)
  , m_cachedState(Shaman::TransactionInterface::NoTransaction)
{

}

AqpmPlugin::~AqpmPlugin()
{
}

void AqpmPlugin::init()
{
    setCapabilities(/*Transaction capabilities*/
                    Shaman::BackendCapabilities::InstallPackagesCapability | 
                    Shaman::BackendCapabilities::RemovePackagesCapability |
                    Shaman::BackendCapabilities::UpgradePackagesCapability | 
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability |
                    Shaman::BackendCapabilities::InstallPackageFromFileCapability | 
                    Shaman::BackendCapabilities::UpgradeSystemCapability,
                    /*Search capabilities*/
                    Shaman::BackendCapabilities::BasicSearchFromViewCapability | 
                    Shaman::BackendCapabilities::ExtendedSearchFromViewCapability,
                    /*Package Grouping capabilities*/
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability |
                    Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    /*Information capabilities*/
                    Shaman::BackendCapabilities::ShowRequiredByCapability |
                    Shaman::BackendCapabilities::ShowDependsOnCapability |
                    Shaman::BackendCapabilities::ShowFileListCapability |
                    Shaman::BackendCapabilities::ShowProvidesCapability |
                    Shaman::BackendCapabilities::ShowConflictsWithCapability |
                    Shaman::BackendCapabilities::ShowPackageChangelog);

    // Now to the process queue options
    Shaman::ProcessQueueOption::List options;
    options.append(new Shaman::ProcessQueueOption(i18n("Force"),
                                              i18n("Processes the queue regardless of errors and file conflicts. "
                                                                  "Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::Force)));
    options.append(new Shaman::ProcessQueueOption(i18n("Ignore dependencies"), i18n("Skips all dependency checks. "
                                                                                "Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::NoDeps)));
    options.append(new Shaman::ProcessQueueOption(i18n("Ignore backups"),
                                              i18n("Ignores file backup designations when removing packages, which means "
                                                   ".pacsave files will not be generated."),
                                              QVariant::fromValue((uint)Aqpm::Globals::NoSave)));
    options.append(new Shaman::ProcessQueueOption(i18n("Cascade"), i18n("When removing packages, removes also all packages that "
                                                                    "depend on one or more target packages. "
                                                                    "This operation is recursive, and must be used with care "
                                                                    "since it can remove many potentially needed packages."),
                                              QVariant::fromValue((uint)Aqpm::Globals::Cascade)));
    options.append(new Shaman::ProcessQueueOption(i18n("Recursive removal"),
                                              i18n("When removing packages, removes also all of their dependencies, provided "
                                                   "that they are not required by other packages and they were not "
                                                   "explicitly installed by the user. "
                                                   "This option helps keeping a clean system without orphans."),
                                              QVariant::fromValue((uint)Aqpm::Globals::Recurse)));
    options.append(new Shaman::ProcessQueueOption(i18n("Complete recursive removal"),
                                              i18n("When removing packages, removes also all of their dependencies, provided "
                                                   "that they are not required by other packages."),
                                              QVariant::fromValue((uint)Aqpm::Globals::RecurseAll)));
    options.append(new Shaman::ProcessQueueOption(i18n("Remove database entry only"),
                                              i18n("When removing packages, removes only the entry from the database, but "
                                                   "not the files. Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::DbOnly)));
    options.append(new Shaman::ProcessQueueOption(i18n("Ignore package scriptlets"),
                                              i18n("Ignores packages' scripts. Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::NoScriptlet)));
    options.append(new Shaman::ProcessQueueOption(i18n("Download packages only"),
                                              i18n("When installing or upgrading packages, downloads the package only "
                                                   "without installing or upgrading it. "
                                                   "Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::DownloadOnly)));
    options.append(new Shaman::ProcessQueueOption(i18n("Install all packages as explicitly installed"),
                                              i18n("When installing or upgrading packages, installs every package marking "
                                                   "it as explicitly installed, even if it is a dependency. "
                                                   "Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::AllExplicit)));
    options.append(new Shaman::ProcessQueueOption(i18n("Install all packages as dependencies"),
                                              i18n("When installing or upgrading packages, installs every package marking "
                                                   "it as a dependency, even if it was explicitly selected. "
                                                   "Enable only if you know what you are doing."),
                                              QVariant::fromValue((uint)Aqpm::Globals::AllDeps)));

    setAvailableProcessQueueOptions(options);

    KAboutPerson dario(ki18n("Dario Freddi"), ki18n("Maintainer"), "drf@kde.org", "http://drfav.wordpress.com");
    // TODO: Change package icon
    setProperties(i18n("Aqpm (Arch Linux/Chakra)"), i18n("a Qt wrapper around libalpm/pacman, version %1",
                                                         Backend::version()), KIcon("chakra-shiny"),
                  KIcon("application-x-deb"), QList<KAboutPerson>() << dario,
                  QStringList() << "kcm_aqpm_general" << "kcm_aqpm_databases");

    Backend::instance()->safeInitialization();
    initAlpm();
}

void AqpmPlugin::initAlpm()
{
    // Connect all backend slots to our handlers.

    connect(Backend::instance(), SIGNAL(dbStatusChanged(const QString&,int)),
            this, SLOT(slotDbStatusChanged(const QString&,int)));
    connect(Backend::instance(), SIGNAL(streamTotalDownloadSize(qint64)),
            manualProgressReporter(), SLOT(setTotalDownloadSize(qint64)));
    connect(Backend::instance(), SIGNAL(streamDlProg(QString,int,int,int,int,int)),
            this, SLOT(slotStreamDlProg(QString,int,int,int,int,int)));
    connect(Backend::instance(), SIGNAL(streamTransProgress(Aqpm::Globals::TransactionProgress,QString,int,int,int)),
            this, SLOT(slotStreamTransProgress(Aqpm::Globals::TransactionProgress,QString,int,int,int)));
    connect(Backend::instance(), SIGNAL(streamTransEvent(Aqpm::Globals::TransactionEvent,QVariantMap)),
            this, SLOT(slotStreamTransEvent(Aqpm::Globals::TransactionEvent,QVariantMap)));
    connect(Backend::instance(), SIGNAL(streamTransQuestion(Aqpm::Globals::TransactionQuestion,QVariantMap)),
            this, SLOT(slotStreamTransQuestion(Aqpm::Globals::TransactionQuestion,QVariantMap)));
    connect(Backend::instance(), SIGNAL(errorOccurred(Aqpm::Globals::Errors,QVariantMap)),
            this, SLOT(slotErrorOccurred(Aqpm::Globals::Errors,QVariantMap)));
    connect(Backend::instance(), SIGNAL(operationFinished(bool)),
            this, SLOT(slotOperationFinished(bool)));
    connect(Backend::instance(), SIGNAL(logMessageStreamed(QString)),
            this, SLOT(slotLogMessageStreamed(QString)));
    connect(Backend::instance(), SIGNAL(additionalTargetsRetrieved(Aqpm::Targets)),
            this, SLOT(slotAdditionalTargetsRetrieved(Aqpm::Targets)));

    // All systems are up and running, let's go!
    setState(Shaman::Backend::Idling);
}

bool AqpmPlugin::aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate)
{
    if (((oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||
         (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) &&
        backend()->operationMode() == Shaman::Base::CachedMode) {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
        // If we're initializing, just don't reject the state transition
        if (oldstate != Shaman::Backend::Initializing) {
            // Otherwise do.
            return false;
        }
    }

    // Else we're fine with defaults
    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

void AqpmPlugin::reload()
{
    setState(Shaman::Backend::Reloading);
    QHash< QString, Shaman::AbstractPackage*> pkglist;
    Shaman::PackageGroup::List grplist;
    Shaman::Repository::List repolist;

    Package::List upgradeable = Backend::instance()->upgradeablePackages();
    Database::List repos = Backend::instance()->availableDatabases();
    repos.append(Backend::instance()->localDatabase());

    foreach (Database *db, repos) {
        Shaman::Repository *r = 0;
        QString name;
        if (db->name() == "local") {
            name = i18n("Local packages");
        } else {
            name = db->name();
        }
        if (AqpmIcons::iconForDatabase(name).isNull()) {
            r = new Shaman::Repository(name, KUrl::fromPathOrUrl(db->path()), this);
        } else {
            r = new Shaman::Repository(name, AqpmIcons::iconForDatabase(name), KUrl::fromPathOrUrl(db->path()), this);
        }
        Package::List packages = db->packages();

        foreach (Package *package, packages) {
            AqpmPackage *p = createShamanPackageFrom(package, upgradeable);

            r->append(p);
            pkglist[p->name()] = p;
        }

        repolist.append(r);
    }

    // Now groups
    foreach (Group *group, Backend::instance()->availableGroups()) {
        Shaman::PackageGroup *g = 0;
        if (AqpmIcons::iconForGroup(group->name()).isNull()) {
            g = new Shaman::PackageGroup(group->name(), this);
        } else {
            g = new Shaman::PackageGroup(group->name(), AqpmIcons::iconForGroup(group->name()), this);
        }

        grplist.append(g);

        foreach (Package *pkg, group->packages()) {
            if (pkglist.contains(pkg->name())) {
                g->append(pkglist[pkg->name()]);
            }
        }
    }

    setPackages(pkglist.values());
    setRepositories(repolist);
    setPackageGroups(grplist);


    setState(Shaman::Backend::Idling);
}

AqpmPackage* AqpmPlugin::createShamanPackageFrom(Package* package, const Package::List &upgradeable)
{
    AqpmPackage *p = 0;

    if (package->isInstalled()) {
        foreach (Package *uppkg, upgradeable) {
            if (uppkg->name() == package->name()) {
                p = new AqpmPackage(uppkg, Shaman::AbstractPackage::Upgradeable, this);
                break;
            }
        }

        if (!p) {
            p = new AqpmPackage(Backend::instance()->package(package->name(), "local"),
                                Shaman::AbstractPackage::Installed, this);
        }
    } else {
        p = new AqpmPackage(package, Shaman::AbstractPackage::NotInstalled, this);
    }

    // Any providers around?
    foreach (const QString &provider, package->providers()) {
        QString rp;
        if (provider.contains('=')) {
            rp = provider.split('=').first();
        } else {
            rp = provider;
        }

        metaPackage(rp)->addPackage(p);
        // If it's installed it's also the root item
        if (p->state() == Shaman::AbstractPackage::Installed || p->state() == Shaman::AbstractPackage::Upgradeable) {
            metaPackage(rp)->setRootPackage(p);
        }
    }

    return p;
}

void AqpmPlugin::updateRepositories()
{
    kDebug() << "Update the database";
    m_dbupdateWeight = 100 / backend()->repositories().size() + backend()->repositories().size() / 2 + 1;
    m_dbupdateProgress = 0;
    //setTransactionProgress(m_dbupdateProgress);
    // We're starting the transaction
    Backend::instance()->updateDatabase();
    setTransactionState(Shaman::TransactionInterface::Processing);
}

bool AqpmPlugin::cancelRepositoriesUpdate()
{
    Backend::instance()->interruptTransaction();
    return true;
}

void AqpmPlugin::processQueue(const Shaman::AbstractPackage::List &packages, const Shaman::ProcessQueueOption::List &options)
{
    Backend::instance()->clearQueue();
    // Clear cache as well
    m_packageNameCache.clear();

    foreach (Shaman::AbstractPackage *package, packages) {
        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
            case Shaman::AbstractPackage::Upgrade:
                Backend::instance()->addItemToQueue(package->name(), QueueItem::Sync);
                break;
            case Shaman::AbstractPackage::Uninstall:
                Backend::instance()->addItemToQueue(package->name(), QueueItem::Remove);
                break;
            default:
                break;
        }
    }

    Globals::TransactionFlags flags = 0;

    foreach (Shaman::ProcessQueueOption * const option, options) {
        flags |= (Globals::TransactionFlag)(option->data().toUInt());
    }

    Backend::instance()->processQueue(flags);
}

bool AqpmPlugin::cancelQueueProcessing()
{
    Backend::instance()->interruptTransaction();
    return true;
}

void AqpmPlugin::slotDbStatusChanged(const QString &repo, int action)
{
    // First, let's find our repo

    Shaman::Repository *backendrepo = 0;

    foreach (Shaman::Repository *r, backend()->repositories()) {
        if (r->name() == repo) {
            backendrepo = r;
            break;
        }
    }

    if (!backendrepo) {
        kDebug() << "Failed finding repo";
        return;
    }

    kDebug() << "Setting repo action";

    switch (action) {
        case Globals::Checking:
        case Globals::Updating:
            backendrepo->setState(Shaman::Repository::Updating);
            break;
        case Globals::Downloading:
            backendrepo->setState(Shaman::Repository::Downloading);
            break;
        case Globals::Unchanged:
        case Globals::Updated:
        case Globals::DatabaseError:
            backendrepo->setState(Shaman::Repository::UpdatingFinished);
            m_dbupdateProgress += m_dbupdateWeight;
            if (m_dbupdateProgress >= 100) {
                //setTransactionProgress(100);
            } else {
                //setTransactionProgress(m_dbupdateProgress);
            }
            break;
        default:
            break;
    }
}

void AqpmPlugin::slotStreamDlProg(const QString& c, int bytedone, int bytetotal, int speed, int listdone, int listtotal)
{
    Q_UNUSED(speed)
    Q_UNUSED(listdone)
    Q_UNUSED(listtotal)

    if (m_runningDistUpgrade.isNull()) {
        populateCacheForPackageName(c);
        setPackageDownloadProgress(m_packageNameCache[c], bytedone, bytetotal);
    } else {
        m_runningDistUpgrade.data()->packageProgress(c, bytedone, bytetotal);
    }
}

void AqpmPlugin::slotStreamTransProgress(Globals::TransactionProgress event, const QString& pkgname, int percent, int howmany, int remain)
{
    Q_UNUSED(event)
    Q_UNUSED(howmany)
    Q_UNUSED(remain)
    kDebug() << pkgname;

    if (!pkgname.isEmpty()) {
        if (m_runningDistUpgrade.isNull()) {
            populateCacheForPackageName(pkgname);
            setPackageTransactionProgress(m_packageNameCache[pkgname], percent);
        } else {
            m_runningDistUpgrade.data()->packageProgress(pkgname, percent, howmany, remain);
        }
    }
}

void AqpmPlugin::slotStreamTransEvent(Globals::TransactionEvent event, const QVariantMap& args)
{
    QString message;
    switch ( event ) {
        case Aqpm::Globals::CheckDepsStart:
            message = i18n("Validating Dependencies...");
            break;
        case Aqpm::Globals::FileConflictsStart:
            message = i18n("Checking for Conflicts...");
            break;
        case Aqpm::Globals::ResolveDepsStart:
            message = i18n("Resolving Dependencies...");
            break;
        case Aqpm::Globals::InterConflictsStart:
            message = i18n("Looking for Inter-Conflicts...");
            break;
        case Aqpm::Globals::AddStart:
            if (m_cachedState != Shaman::TransactionInterface::Processing) {
                setTransactionState(Shaman::TransactionInterface::Processing);
                m_cachedState = Shaman::TransactionInterface::Processing;
            }
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("Installing %1...", args["PackageName"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::AddDone:
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("%1 (%2) installed successfully.",
                                                                  args["PackageName"].toString(),
                                                                  args["PackageVersion"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::RemoveStart:
            if (m_cachedState != Shaman::TransactionInterface::Processing) {
                setTransactionState(Shaman::TransactionInterface::Processing);
                m_cachedState = Shaman::TransactionInterface::Processing;
            }
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("Removing %1...", args["PackageName"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::RemoveDone:
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("Removing %1...", args["PackageName"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::UpgradeStart:
            if (m_cachedState != Shaman::TransactionInterface::Processing) {
                setTransactionState(Shaman::TransactionInterface::Processing);
                m_cachedState = Shaman::TransactionInterface::Processing;
            }
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("Upgrading %1...", args["PackageName"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::UpgradeDone:
            if (!m_runningDistUpgrade.isNull()) {
                emit m_runningDistUpgrade.data()->newMessage(i18n("Upgraded %1 successfully (%2 -> %3)",
                                                                  args["PackageName"].toString(), args["OldVersion"].toString(),
                                                                  args["NewVersion"].toString()),
                                                             Shaman::TransactionInterface::TransactionStatusLevel);
            }
            break;
        case Aqpm::Globals::IntegrityStart:
            if (m_cachedState != Shaman::TransactionInterface::Processing) {
                setTransactionState(Shaman::TransactionInterface::Processing);
                m_cachedState = Shaman::TransactionInterface::Processing;
            }
            message = i18n("Checking package integrity...");
            break;
        case Aqpm::Globals::DeltaIntegrityStart:
            if (m_cachedState != Shaman::TransactionInterface::Processing) {
                setTransactionState(Shaman::TransactionInterface::Processing);
                m_cachedState = Shaman::TransactionInterface::Processing;
            }
            message = i18n("Checking delta integrity...");
            break;
        case Aqpm::Globals::DeltaPatchesStart:
            message = i18n("Applying deltas...");
            break;
        case Aqpm::Globals::DeltaPatchStart:
            message = i18n("Generating %1 with %2...", args["From"].toString(), args["To"].toString());
            break;
        case Aqpm::Globals::DeltaPatchDone:
            message = i18n("Success.");
            break;
        case Aqpm::Globals::DeltaPatchFailed:
            message = i18n("Failed.");
            break;
        case Aqpm::Globals::ScriptletInfo:
            message = QString("%1").arg(args["Text"].toString().remove('\n'));
            break;
        case Aqpm::Globals::RetrieveStart:
            if (m_cachedState != Shaman::TransactionInterface::Downloading) {
                setTransactionState(Shaman::TransactionInterface::Downloading);
                m_cachedState = Shaman::TransactionInterface::Downloading;
            }
            message = i18n("Starting downloading packages from %1...", args["Repo"].toString());
            break;
            /* all the simple done events, with fallthrough for each */
        default:
            break;
    }

    if (!message.isEmpty()) {
        if (m_runningDistUpgrade.isNull()) {
            addTransactionMessage(message);
        } else {
            emit m_runningDistUpgrade.data()->newMessage(message, Shaman::TransactionInterface::TransactionStatusLevel);
        }
    }
}


void AqpmPlugin::slotErrorOccurred(Globals::Errors code, const QVariantMap& args)
{
    kDebug() << args;
    QString message;
    Shaman::TransactionInterface::Error error = Shaman::TransactionInterface::UnknownError;
    if (code == Aqpm::Globals::UnsatisfiedDependencies) {
        message = i18n("Some dependencies could not be satisfied");
        message.append("\n\n");

        QVariantMap::const_iterator i;
        QVariantMap map = args["UnsatisfiedDeps"].toMap();
        for (i = map.constBegin(); i != map.constEnd(); ++i) {
            message.append(i18n("%1: requires %2", i.key(), i.value().toString()));
            message.append('\n');
        }

        error = Shaman::TransactionInterface::DependenciesUnsatisfiedError;
    } else if (code == Aqpm::Globals::ConflictingDependencies) {
        message = i18n("Some dependencies create a conflict with already installed packages");
        message.append("\n\n");

        QMap<QString, QVariant>::const_iterator i;
        for (i = args["ConflictingDeps"].toMap().constBegin(); i != args["ConflictingDeps"].toMap().constEnd(); ++i) {
            message.append(i18n("%1: conflicts with %2", i.key(), i.value().toString()));
            message.append('\n');
        }

        error = Shaman::TransactionInterface::DependencyConflictError;
    } else if (code == Aqpm::Globals::PrepareError || code == Aqpm::Globals::CommitError) {
        message = i18n("No further details were given. Last error string was: %1", args["ErrorString"].toString());

        error = Shaman::TransactionInterface::ProcessingFailedError;
    }
    else if (code == Aqpm::Globals::FileConflictTarget) {
        message = i18n("Some files in the packages being processed are conflicting");
        message.append("\n\n");

        QMap<QString, QVariant>::const_iterator i;
        for (i = args["ConflictingTargets"].toMap().constBegin(); i != args["ConflictingTargets"].toMap().constEnd(); ++i) {
            message.append(i18n("%1 exists in both '%2' and '%3'", i.key(),
                                i.value().toStringList().first(), i.value().toStringList().last()));
            message.append('\n');
        }

        error = Shaman::TransactionInterface::FileConflictError;
    } else if (code == Aqpm::Globals::FileConflictFilesystem) {
        message = i18n("Some files in the packages being processed conflict with the local filesystem");
        message.append("\n\n");

        QMap<QString, QVariant>::const_iterator i;
        for (i = args["ConflictingFiles"].toMap().constBegin(); i != args["ConflictingFiles"].toMap().constEnd(); ++i) {
            message.append(i18n("%1: %2 exists in the filesystem", i.key(), i.value().toString()));
            message.append('\n');
        }

        error = Shaman::TransactionInterface::FileConflictError;
    } else if (code == Aqpm::Globals::CorruptedFile) {
        message = i18n("Some downloaded packages are corrupted or invalid");
        message.append("\n\n");

        foreach(const QString &ent, args["Filenames"].toStringList()) {
            message.append(tr("%1 is invalid or corrupted").arg(ent));
            message.append('\n');
        }

        error = Shaman::TransactionInterface::FileConflictError;
    } else if (code == Aqpm::Globals::AddTargetError || code == Aqpm::Globals::DuplicateTarget ||
               code == Aqpm::Globals::PackageIgnored || code == Aqpm::Globals::PackageNotFound) {
        message = i18n("This is probably an internal Aqpm error. Please report it to help "
                               "the developers solving it");

        error = Shaman::TransactionInterface::BackendError;
    } else if (code == Aqpm::Globals::AuthorizationNotGranted) {
        message = i18n("It looks like the system policy prevents you from doing this action, or "
                               "you failed or refused to authenticate. Please contact your system administrator "
                               "for further details");

        error = Shaman::TransactionInterface::ProcessingFailedError;
    } else if (code == Aqpm::Globals::WorkerInitializationFailed) {
        message = i18n("Aqpm, the library underneath Shaman, reported that it could not start a "
                               "privileged operation. This is probably due to a bug in Aqpm or to a "
                               "misconfiguration of your system");

        error = Shaman::TransactionInterface::BackendError;
    } else if (code == Aqpm::Globals::WorkerDisappeared) {
        message = i18n("It looks like Aqpm's worker crashed or disappeared. "
                               "This is probably an internal Aqpm error. Please report it to help "
                               "the developers solving it");

        error = Shaman::TransactionInterface::BackendError;
    } else if (code == Aqpm::Globals::TransactionInterruptedByUser) {
        message = i18n("The transaction was interrupted");

        error = Shaman::TransactionInterface::ProcessingFailedError;
    } else {
        message = i18n("An unknown error was encountered");
        error = Shaman::TransactionInterface::UnknownError;
    }

    if (m_runningDistUpgrade.isNull()) {
        setTransactionError(error, message);
    } else {
        emit m_runningDistUpgrade.data()->errorTriggered(error, message);
        m_runningDistUpgrade.data()->setState(Shaman::DistUpgrade::NotRunning);
    }
}

void AqpmPlugin::slotOperationFinished(bool success)
{
    // We're handling errors in slotErrorOccurred
    Q_UNUSED(success)

    setIdle();
}

void AqpmPlugin::setIdle()
{
    // Back to idle
    setState(Shaman::Backend::Idling);
}

void AqpmPlugin::slotLogMessageStreamed(const QString& msg)
{
    if (m_runningDistUpgrade.isNull()) {
        addTransactionMessage(msg);
    } else {
        emit m_runningDistUpgrade.data()->newMessage(msg, Shaman::TransactionInterface::BackendMessageLevel);
    }
}

void AqpmPlugin::slotStreamTransQuestion(Globals::TransactionQuestion event, const QVariantMap& args)
{
    QString message;

    switch (event) {
    case Aqpm::Globals::IgnorePackage:
        if (args.contains("SecondPackage")) {
            message = i18n("%1 requires installing %2 from IgnorePkg/IgnoreGroup. Install anyway?", 
                           args["FirstPackage"].toString(), args["SecondPackage"].toString());
        } else {
            message = i18n("%1 is in IgnorePkg/IgnoreGroup. Install anyway?", args["FirstPackage"].toString());
        }
        break;
    case Aqpm::Globals::ReplacePackage:
        message = i18n("Replace %1 with %2/%3?", args["OldPackage"].toString(), 
                       args["NewPackageRepo"].toString(), args["NewPackage"].toString());
        break;
    case Aqpm::Globals::PackageConflict:
        message = i18n("%1 conflicts with %2. Remove %1?", args["OldPackage"].toString(), args["NewPackage"].toString());
        break;
    case Aqpm::Globals::UnresolvedDependencies:
        message = i18np("%2 cannot be upgraded due to unresolvable dependencies. Do you want to skip it for this upgrade?",
                        "%1 packages (%2) cannot be upgraded due to unresolvable dependencies. "
                        "Do you want to skip them for this upgrade?",
                        args["Packages"].toStringList().count(), args["Packages"].toStringList().join(", "));
        break;
    case Aqpm::Globals::NewerLocalPackage:
        message = i18n("%1-%2: local version is newer. Upgrade anyway?", args["PackageName"].toString(),
                       args["PackageVersion"].toString());
        break;
    case Aqpm::Globals::CorruptedPackage:
        message = i18n("File %1 is corrupted. Do you want to delete it?", args["Filename"].toString());
        break;
    default:
        break;
    }

    KIcon icon = KIcon("dialog-information");
    Shaman::TransactionQuestion *question = new Shaman::StringTransactionQuestion(message,
                                                                                  Shaman::TransactionQuestion::AnswerYes |
                                                                                  Shaman::TransactionQuestion::AnswerNo,
                                                                                  icon);

    if (m_runningDistUpgrade.isNull()) {
        newTransactionQuestion(question);
    } else {
        m_runningDistUpgrade.data()->newQuestion(question);
    }
}

void AqpmPlugin::populateCacheForPackageName(const QString& name)
{
    if (!m_packageNameCache.contains(name)) {
        // Ok, we need to trim the shit out of the package, let's try
        Shaman::AbstractPackage::List packages;

        foreach (Shaman::AbstractPackage *package, Shaman::Base::instance()->core()->transactionInterface()->packages()) {
            if (name.contains(package->name())) {
                packages.append(package);
            }
        }

        if (packages.size() == 1) {
            // Cool, smooth one
            m_packageNameCache[name] = packages.first();
        } else {
            // Oh shit. Find the one that contains all the others then
            foreach (Shaman::AbstractPackage *package, packages) {
                bool contained = true;
                foreach (Shaman::AbstractPackage *other, packages) {
                    if (!package->name().contains(other->name())) {
                        contained = false;
                        break;
                    }
                }
                if (contained) {
                    m_packageNameCache[name] = package;
                    break;
                }
            }
        }
    }
}

void AqpmPlugin::transactionQuestionAnswered(Shaman::TransactionQuestion *question)
{
    if (question->answer() == Shaman::TransactionQuestion::AnswerYes) {
        Backend::instance()->setAnswer(1);
    } else {
        Backend::instance()->setAnswer(0);
    }

    delete question;
}

Shaman::AbstractPackage *AqpmPlugin::findPackageByName(const QString &name)
{
    Shaman::AbstractPackage *ret = 0;

    foreach (Shaman::AbstractPackage *ent, backend()->packages()) {
        if (ent->name() == name) {
            ret = ent;
            break;
        }
    }

    if (!ret && backend()->operationMode() == Shaman::Base::UncachedMode) {
        // We probably need to add it to the cache. Find it out first.
        Aqpm::Package *package = Aqpm::Backend::instance()->package(name, QString());
        if (package) {
            Shaman::AbstractPackage *p = createShamanPackageFrom(package, Aqpm::Backend::instance()->upgradeablePackages());
            ret = p;
            packagesRetrieved(Shaman::AbstractPackage::List() << p);
        }
    }

    return ret;
}

void AqpmPlugin::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    Backend::instance()->clearQueue();

    foreach (Shaman::AbstractPackage *package, backend()->queuedPackages()) {
        switch (package->type()) {
            case Shaman::AbstractPackage::LocalPackageType:
                Backend::instance()->addItemToQueue(dynamic_cast< Shaman::LocalPackage* >(package)->path().toLocalFile(),
                                                    QueueItem::FromFile);
                break;
            case Shaman::AbstractPackage::BackendPackageType:
                switch (package->action()) {
                    case Shaman::AbstractPackage::Install:
                    case Shaman::AbstractPackage::Upgrade:
                        Backend::instance()->addItemToQueue(package->name(), QueueItem::Sync);
                        break;
                    case Shaman::AbstractPackage::Uninstall:
                        Backend::instance()->addItemToQueue(package->name(), QueueItem::Remove);
                        break;
                    default:
                        break;
                }
                break;
            default:
                kWarning() << "I got a package which is not from local fs or from backend, there's something screwed";
                break;
        }
    }

    Globals::TransactionFlags flags = 0;

    foreach (Shaman::ProcessQueueOption * const option, options) {
        flags |= (Globals::TransactionFlag)(option->data().toUInt());
    }

    Backend::instance()->retrieveAdditionalTargetsForQueue(flags);
}

void AqpmPlugin::slotAdditionalTargetsRetrieved(const Aqpm::Targets& targets)
{
    Aqpm::Targets::const_iterator i;
    for (i = targets.constBegin(); i != targets.constEnd(); ++i) {
        Shaman::AbstractPackage *p = findPackageByName(i.key());
        if (p) {
            if (i.value() == QueueItem::Remove) {
                p->setAction(Shaman::AbstractPackage::TransactionDependencyUninstall);
            } else {
                if (p->state() == Shaman::AbstractPackage::Installed) {
                    p->setAction(Shaman::AbstractPackage::TransactionDependencyUpgrade);
                } else {
                    p->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);
                }
            }
        }
    }

    setState(Shaman::Backend::Idling);
}

Shaman::AbstractPackage* AqpmPlugin::packageFromFile(const KUrl& path)
{
    Aqpm::Package *p = Backend::instance()->loadPackageFromLocalFile(path.toLocalFile());

    if (p) {
        kDebug() << "Loaded a valid package from file";
        return new AqpmLocalPackage(path, p, this);
    } else {
        kDebug() << "Failed to load the package, it is probably invalid";
        return 0;
    }
}

Q_EXPORT_PLUGIN2(shaman_backend, AqpmPlugin)

#include "AqpmPlugin.moc"
