/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "KdemodDistUpgrade.h"
#include <shamanwidgetfactory.h>
#include <KListWidget>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanbackend.h>
#include <aqpm/Backend.h>
#include <aqpm/Maintenance.h>
#include <QLabel>
#include <QVBoxLayout>
#include <KIcon>

class KdemodSetupFactory : public Shaman::WidgetFactory
{
    public:
        KdemodSetupFactory(Shaman::AbstractPackage::List packages) : m_packages(packages) {}
        virtual ~KdemodSetupFactory() {}
        virtual QWidget* createInstance() const {
            KListWidget *lw = new KListWidget;
            lw->setObjectName("__KDEMOD_DIST_UPGRADE_WIDGET___");
            foreach (Shaman::AbstractPackage * const package, m_packages) {
                if (package->state() == Shaman::AbstractPackage::NotInstalled) {
                    // Just append it as unchecked
                    QListWidgetItem *item = new QListWidgetItem(package->name());
                    item->setCheckState(Qt::Unchecked);
                    lw->addItem(item);
                } else {
                    // If it's no longer in the repositories, append it as unchecked and disable it
                    Aqpm::Package *p = Aqpm::Backend::instance()->package(package->name(), QString());
                    if (p) {
                        if (p->database()->name() == "local") {
                            // Ok, it's no longer here. Pass by
                        } else {
                            // Append it as checked
                            QListWidgetItem *item = new QListWidgetItem(package->name());
                            item->setCheckState(Qt::Checked);
                            lw->addItem(item);
                        }
                    }
                }
            }

            QLabel *label = new QLabel(i18n("Before starting the upgrade, you can choose which packages will be "
                                            "kept. Shaman already found the best solution for your system, "
                                            "but you can review the selection and make some additional changes. Note that you "
                                            "are also able to add new packages during the upgrade."));

            QVBoxLayout *lay = new QVBoxLayout;
            lay->addWidget(label);
            lay->addWidget(lw);
            QWidget *retwidget = new QWidget;
            retwidget->setLayout(lay);
            return retwidget;
        };

    private:
        Shaman::AbstractPackage::List m_packages;
};

KdemodDistUpgrade::KdemodDistUpgrade(QObject* parent)
    : Shaman::DistUpgrade(parent)
{
    foreach (Shaman::AbstractPackage * const package, Shaman::Base::instance()->core()->backend()->packages()) {
        if (package->name().startsWith(QLatin1String("kdemod-"))) {
            m_kdemodPackages.append(package);
        }
    }

    setSetup(new KdemodSetupFactory(m_kdemodPackages));

    setCustomStateName(CustomState1, i18n("Updating repositories"));
    setCustomStateViewType(CustomState1, ProgressView);
    setCustomStateIconOrMovieByName(CustomState1, "shaman-refresh-cache");

    setCustomStateName(CustomState2, i18n("Downloading new packages"));
    setCustomStateViewType(CustomState2, DownloadView);
    setCustomStateIconOrMovieByName(CustomState2, "shaman-downloading");

    setCustomStateName(CustomState3, i18n("Removing old packages"));
    setCustomStateViewType(CustomState3, ProgressView);
    setCustomStateIconOrMovieByName(CustomState3, "shaman-misc-processing");

    setCustomStateName(CustomState4, i18n("Installing new packages"));
    setCustomStateViewType(CustomState4, ProgressView);
    setCustomStateIconOrMovieByName(CustomState4, "shaman-installing");

    setCustomStateName(CustomState5, i18n("Cleaning up"));
    setCustomStateViewType(CustomState5, ProgressView);
    setCustomStateIconOrMovieByName(CustomState5, "shaman-cleanup");

    connect(Aqpm::Backend::instance(), SIGNAL(operationFinished(bool)),
            this, SLOT(slotOperationFinished(bool)));
    connect(Aqpm::Backend::instance(), SIGNAL(streamTotalDownloadSize(qint64)),
            this, SLOT(streamTotalDownloadSize(qint64)));
}

KdemodDistUpgrade::~KdemodDistUpgrade()
{

}

void KdemodDistUpgrade::streamTotalDownloadSize(qint64 size)
{
    m_totalDlSize = size;
}

void KdemodDistUpgrade::packageProgress(const QString& package, qint64 done, qint64 total)
{
    emit entityDownloadProgressChanged(package, done, total);

    // Now compute the total progress
    if (!m_packageDownload.contains(package)) {
        m_packageDownload[package] = 0;
    }
    qint64 step = done - m_packageDownload[package];
    m_downloaded += step;
    emit downloadProgressChanged(m_downloaded, m_totalDlSize);
}

void KdemodDistUpgrade::packageProgress(const QString& package, int percent, int howmany, int remain)
{
    if (m_packageWeight < 0) {
        m_packageWeight = 100 / (howmany+remain);
    }
    int progress = (int)(m_packagesDone.count() * m_packageWeight) + (int)((percent / 100) * m_packageWeight);
    if (percent >= 100 && !m_packagesDone.contains(package)) {
        m_packagesDone.append(package);
    }
    emit progressChanged(progress);
}

void KdemodDistUpgrade::questionAnswered(Shaman::TransactionQuestion* question)
{
    if (question->answer() == Shaman::TransactionQuestion::AnswerYes) {
        //reboot
    }
}

void KdemodDistUpgrade::start(QWidget* setupResult)
{
    m_installPackages.clear();
    KListWidget *lw = setupResult->findChild<KListWidget*>("__KDEMOD_DIST_UPGRADE_WIDGET___");
    for (int i = 0; i < lw->count(); ++i) {
        if (lw->item(i)->checkState() != Qt::Unchecked) {
            m_installPackages.append(lw->item(i)->text());
        }
    }

    // Update databases, just to start
    setState(CustomState1);
    Aqpm::Backend::instance()->updateDatabase();
}

void KdemodDistUpgrade::slotOperationFinished(bool result)
{
    if (!result) {
        // Return, we failed
        return;
    }

    if (state() == CustomState1) {
        setState(CustomState2);
        // First of all, download kdemod packages, so that the user can resume from console
        Aqpm::Backend::instance()->clearQueue();

        foreach (const QString &package, m_installPackages) {
            Aqpm::Backend::instance()->addItemToQueue(package, Aqpm::QueueItem::Sync);
        }

        Aqpm::Backend::instance()->downloadQueue();
        return;
    } else if (state() == CustomState2) {
        setState(CustomState3);
        // Ok, now that we stored everything, remove all kdemod packages.
        Aqpm::Backend::instance()->clearQueue();
        int packageCount = 0;

        foreach (Shaman::AbstractPackage * const package, m_kdemodPackages) {
            Aqpm::Backend::instance()->addItemToQueue(package->name(), Aqpm::QueueItem::Remove);
            ++packageCount;
        }

        m_packageWeight = 100 / packageCount;

        Aqpm::Backend::instance()->processQueue(Aqpm::Globals::NoDeps);
        return;
    } else if (state() == CustomState3) {
        setState(CustomState4);
        // Now install the packages
        Aqpm::Backend::instance()->clearQueue();

        foreach (const QString &package, m_installPackages) {
            Aqpm::Backend::instance()->addItemToQueue(package, Aqpm::QueueItem::Sync);
        }

        m_packageWeight = -1;

        Aqpm::Backend::instance()->processQueue(Aqpm::Globals::AllDeps);
        return;
    } else if (state() == CustomState4) {
        setState(CustomState5);
        newMessage(i18n("Cleaning up the package cache..."), Shaman::TransactionInterface::BackendMessageLevel);
        // Everything went fine! Clean up the cache to remove leftovers from old packages
        connect(Aqpm::Maintenance::instance(), SIGNAL(actionPerformed(bool)),
                this, SLOT(slotOperationFinished(bool)));
        Aqpm::Maintenance::instance()->performAction(Aqpm::Maintenance::CleanCache);
        return;
    } else if (state() == CustomState5) {
        // Ask through a non modal question if we want to reboot, since we definitely want to.
        QString text = i18n("The upgrade has been completed successfully. It is strongly advised to reboot as soon as possible, "
                            "since your system might be unstable until then. Do you want to do it now?");
        newQuestion(new Shaman::StringTransactionQuestion(text, Shaman::TransactionQuestion::AnswerYes |
                                                          Shaman::TransactionQuestion::AnswerNo, KIcon("system-reboot"), false));
        // We're 100% done!! Wow!
        setState(NotRunning);
    }
}

