/***************************************************************************
 *   Copyright (C) 2009 by Dario Freddi                                    *
 *   drf@chakra-project.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#include "AqpmConfig.h"

#include <aqpm/Backend.h>
#include <aqpm/Configuration.h>

#include <QDebug>

#include "ui_aqpmConfig.h"

using namespace Aqpm;

#include <KPluginFactory>
#include <KAboutData>
#include <klocalizedstring.h>
#include <KMessageBox>

K_PLUGIN_FACTORY(AqpmConfigFactory,
                 registerPlugin<AqpmConfig>();
                )
K_EXPORT_PLUGIN(AqpmConfigFactory("kcm_aqpm_general"))


AqpmConfig::AqpmConfig(QWidget *parent, const QVariantList &args)
        : KCModule(AqpmConfigFactory::componentData(), parent, args)
        , m_ui(new Ui::AqpmConfig)
{
    // Initialize the backend correctly
    Backend::instance()->safeInitialization();
    //Backend::instance()->setShouldHandleAuthorization(true);

    QLayout *layout = new QVBoxLayout(this);
    layout->setMargin(0);

    setButtons(Apply);

    KAboutData *about =
        new KAboutData("kcm_aqpm_general", "kcm_aqpm_general", ki18n("Aqpm Configuration"),
                       "bla", ki18n("Configures Aqpm global options"),
                       KAboutData::License_GPL, ki18n("(c), 2009 Dario Freddi"));

    about->addAuthor(ki18n("Dario Freddi"), ki18n("Maintainer") , "drf@chakra-project.org",
                     "http://drfav.wordpress.com");

    setAboutData(about);

    QWidget *widget = new QWidget(this);
    m_ui->setupUi(widget);
    layout->addWidget(widget);

    connect(m_ui->ignorePkgLine, SIGNAL(textChanged(QString)), this, SLOT(changed()));
    connect(m_ui->ignoreGrpsLine, SIGNAL(textChanged(QString)), this, SLOT(changed()));
    connect(m_ui->noExtractLine, SIGNAL(textChanged(QString)), this, SLOT(changed()));
    connect(m_ui->noUpgradeLine, SIGNAL(textChanged(QString)), this, SLOT(changed()));

    connect(m_ui->logFileLine, SIGNAL(textChanged(QString)), this, SLOT(changed()));
    connect(m_ui->useDeltaBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->sysLogBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->multiServerBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
}


void AqpmConfig::load()
{
    QString txt;

    foreach(const QString &str, Aqpm::Configuration::instance()->value("options/IgnorePkg").toStringList()) {
        txt.append(str);
        txt.append(QChar(' '));
    }
    if (txt.size()) {
        txt.remove(txt.size() - 1, 1);
    }
    m_ui->ignorePkgLine->setText(txt);
    txt.clear();

    foreach(const QString &str, Aqpm::Configuration::instance()->value("options/IgnoreGroup").toStringList()) {
        txt.append(str);
        txt.append(QChar(' '));
    }
    if (txt.size()) {
        txt.remove(txt.size() - 1, 1);
    }
    m_ui->ignoreGrpsLine->setText(txt);
    txt.clear();

    foreach(const QString &str, Aqpm::Configuration::instance()->value("options/NoExtract").toStringList()) {
        txt.append(str);
        txt.append(QChar(' '));
    }
    if (txt.size()) {
        txt.remove(txt.size() - 1, 1);
    }
    m_ui->noExtractLine->setText(txt);
    txt.clear();

    foreach(const QString &str, Aqpm::Configuration::instance()->value("options/NoUpgrade").toStringList()) {
        txt.append(str);
        txt.append(QChar(' '));
    }
    if (txt.size()) {
        txt.remove(txt.size() - 1, 1);
    }
    m_ui->noUpgradeLine->setText(txt);
    txt.clear();

    m_ui->logFileLine->setText(Aqpm::Configuration::instance()->value("options/LogFile").toString());
    m_ui->useDeltaBox->setChecked(Aqpm::Configuration::instance()->value("options/UseDelta").toBool());
    m_ui->sysLogBox->setChecked(Aqpm::Configuration::instance()->value("options/UseSyslog").toBool());
    m_ui->multiServerBox->setChecked(Aqpm::Configuration::instance()->value("options/MultiDownload").toBool());

    emit changed(false);
}


void AqpmConfig::save()
{
    Configuration::instance()->setValue("options/IgnorePkg", m_ui->ignorePkgLine->text().split(QChar(' ')));
    Configuration::instance()->setValue("options/IgnoreGroup", m_ui->ignoreGrpsLine->text().split(QChar(' ')));
    Configuration::instance()->setValue("options/NoExtract", m_ui->noExtractLine->text().split(QChar(' ')));
    Configuration::instance()->setValue("options/NoUpgrade", m_ui->noUpgradeLine->text().split(QChar(' ')));

    Configuration::instance()->setValue("options/LogFile", m_ui->logFileLine->text());
    Configuration::instance()->setValue("options/UseDelta", m_ui->useDeltaBox->isChecked());
    Configuration::instance()->setValue("options/UseSyslog", m_ui->sysLogBox->isChecked());
    Configuration::instance()->setValue("options/MultiDownload", m_ui->multiServerBox->isChecked());

    if (!Configuration::instance()->saveConfiguration()) {
        qDebug() << "argh";
        KMessageBox::error(this, i18n("There has been a problem while saving the configuration."));
    }
}


void AqpmConfig::defaults()
{

}

#include "AqpmConfig.moc"
