/***************************************************************************
 *   Copyright (C) 2009 by Dario Freddi                                    *
 *   drf@chakra-project.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#include "DatabaseConfig.h"

#include <aqpm/Backend.h>
#include <aqpm/Configuration.h>

#include <KDebug>
#include <QUrl>

#include <polkit-qt/ActionButton>

#include "ui_databaseConfig.h"

using namespace Aqpm;

#include <KPluginFactory>
#include <KAboutData>
#include <klocalizedstring.h>
#include <KMessageBox>
#include <QPainter>
#include <QFormLayout>
#include <KLineEdit>
#include "EditMirrorsDialog.h"

const int SPACING = 2;

K_PLUGIN_FACTORY(DatabaseConfigFactory,
                 registerPlugin<DatabaseConfig>();
                )
K_EXPORT_PLUGIN(DatabaseConfigFactory("kcm_aqpm_databases"))

ThirdPartyItemDelegate::ThirdPartyItemDelegate(QObject* parent): QStyledItemDelegate(parent)
{

}

ThirdPartyItemDelegate::~ThirdPartyItemDelegate()
{

}

void ThirdPartyItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    QStyleOptionViewItemV4 optV4 = option;
    initStyleOption(&optV4, index);

    painter->save();

    painter->setClipRect(optV4.rect);

    QStyle *style = QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter);

    QRect iconRect = optV4.rect;
    iconRect.setSize(QSize(optV4.rect.height() - SPACING * 2, optV4.rect.height() - SPACING * 2));
    iconRect.moveTo(QPoint(optV4.rect.height() / 1.7 + iconRect.x() + SPACING, iconRect.y() + SPACING));

    painter->drawPixmap(iconRect, optV4.icon.pixmap(iconRect.size()));

    painter->save();

    QFont font = painter->font();
    font.setWeight(QFont::Bold);
    painter->setFont(font);

    QRect textRect = optV4.rect;
    textRect.setX(iconRect.x() + iconRect.width() + SPACING);
    textRect = painter->boundingRect(textRect, Qt::AlignLeft | Qt::AlignTop, optV4.text);
    textRect.moveTo(QPoint(textRect.x(), textRect.y() + SPACING));

    painter->drawText(textRect, optV4.text);

    painter->restore();

    font = painter->font();
    font.setItalic(true);
    painter->setFont(font);

    QRect serverRect = optV4.rect;
    serverRect.setX(iconRect.x() + iconRect.width() + SPACING);
    serverRect = painter->boundingRect(serverRect, Qt::AlignLeft | Qt::AlignBottom, index.data(Qt::UserRole).toString());
    serverRect.moveTo(QPoint(serverRect.x(), serverRect.y() - SPACING));
    painter->drawText(serverRect, index.data(Qt::UserRole).toString());

    painter->restore();
}

QSize ThirdPartyItemDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    return QSize(0, QFontMetrics(option.font).height() * 2 + 4 * SPACING);
}

///

DatabaseConfig::DatabaseConfig(QWidget *parent, const QVariantList &args)
        : KCModule(DatabaseConfigFactory::componentData(), parent, args)
        , m_ui(new Ui::DatabaseConfig)
{
    // Initialize the backend correctly
    Backend::instance()->safeInitialization();
    //Backend::instance()->setShouldHandleAuthorization(true);

    QLayout *layout = new QVBoxLayout(this);
    layout->setMargin(0);

    setButtons(Apply);

    KAboutData *about =
        new KAboutData("kcm_aqpm_databases", "kcm_aqpm_databases", ki18n("Aqpm Database Configuration"),
                       "bla", ki18n("A configurator for Aqpm databases"),
                       KAboutData::License_GPL, ki18n("(c), 2009 Dario Freddi"));

    about->addAuthor(ki18n("Dario Freddi"), ki18n("Maintainer") , "drf@chakra-project.org",
                     "http://drfav.wordpress.com");

    setAboutData(about);

    QWidget *widget = new QWidget(this);
    m_ui->setupUi(widget);
    layout->addWidget(widget);

    init();
}

void DatabaseConfig::init()
{
    m_ui->thirdPartyListWidget->setItemDelegate(new ThirdPartyItemDelegate);

    m_ui->repoTab->setTabIcon(3, KIcon("format-list-ordered"));
    m_ui->addThirdPartyButton->setIcon(KIcon("list-add"));

    connect(m_ui->changeArchMirrorsButton, SIGNAL(clicked(bool)), SLOT(changeArchMirrors()));
    connect(m_ui->changeKdemodMirrorsButton, SIGNAL(clicked(bool)), SLOT(changeKdemodMirrors()));
    connect(m_ui->addThirdPartyButton, SIGNAL(clicked(bool)), this, SLOT(addThirdPartyRepo()));

    connect(m_ui->coreBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->communityBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->extraBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->testingBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod3Box, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod4Box, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod4ExtragearBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod4PlaygroundBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod4TestingBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->KDEMod4UnstableBox, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->dbOrderList, SIGNAL(indexesMoved(QModelIndexList)), this, SLOT(changed()));

    connect(m_ui->coreBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->communityBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->extraBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->testingBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod3Box, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod4Box, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod4ExtragearBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod4PlaygroundBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod4TestingBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
    connect(m_ui->KDEMod4UnstableBox, SIGNAL(stateChanged(int)), this, SLOT(updateDatabaseList()));
}

void DatabaseConfig::load()
{
    Database::List repos = Backend::instance()->availableDatabases();

    foreach (Database *db, repos) {
        if (db->name() == "core") {
            m_ui->coreBox->setChecked(true);
        } else if (db->name() == "extra")
            m_ui->extraBox->setChecked(true);
        else if (db->name() == "community")
            m_ui->communityBox->setChecked(true);
        else if (db->name() == "testing")
            m_ui->testingBox->setChecked(true);
        else if (db->name() == "kdemod-core") {
            m_ui->KDEMod4Box->setChecked(true);
        } else if (db->name() == "kdemod-extragear") {
            m_ui->KDEMod4ExtragearBox->setChecked(true);
        } else if (db->name() == "kdemod-playground") {
            m_ui->KDEMod4PlaygroundBox->setChecked(true);
        } else if (db->name() == "kdemod-testing") {
            m_ui->KDEMod4TestingBox->setChecked(true);
        } else if (db->name() == "kdemod-unstable") {
            m_ui->KDEMod4UnstableBox->setChecked(true);
        } else if (db->name() == "kdemod-legacy") {
            m_ui->KDEMod3Box->setChecked(true);
        }
    }

    // Now third parties
    foreach (const QString &string, Configuration::instance()->mirrors(true)) {
        QListWidgetItem *item = new QListWidgetItem(KIcon("repository"),
                                                    Configuration::instance()->databasesForMirror(string).first());
        item->setData(Qt::UserRole, Configuration::instance()->serversForMirror(string).first());
        m_ui->thirdPartyListWidget->addItem(item);
    }

    // Now set the databases
    m_ui->dbOrderList->clear();
    m_ui->dbOrderList->addItems(Configuration::instance()->databases());

    emit changed(false);
}

void DatabaseConfig::save()
{
    QStringList dbs;

    for (int i = 0; i < m_ui->dbOrderList->count(); ++i) {
        dbs.append(m_ui->dbOrderList->item(i)->text());
    }

    Configuration::instance()->setDatabases(dbs);

    // Save mirrors
    /*QStringList mirrors;

    foreach (MirrorWidget *wg, m_archMirrors) {
        mirrors.append(wg->mirror());
    }
    Configuration::instance()->setServersForMirror(mirrors, "arch");

    mirrors.clear();
    foreach (MirrorWidget *wg, m_kdemodMirrors) {
        mirrors.append(wg->mirror());
    }
    Configuration::instance()->setServersForMirror(mirrors, "kdemod");*/

    // Third party stuff
    for (int i = 0; i < m_ui->thirdPartyListWidget->count(); ++i) {
        Configuration::instance()->setDatabasesForMirror(QStringList() << m_ui->thirdPartyListWidget->item(i)->text(),
                                                         m_ui->thirdPartyListWidget->item(i)->text());
        Configuration::instance()->setServersForMirror(QStringList() <<
                                                         m_ui->thirdPartyListWidget->item(i)->data(Qt::UserRole).toString(),
                                                         m_ui->thirdPartyListWidget->item(i)->text());
    }

    // That's it, let's do this
    //Backend::instance()->setShouldHandleAuthorization(true);
    if (!Configuration::instance()->saveConfiguration()) {
        KMessageBox::error(this, i18n("There has been a problem while saving the configuration."));
    }
    //Backend::instance()->setShouldHandleAuthorization(false);
}

void DatabaseConfig::defaults()
{
}

void DatabaseConfig::updateDatabaseList()
{
    qDebug() << "Updating dbs";
    QStringList databases;

    if (m_ui->coreBox->isChecked()) {
        databases.append("core");
    }
    if (m_ui->extraBox->isChecked()) {
        databases.append("extra");
    }
    if (m_ui->communityBox->isChecked()) {
        databases.append("community");
    }
    if (m_ui->testingBox->isChecked()) {
        databases.append("testing");
    }
    if (m_ui->KDEMod3Box->isChecked()) {
        databases.append("kdemod-legacy");
    }
    if (m_ui->KDEMod4Box->isChecked()) {
        databases.append("kdemod-core");
    }
    if (m_ui->KDEMod4ExtragearBox->isChecked()) {
        databases.append("kdemod-extragear");
    }
    if (m_ui->KDEMod4PlaygroundBox->isChecked()) {
        databases.append("kdemod-playground");
    }
    if (m_ui->KDEMod4TestingBox->isChecked()) {
        databases.append("kdemod-testing");
    }
    if (m_ui->KDEMod4UnstableBox->isChecked()) {
        databases.append("kdemod-unstable");
    }

    for (int i = 0; i < m_ui->thirdPartyListWidget->count(); ++i) {
        databases.append(m_ui->thirdPartyListWidget->item(i)->text());
    }

    // Now check
    foreach (const QString &db, databases) {
        if (m_ui->dbOrderList->findItems(db, Qt::MatchRegExp).isEmpty()) {
            m_ui->dbOrderList->addItem(db);
        }
    }

    qDebug() << databases;
    qDebug() << m_ui->dbOrderList->count();

    // Any removals?
    if (databases.count() < m_ui->dbOrderList->count()) {
        for (int i = 0; i < m_ui->dbOrderList->count(); ++i) {
            qDebug() << m_ui->dbOrderList->item(i)->text();
            if (!databases.contains(m_ui->dbOrderList->item(i)->text())) {
                qDebug() << "Ditchit";
                QListWidgetItem *old = m_ui->dbOrderList->takeItem(i);
                delete old;
                --i;
            }
        }
    }
}

void DatabaseConfig::addThirdPartyRepo()
{
    KDialog *dialog = new KDialog(this);
    //dialog->setAttribute(Qt::WA_DeleteOnClose);
    QLabel *label = new QLabel(i18n("Fill the fields below to add a new repository"));
    QFormLayout *lay = new QFormLayout;
    QLabel *name = new QLabel(i18n("Name"));
    QLabel *server = new QLabel(i18n("Server"));
    KLineEdit *nameEdit = new KLineEdit();
    KLineEdit *serverEdit = new KLineEdit();
    QVBoxLayout *vlay = new QVBoxLayout;
    QWidget *w = new QWidget;

    lay->addRow(name, nameEdit);
    lay->addRow(server, serverEdit);

    vlay->addWidget(label);
    vlay->addLayout(lay);

    w->setLayout(vlay);
    dialog->setMainWidget(w);
    dialog->setModal(true);
    dialog->setButtons(KDialog::Ok | KDialog::Cancel);

    if (dialog->exec() == KDialog::Accepted) {
        if (serverEdit->text().isEmpty() || nameEdit->text().isEmpty()) {
            kDebug() << serverEdit->text() << nameEdit->text();
            KMessageBox::error(this, i18n("You have to fill out all the fields"));
            return;
        }

        KUrl mirror(serverEdit->text());

        if (mirror.protocol() != "http" && mirror.protocol() != "file" && mirror.protocol() != "ftp") {
            KMessageBox::error(this, i18n("Mirror Format is incorrect. It should be a valid http, ftp, or file URL"));
            return;
        }

        // Everything's fine, let's go
        QListWidgetItem *item = new QListWidgetItem(KIcon("repository"), nameEdit->text());
        item->setData(Qt::UserRole, serverEdit->text());
        m_ui->thirdPartyListWidget->addItem(item);
        updateDatabaseList();
    }
}

void DatabaseConfig::changeArchMirrors()
{
    KDialog *dialog = new EditMirrorsDialog(Aqpm::Configuration::ArchMirror, this);
    dialog->exec();
}

void DatabaseConfig::changeKdemodMirrors()
{
    KDialog *dialog = new EditMirrorsDialog(Aqpm::Configuration::KdemodMirror, this);
    dialog->exec();
}

/*void DatabaseConfig::addMirror()
{
    if (m_ui->addMirrorLine->text().isEmpty()) {
        return;
    }

    QString mirror(m_ui->addMirrorLine->text());

    if (!mirror.contains(QString("$repo")) || (!mirror.startsWith(QString("http://")) &&
            !mirror.startsWith(QString("ftp://")) && !mirror.startsWith(QString("file://"))) ||
            mirror.contains(QString(" ")) || !QUrl(mirror).isValid()) {
        KMessageBox::error(this, i18nc("Obviously keep the example as it is ;)", "Mirror Format is incorrect. "
                                  "The mirror should be of the form:\nhttp://mirror.org/$repo/os/i686"));
        return;
    }*/

    /* Ok, our mirror should be valid. Let's add it to mirrorlist. */
    /*if (Configuration::instance()->addMirrorToMirrorList(mirror, Configuration::ArchMirror)) {
        KMessageBox::information(this, i18nc("mirrorlist here means /etc/pacman.d/mirrorlist, so it should not "
                                             "be translated.",
                                             "The mirror was successfully added.\nIt is now available in the mirror list."));

        m_ui->addMirrorLine->clear();
    } else {
        KMessageBox::error(this, i18n("There was an error while trying to add the mirror. This could be due "
                                      "to your system policy preventing you from performing this action."));
    }
}*/
