/***************************************************************************
 *   Copyright (C) 2009 by Dario Freddi                                    *
 *   drf@chakra-project.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
 ***************************************************************************/

#include "EditMirrorsDialog.h"
#include "ui_editMirrors.h"
#include <QPainter>
#include <KMessageBox>

const int SPACING = 2;
const KIcon s_g_masterIcon = KIcon("bookmark-new-list");
const KIcon s_g_httpIcon = KIcon("applications-internet");
const KIcon s_g_ftpIcon = KIcon("folder-remote");
const KIcon s_g_fileIcon = KIcon("folder-downloads");

MirrorsDelegate::MirrorsDelegate(QObject* parent)
    : QStyledItemDelegate(parent)
{

}

MirrorsDelegate::~MirrorsDelegate()
{

}

void MirrorsDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    QStyleOptionViewItemV4 optV4 = option;
    initStyleOption(&optV4, index);

    painter->save();

    painter->setClipRect(optV4.rect);

    QStyle *style = QApplication::style();
    style->drawPrimitive(QStyle::PE_PanelItemViewItem, &option, painter);

    QRect iconRect = optV4.rect;
    iconRect.setSize(QSize(optV4.rect.height() - SPACING * 2, optV4.rect.height() - SPACING * 2));
    iconRect.moveTo(QPoint(iconRect.x() + SPACING, iconRect.y() + SPACING));

    QString text = optV4.text;

    if (index.row() == 0) {
        painter->drawPixmap(iconRect, s_g_masterIcon.pixmap(iconRect.size()));

        QFont font = painter->font();
        font.setWeight(QFont::Bold);
        painter->setFont(font);
        text = optV4.text + " (" + i18n("master repository") + ')';
    } else if (text.startsWith(QLatin1String("file"))) {
        painter->drawPixmap(iconRect, s_g_fileIcon.pixmap(iconRect.size()));
    } else if (text.startsWith(QLatin1String("ftp"))) {
        painter->drawPixmap(iconRect, s_g_ftpIcon.pixmap(iconRect.size()));
    } else {
        painter->drawPixmap(iconRect, s_g_httpIcon.pixmap(iconRect.size()));
    }

    QRect textRect = optV4.rect;
    textRect.setX(iconRect.x() + iconRect.width() + SPACING);
    textRect = painter->boundingRect(textRect, Qt::AlignLeft | Qt::AlignVCenter, text);

    painter->drawText(textRect, text);

    painter->restore();
}

QSize MirrorsDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    if (index.row() > 0) {
        return QSize(0, QFontMetrics(option.font).height() + 2 * SPACING);
    } else {
        return QSize(0, (QFontMetrics(option.font).height() + 2 * SPACING) * 2);
    }
}

////

EditMirrorsDialog::EditMirrorsDialog(Aqpm::Configuration::MirrorType mirrorType, QWidget* parent)
    : KDialog(parent)
    , m_mirrorType(mirrorType)
    , m_ui(new Ui::EditMirrors)
{
    QWidget *w = new QWidget;
    m_ui->setupUi(w);
    setMainWidget(w);

    setModal(true);
    setButtons(Apply | Cancel);

    m_ui->listWidget->setItemDelegate(new MirrorsDelegate(this));

    // load the mirrors
    QStringList mirrors = Aqpm::Configuration::instance()->getMirrorList(m_mirrorType);
    foreach (const QString &mirror, mirrors) {
        m_ui->listWidget->addItem(new QListWidgetItem(mirror));
    }

    // Attach the signals
    connect(this, SIGNAL(accepted()), this, SLOT(slotAccepted()));
}

EditMirrorsDialog::~EditMirrorsDialog()
{

}

void EditMirrorsDialog::slotAccepted()
{
    // Attempt to save the mirrors
    QStringList mirrors;
    for (int i = 0; i < m_ui->listWidget->count(); ++i) {
        mirrors.append(m_ui->listWidget->item(i)->text());
    }

    if (Aqpm::Configuration::instance()->setMirrorList(mirrors, m_mirrorType)) {
        deleteLater();
    } else {
        KMessageBox::error(this, i18n("There has been an error while saving the mirror list"));
        show();
    }
}
