/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PKPlugin.h"
#include "PKPackage.h"

#include <shamantransactioninterface.h>

#include <KDebug>
#include <KLocale>
#include <KAboutData>
#include <kworkspace/kworkspace.h>

#include <QTime>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <KIcon>
#include "PKStringsAndIcons.h"
#include <QWidget>
#include <QLabel>
#include <KTextBrowser>
#include <QBoxLayout>
#include "PKPackageListener.h"
#include <shamanquery.h>

using namespace Shaman;

QWidget* EulaWidgetFactory::createInstance() const
{
    QWidget *w = new QWidget();
    QLabel *l = new QLabel(m_header);
    KTextBrowser *tb = new KTextBrowser();
    tb->setText(m_eula);
    QVBoxLayout *lay = new QVBoxLayout;
    lay->addWidget(l);
    lay->addWidget(tb);
    w->setLayout(lay);

    return w;
}

class PKPlugin::Private
{
    public:
        Private() : stopBgTransactions(false), waiting(false) {}
        ~Private() {}

        Shaman::Repository::List repos;
        Shaman::AbstractPackage::List packages;
        QHash<QString, AbstractPackage*> nativePackagesHash;
        QHash<QString, PKPackage*> nameToPackageHash;
        Shaman::TransactionInterface::State cachedState;
        QString queueTid;
        AbstractPackage *currentTransactionPackage;
        PackageGroup *currentGroup;
        QList<PackageKit::Package*> installQueue;
        QList<PackageKit::Package*> upgradeQueue;
        QList<PackageKit::Package*> removeQueue;
        QHash<QString, Shaman::AbstractPackage::List> searchPackages;
        QHash<QString, QUuid> searchUuids;
        QHash<QUuid, QString> questions;
        QSet<PackageKit::Client::Group> nativeGroups;
        QSet<PackageKit::Client::Group>::const_iterator groupsIterator;
        QList<QObject*> garbage;
        Shaman::PackageGroup::List groups;
        bool stopBgTransactions;
        QString operationString;
        int groupAt;
        Backend::State backendCachedState;
        bool waiting;
        PackageKit::Client::EulaInfo lastEula;
        QString lastSimulateString;
        int lastSimulateProgress;
};

PKPlugin::PKPlugin()
  : BackendInterface(0),
    d(new Private())
{
}

PKPlugin::~PKPlugin()
{
    delete d;
}

void PKPlugin::init()
{
//     PackageKit::Client::instance()->setHints(QStringList() <<
//                                              "locale=" + KGlobal::locale()->language() + '.' + KGlobal::locale()->encoding());

    PKPackageListener::instance()->setPlugin(this);
    // Create a new daemon
    PackageKit::Client *client = PackageKit::Client::instance();

    connect(client, SIGNAL(error(PackageKit::Client::DaemonError)),
            this, SLOT(errorDetected(PackageKit::Client::DaemonError)));

    PackageKit::Client::Actions actions = client->getActions();

    Shaman::BackendCapabilities::TransactionCapabilities tcap;

    if (actions & PackageKit::Client::ActionInstallPackages) {
        tcap |= Shaman::BackendCapabilities::InstallPackagesCapability;
    }
    if (actions & PackageKit::Client::ActionRemovePackages) {
        tcap |= Shaman::BackendCapabilities::RemovePackagesCapability;
    }
    if (actions & PackageKit::Client::ActionUpdatePackages) {
        tcap |= Shaman::BackendCapabilities::UpgradePackagesCapability;
    }
    if (actions & PackageKit::Client::ActionUpdateSystem) {
        tcap |= Shaman::BackendCapabilities::UpgradeSystemCapability;
    }
    if (actions & PackageKit::Client::ActionRefreshCache) {
        tcap |= Shaman::BackendCapabilities::UpdateRepositoriesCapability;
    }
    tcap |= Shaman::BackendCapabilities::DistUpgradeCapability;

    Shaman::BackendCapabilities::SearchCapabilities scap;
    if (actions & PackageKit::Client::ActionSearchDetails) {
        scap |= Shaman::BackendCapabilities::ExtendedSearchFromBackendCapability;
    }
    scap |= Shaman::BackendCapabilities::BasicSearchFromViewCapability;

    Shaman::BackendCapabilities::PackageGroupingCapabilities pcap;
    if (actions & PackageKit::Client::ActionSearchGroup) {
        pcap |= Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability;
    }

    Shaman::BackendCapabilities::InformationCapabilities icap;
    if (actions & PackageKit::Client::ActionGetDepends) {
        icap |= Shaman::BackendCapabilities::ShowDependsOnCapability;
    }
    if (actions & PackageKit::Client::ActionGetRequires) {
        icap |= Shaman::BackendCapabilities::ShowRequiredByCapability;
    }
    if (actions & PackageKit::Client::ActionGetFiles) {
        icap |= Shaman::BackendCapabilities::ShowFileListCapability;
    }
    icap |= Shaman::BackendCapabilities::ShowUpdateType;

    setCapabilities(tcap, scap, pcap, icap);
    PackageKit::Client::BackendDetail detail = PackageKit::Client::instance()->getBackendDetail();

    KAboutPerson dario(ki18n("Dario Freddi"), ki18n("Maintainer"), "drf@kde.org", "http://drfav.wordpress.com");
    // People might want to patch out the package icon here.
    setProperties(i18n("PackageKit"), i18n("a universal package management system running with %1 by %2", detail.name,
                                           detail.author),
//                                        .subs(QString("%1.%2.%3").arg(client->versionMajor()).arg(client->versionMinor())
//                                                                 .arg(client->versionMicro()))/*.subs(client->backendName())*/,
                  KIcon("applications-other"), KIcon("application-x-rpm"),
                  QList<KAboutPerson>() << dario, QStringList());

    // Track state changes
    connect(backend(), SIGNAL(stateChanged(Shaman::Backend::State)), this, SLOT(slotStateChanged(Shaman::Backend::State)));

    setState(Backend::Idling);
}

void PKPlugin::reload()
{
    setState(Shaman::Backend::Reloading);
    stopBackgroundTransactions();

    setRepositories(Shaman::Repository::List());
    setPackages(Shaman::AbstractPackage::List());
    setPackageGroups(Shaman::PackageGroup::List());

    d->repos.clear();
    d->groups.clear();
    d->nativeGroups.clear();
    d->nameToPackageHash.clear();
    d->nativePackagesHash.clear();
    d->packages.clear();

    reloadRepositories();
}

void PKPlugin::reloadRepositories()
{
    d->operationString = i18n("Loading repositories...");
    PackageKit::Transaction * transaction = PackageKit::Client::instance()->getRepoList();
    connectTransaction(transaction);
    connect(transaction, SIGNAL(repoDetail(QString, QString, bool)), SLOT(addRepo(QString, QString, bool)));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(backgroundOperationProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(reloadPackages()));
}

void PKPlugin::addRepo(const QString &id, const QString &description, bool enabled)
{
    Q_UNUSED(description)

    if (enabled) {
        d->repos.append(new Repository(id, KUrl(), this));
    }
}

void PKPlugin::reloadPackageGroups()
{
    setPackages(d->packages);

    d->groupAt = 0;

    d->stopBgTransactions = false;

    d->nativeGroups = PackageKit::Client::instance()->getGroups();
    d->groupsIterator = d->nativeGroups.constBegin();
    // And set back to idle
    setIdle();

    // Start caching updates
    checkAndMarkUpdates();
}

void PKPlugin::reloadNextPackageGroup()
{
    if (d->stopBgTransactions) {
        --d->groupsIterator;
        return;
    }

    d->operationString = i18n("Loading package groups...");

    if (d->groupsIterator != d->nativeGroups.constEnd()) {
        ++d->groupAt;
        d->currentGroup = new PackageGroup(PKStringsAndIcons::groupName(*d->groupsIterator).first,
                                           PKStringsAndIcons::groupName(*d->groupsIterator).second, this);
        d->groups.append(d->currentGroup);
        PackageKit::Transaction *transaction = PackageKit::Client::instance()->searchGroup(*d->groupsIterator);
        connectTransaction(transaction);
        connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(addPackageToGroup(PackageKit::Package*)));
        connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(reloadNextPackageGroup()));
        connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
                this, SLOT(groupLoadingProgress(PackageKit::Transaction::ProgressInfo)));
        ++d->groupsIterator;
    } else {
        setPackageGroups(d->groups);
    }
}

void PKPlugin::addPackageToGroup(PackageKit::Package *package)
{
    if (d->nativePackagesHash.contains(package->id())) {
        d->currentGroup->append(d->nativePackagesHash[package->id()]);
    }
}

void PKPlugin::searchPackages(Shaman::Query *query)
{
    // It can be only extended
    d->operationString = i18n("Searching for %1...", query->query().pattern());
    PackageKit::Transaction * transaction = PackageKit::Client::instance()->searchDetails(query->query().pattern());
    connectTransaction(transaction);
    transaction->setProperty("__shaman__query", query->id().toString());
    d->searchUuids[query->id().toString()] = query->id();
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(addSearchPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(backgroundOperationProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)),
            this, SLOT(slotSearchFinished()));
}

void PKPlugin::addSearchPackage(PackageKit::Package* p)
{
    if (d->nativePackagesHash[p->id()]) {
        d->searchPackages[sender()->property("__shaman__query").toString()].append(d->nativePackagesHash[p->id()]);
    }
}

void PKPlugin::slotSearchFinished()
{
    searchCompleted(d->searchUuids[sender()->property("__shaman__query").toString()],
                    d->searchPackages[sender()->property("__shaman__query").toString()]);
    setState(Backend::Idling);
}

void PKPlugin::reloadPackages()
{
    setRepositories(d->repos);
    d->operationString = i18n("Loading packages...");

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->getPackages();
    connectTransaction(transaction);
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(addPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(backgroundOperationProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(reloadPackageGroups()));
}

void PKPlugin::checkAndMarkUpdates()
{
    d->operationString = i18n("Checking for updates...");
    PackageKit::Transaction * transaction = PackageKit::Client::instance()->getUpdates();
    connectTransaction(transaction);
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(setUpgradeable(PackageKit::Package*)));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(backgroundOperationProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), SLOT(reloadNextPackageGroup()));
}

bool PKPlugin::aboutToChangeState(Backend::State oldstate, Backend::State newstate)
{
    if (oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
    } else if (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling) {
        // Time to reload, guys.
        if (!(PackageKit::Client::instance()->getActions() & PackageKit::Client::ActionResolve)) {
            // No room for partial reloading...
            reload();
            return false;
        }
        // Reload only the interested packages
        AbstractPackage::List toReload;
        foreach (Shaman::AbstractPackage *package, backend()->packages()) {
            if (package->action() != AbstractPackage::NoAction) {
                kDebug() << package->name() << " should be reloaded";
                toReload.append(package);
            }
        }
        partialReload(toReload);
        return false;
    }

    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

void PKPlugin::partialReload(const Shaman::AbstractPackage::List& packages)
{
    setState(Shaman::Backend::Reloading);

    qDebug() << "Partial reload triggered";

    QStringList packageNames;
    foreach (AbstractPackage *p, packages) {
        packageNames.append(p->name());
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->resolve(packageNames);
    connectTransaction(transaction);
    connect(transaction, SIGNAL(package(PackageKit::Package*)), SLOT(reloadPackage(PackageKit::Package*)));
    waitForTransactionFinished(transaction);

    setPackages(d->packages);

    checkAndMarkUpdates();

    qDebug() << "Partial reload finished";
    setState(Backend::Idling);
}

void PKPlugin::setUpgradeable(PackageKit::Package *package)
{
    // Mark the package as upgradeable
    if (d->nameToPackageHash[package->name()]) {
        d->nameToPackageHash[package->name()]->setState(Shaman::AbstractPackage::Upgradeable);
        d->nameToPackageHash[package->name()]->setAvailableVersion(package->version());
        switch (package->state()) {
            case PackageKit::Package::StateBugfix:
                d->nameToPackageHash[package->name()]->setUpdateType(AbstractPackage::BugFixUpdate);
                break;
            case PackageKit::Package::StateEnhancement:
                d->nameToPackageHash[package->name()]->setUpdateType(AbstractPackage::FeatureEnhancementUpdate);
                break;
            case PackageKit::Package::StateImportant:
                d->nameToPackageHash[package->name()]->setUpdateType(AbstractPackage::ImportantUpdate);
                break;
            case PackageKit::Package::StateLow:
                d->nameToPackageHash[package->name()]->setUpdateType(AbstractPackage::LowPriorityUpdate);
                break;
            case PackageKit::Package::StateSecurity:
                d->nameToPackageHash[package->name()]->setUpdateType(AbstractPackage::SecurityUpdate);
                break;
            default:
                // Default is generic update
                break;
        }
    }
}

void PKPlugin::reloadPackage(PackageKit::Package *package)
{
    // Reload each package
    if (!d->nativePackagesHash.contains(package->id()) && !d->nameToPackageHash.contains(package->name()))
    {
        qDebug() << "A different ID and Name";
        PKPackage *p = new PKPackage(package, this);
        d->nativePackagesHash.insert(package->id(), p);
        d->nameToPackageHash.insert(package->name(), p);
        d->packages.append(p);
    } else {
        AbstractPackage *oldp = 0;
        bool idMatches = true;
        oldp = d->nativePackagesHash[package->id()];
        if (!oldp) {
            oldp = d->nameToPackageHash[package->name()];
            idMatches = false;
        }
        if (!oldp) {
            // Give up
            return;
        }

        qDebug() << "The same ID or Name";
        PKPackage *newp = new PKPackage(package, this);

        // Readd to groups
        foreach (PackageGroup *group, backend()->packageGroups()) {
            if (group->contains(oldp)) {
                group->removeOne(oldp);
                group->append(newp);
            }
        }
        // Delete
        d->packages.removeOne(oldp);
        if (idMatches) {
            d->nativePackagesHash.remove(package->id());
        } else {
            // What the fuck??? TODO
            d->nativePackagesHash.remove(d->nativePackagesHash.key(oldp = d->nameToPackageHash[package->name()]));
        }
        d->nameToPackageHash.remove(package->name());

        // Reinsert
        d->nativePackagesHash.insert(package->id(), newp);
        d->nameToPackageHash.insert(package->name(), newp);
        d->packages.append(newp);

    }
}

void PKPlugin::addPackage(PackageKit::Package* p)
{
    PKPackage *package = new PKPackage(p, this);
    d->nativePackagesHash.insert(p->id(), package);
    d->nameToPackageHash.insert(p->name(), package);
    d->packages.append(package);
}

void PKPlugin::updateRepositories()
{
    stopBackgroundTransactions();
    setState(Backend::OnTransaction);
    foreach (Repository *repo, backend()->repositories()) {
        repo->setState(Shaman::Repository::Downloading);
    }
    PackageKit::Transaction * transaction = PackageKit::Client::instance()->refreshCache(false);
    connectTransaction(transaction);
    if (!transaction) {
        return;//TODO: Throw an error
    }
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus, uint)), SLOT(setUpdateDatabaseFinished()));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(reportUpdateDatabaseProgress(PackageKit::Transaction::ProgressInfo)));
    setTransactionState(Shaman::TransactionInterface::Processing);
}

void PKPlugin::setUpdateDatabaseFinished()
{
    foreach (Repository *repo, backend()->repositories()) {
        repo->setState(Shaman::Repository::UpdatingFinished);
    }

    setState(Backend::Idling);
}

bool PKPlugin::cancelRepositoriesUpdate()
{
    return false;
}

void PKPlugin::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    // Ok, we're supposed to actually mark additional packages here. With PK,
    // we need to simulate a transaction
    d->installQueue.clear();
    d->upgradeQueue.clear();
    d->removeQueue.clear();
    foreach (AbstractPackage * package, backend()->queuedPackages()) {
        if (!dynamic_cast<PKPackage*>(package)) {
            continue;
        }
        if (package->action() == Shaman::AbstractPackage::Install) {
            d->installQueue.append(dynamic_cast<PKPackage*>(package)->package());
        }
        else if (package->action() == Shaman::AbstractPackage::Upgrade) {
            d->upgradeQueue.append(dynamic_cast<PKPackage*>(package)->package());
        }
        else if (package->action() == Shaman::AbstractPackage::Uninstall) {
            d->removeQueue.append(dynamic_cast<PKPackage*>(package)->package());
        }
    }

    simulateRemove();
}

void PKPlugin::simulateInstall()
{
    if (d->installQueue.isEmpty() || !(PackageKit::Client::instance()->getActions() &
                                       PackageKit::Client::ActionSimulateInstallPackages)) {
        setState(Backend::Idling);
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->simulateInstallPackages(d->installQueue);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(simulationPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(setIdle()));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(simulateProgressChanged(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(statusChanged(PackageKit::Transaction::Status)),
            this, SLOT(reportStatus(PackageKit::Transaction::Status)));
}

void PKPlugin::simulateUpgrade()
{
    if (d->upgradeQueue.isEmpty() || !(PackageKit::Client::instance()->getActions() &
                                       PackageKit::Client::ActionSimulateUpdatePackages)) {
        simulateInstall();
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->simulateUpdatePackages(d->upgradeQueue);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(simulationPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(simulateInstall()));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(simulateProgressChanged(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(statusChanged(PackageKit::Transaction::Status)),
            this, SLOT(reportStatus(PackageKit::Transaction::Status)));
}

void PKPlugin::simulateRemove()
{
    if (d->removeQueue.isEmpty() || !(PackageKit::Client::instance()->getActions() &
                                      PackageKit::Client::ActionSimulateRemovePackages)) {
        simulateUpgrade();
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->simulateRemovePackages(d->removeQueue);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(simulationPackage(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(simulateUpgrade()));
    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)),
            this, SLOT(simulateProgressChanged(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(statusChanged(PackageKit::Transaction::Status)),
            this, SLOT(reportStatus(PackageKit::Transaction::Status)));
}

void PKPlugin::simulateProgressChanged(PackageKit::Transaction::ProgressInfo info)
{
    d->lastSimulateProgress = info.percentage;
    showOperationProgress(d->lastSimulateString, d->lastSimulateProgress);
}

void PKPlugin::simulationPackage(PackageKit::Package* p)
{
    if (d->nativePackagesHash[p->id()]->action() != AbstractPackage::NoAction) {
        // Ok, pass by
        return;
    }

    switch (p->state()) {
        case PackageKit::Package::StateInstalling:
            d->nativePackagesHash[p->id()]->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);
            break;
        case PackageKit::Package::StateRemoving:
            d->nativePackagesHash[p->id()]->setAction(Shaman::AbstractPackage::TransactionDependencyUninstall);
            break;
        case PackageKit::Package::StateUpdating:
        case PackageKit::Package::StateDowngrading:
            d->nativePackagesHash[p->id()]->setAction(Shaman::AbstractPackage::TransactionDependencyUpgrade);
            break;
        default:
            break;
    }
}

void PKPlugin::processQueue(const Shaman::AbstractPackage::List & packages, const Shaman::ProcessQueueOption::List &options)
{
    Q_UNUSED(options)
    stopBackgroundTransactions();
    d->installQueue.clear();
    d->upgradeQueue.clear();
    d->removeQueue.clear();
    foreach (AbstractPackage * package, packages) {
        PKPackage *p = dynamic_cast<PKPackage*>(package);
        if (!p) {
            continue;
        }
        if (package->action() == Shaman::AbstractPackage::Install) {
            d->installQueue.append(p->package());
        }
        else if (package->action() == Shaman::AbstractPackage::Upgrade) {
            d->upgradeQueue.append(p->package());
        }
        else if (package->action() == Shaman::AbstractPackage::Uninstall) {
            d->removeQueue.append(p->package());
        }
    }

    removePackages();
}

void PKPlugin::installPackages()
{
    if (d->installQueue.isEmpty()) {
        setState(Backend::Idling);
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->installPackages(false, d->installQueue);
    connectTransaction(transaction);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)), SLOT(reportProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(slotCurrentPackageChanged(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(setIdle()));
}

void PKPlugin::upgradePackages()
{
    if (d->upgradeQueue.isEmpty()) {
        installPackages();
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->updatePackages(false, d->upgradeQueue);
    connectTransaction(transaction);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)), SLOT(reportProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(slotCurrentPackageChanged(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(installPackages()));
}

void PKPlugin::removePackages()
{
    if (d->removeQueue.isEmpty()) {
        upgradePackages();
        return;
    }

    PackageKit::Transaction * transaction = PackageKit::Client::instance()->removePackages(d->removeQueue, true, false);
    connectTransaction(transaction);
    if (!transaction) {//TODO: Throw an error
        return;
    }

    connect(transaction, SIGNAL(progressChanged(PackageKit::Transaction::ProgressInfo)), SLOT(reportProgress(PackageKit::Transaction::ProgressInfo)));
    connect(transaction, SIGNAL(package(PackageKit::Package*)), this, SLOT(slotCurrentPackageChanged(PackageKit::Package*)));
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)), this, SLOT(upgradePackages()));
}

void PKPlugin::reportProgress(PackageKit::Transaction::ProgressInfo info)
{
    if (!d->currentTransactionPackage) {
        // We still don't have this info yet
        if (d->cachedState == TransactionInterface::Downloading && info.percentage > 5) {
            kDebug() << "Using the manual reporter, I have no package yet";
            manualProgressReporter()->setTransactionProgress(info.percentage);
        }
        qDebug() << "No package to report progress for";
        return;
    }
    if (d->cachedState == TransactionInterface::Downloading) {
        qDebug() << "Reporting download progress";
        // Download progress, then. Let's try to guess the total size
        if (info.subpercentage == 100) {
            // Special case
            setPackageDownloadProgress(d->currentTransactionPackage,
                                       d->currentTransactionPackage->size(), d->currentTransactionPackage->size());
        }
        qint64 bytesDownloaded = (d->currentTransactionPackage->size() / 100) * info.subpercentage;
        setPackageDownloadProgress(d->currentTransactionPackage, bytesDownloaded,
                                   d->currentTransactionPackage->size());
    } else {
        qDebug() << "Reporting transaction progress";
        setPackageTransactionProgress(d->currentTransactionPackage, info.subpercentage);
    }
}

void PKPlugin::groupLoadingProgress(PackageKit::Transaction::ProgressInfo info)
{
    if (info.percentage && d->groupAt == d->nativeGroups.size()) {
        showOperationProgress(d->operationString, 100);
    } else if (d->groupAt > 0) {
        int percent = (d->groupAt - 1) * (100 / d->nativeGroups.size());
        percent += (info.percentage * 0.01) * (100 / d->nativeGroups.size());
        showOperationProgress(d->operationString, percent);
    }
}

void PKPlugin::reportUpdateDatabaseProgress(PackageKit::Transaction::ProgressInfo info)
{
    manualProgressReporter()->setTransactionProgress(info.percentage);
}

void PKPlugin::backgroundOperationProgress(PackageKit::Transaction::ProgressInfo info)
{
    if (info.percentage > 100) {
        showOperationProgress(d->operationString, -1);
    } else {
        showOperationProgress(d->operationString, info.percentage);
    }
}

void PKPlugin::reportStatus(PackageKit::Transaction::Status status)
{
    if (backend()->state() != Backend::MarkingAdditionalTargets) {
        addTransactionMessage(PKStringsAndIcons::stringForStatus(status));
    } else {
        // We want to show a progress only here
        d->lastSimulateString = PKStringsAndIcons::stringForStatus(status);
        showOperationProgress(d->lastSimulateString, d->lastSimulateProgress);
        return;
    }

    // Status stuff
    switch (status)
    {
        case PackageKit::Transaction::StatusDownload:
            setTransactionState(TransactionInterface::Downloading);
            d->cachedState = TransactionInterface::Downloading;
            kDebug() << "Downloading";
            break;
        case PackageKit::Transaction::StatusRemove:
        case PackageKit::Transaction::StatusUpdate:
        case PackageKit::Transaction::StatusInstall:
            setTransactionState(TransactionInterface::Processing);
            d->cachedState = TransactionInterface::Processing;
            kDebug() << "Processing";
            break;
        case PackageKit::Transaction::StatusCleanup:
            setTransactionState(TransactionInterface::Cleaning);
            d->cachedState = TransactionInterface::Cleaning;
            kDebug() << "Cleanup";
            break;
        case PackageKit::Transaction::StatusFinished:
            break;
        case PackageKit::Transaction::StatusWait:
            break;
        case PackageKit::Transaction::StatusSetup:
        case PackageKit::Transaction::StatusRunning:
        case PackageKit::Transaction::StatusQuery:
        case PackageKit::Transaction::StatusInfo:
        case PackageKit::Transaction::StatusRefreshCache:
        case PackageKit::Transaction::StatusObsolete:
        case PackageKit::Transaction::StatusDepResolve:
        case PackageKit::Transaction::StatusSigCheck:
        case PackageKit::Transaction::StatusRollback:
        case PackageKit::Transaction::StatusTestCommit:
        case PackageKit::Transaction::StatusCommit:
        case PackageKit::Transaction::StatusRequest:
        case PackageKit::Transaction::StatusCancel:
        case PackageKit::Transaction::StatusDownloadRepository:
        case PackageKit::Transaction::StatusDownloadPackagelist:
        case PackageKit::Transaction::StatusDownloadFilelist:
        case PackageKit::Transaction::StatusDownloadChangelog:
        case PackageKit::Transaction::StatusDownloadGroup:
        case PackageKit::Transaction::StatusDownloadUpdateinfo:
        case PackageKit::Transaction::StatusRepackaging:
        case PackageKit::Transaction::StatusLoadingCache:
        case PackageKit::Transaction::StatusScanApplications:
        case PackageKit::Transaction::StatusGeneratePackageList:
        case PackageKit::Transaction::UnknownStatus:
            break;
        case PackageKit::Transaction::StatusWaitingForLock:
        case PackageKit::Transaction::StatusWaitingForAuth:
            d->backendCachedState = backend()->state();
            d->waiting = true;
            setState(Backend::WaitingForLock);
            kDebug() << "Waiting for a lock on the backend";
            break;
        case PackageKit::Transaction::StatusScanProcessList:
        case PackageKit::Transaction::StatusCheckExecutableFiles:
        case PackageKit::Transaction::StatusCheckLibraries:
        //case PackageKit::Transaction::StatusCopyFiles:
            qDebug() << "Other status " << status;
            break;
        default:
            break;
    }

    if (d->waiting && status != PackageKit::Transaction::StatusWaitingForLock &&
        status != PackageKit::Transaction::StatusWaitingForLock) {
        setState(d->backendCachedState);
        d->waiting = false;
    }
}

bool PKPlugin::cancelQueueProcessing()
{
    return false;
}

void PKPlugin::transactionQuestionAnswered(Shaman::TransactionQuestion *question)
{
    QString task = d->questions[question->id()];
    if (task == "restart" && question->answer() == TransactionQuestion::AnswerYes) {
        // Restart the system
        KWorkSpace::requestShutDown(KWorkSpace::ShutdownConfirmDefault, KWorkSpace::ShutdownTypeReboot,
                                    KWorkSpace::ShutdownModeDefault);
    } else if (task == "eula") {
        if (question->answer() == TransactionQuestion::AnswerUser1) {
            // Accept the EULA
            PackageKit::Client::instance()->acceptEula(d->lastEula);
        }
        // TODO: Now restart the transaction
    } else if (task == "media") {
        // TODO: Now restart the transaction
    }
}

void PKPlugin::slotCurrentPackageChanged(PackageKit::Package* package)
{
    if (d->nativePackagesHash.contains(package->id())) {
        d->currentTransactionPackage = d->nativePackagesHash[package->id()];
    } else {
        // Some packages (like updates, to name one) might have a different ID.
        // We have a name based index not for nothing, though :)
        d->currentTransactionPackage = d->nameToPackageHash[package->name()];
    }
}

void PKPlugin::errorDetected(PackageKit::Client::DaemonError error)
{
    setTransactionError(Shaman::TransactionInterface::BackendError, PKStringsAndIcons::daemonErrorToString(error));
}

Shaman::AbstractPackage::List PKPlugin::pkPackageListToList(QList< PackageKit::Package* > packages)
{
    Shaman::AbstractPackage::List retlist;
    foreach (PackageKit::Package *p, packages) {
        if (d->nativePackagesHash.contains(p->id())) {
            retlist.append(d->nativePackagesHash[p->id()]);
        }
    }

    return retlist;
}

void PKPlugin::waitForTransactionFinished(PackageKit::Transaction* transaction)
{
    QEventLoop e;
    connect(transaction, SIGNAL(finished(PackageKit::Transaction::ExitStatus, uint)), &e, SLOT(quit()));
    e.exec();
}

void PKPlugin::setIdle()
{
    setState(Backend::Idling);
}

void PKPlugin::stopBackgroundTransactions()
{
    d->stopBgTransactions = true;

    foreach (PackageKit::Transaction *t, PackageKit::Client::instance()->getTransactions()) {
        QEventLoop e;
        connect(t, SIGNAL(finished(PackageKit::Transaction::ExitStatus,uint)),
                &e, SLOT(quit()));

        if (t->allowCancel()) {
            t->cancel();
        }

        e.exec();
    }
}

void PKPlugin::connectTransaction(PackageKit::Transaction* transaction)
{
    d->currentTransactionPackage = 0;
    connect(transaction, SIGNAL(statusChanged(PackageKit::Transaction::Status)),
            this, SLOT(reportStatus(PackageKit::Transaction::Status)));
    connect(transaction,SIGNAL(requireRestart(PackageKit::Client::RestartType,Package*)),
            this, SLOT(slotRequireRestart(PackageKit::Client::RestartType,Package*)));
    connect(transaction, SIGNAL(eulaRequired(PackageKit::Client::EulaInfo)),
            this, SLOT(slotEulaRequired(PackageKit::Client::EulaInfo)));
    connect(transaction, SIGNAL(mediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)),
            this, SLOT(slotMediaChangeRequired(PackageKit::Transaction::MediaType,QString,QString)));
    connect(transaction, SIGNAL(message(PackageKit::Client::MessageType,QString)),
            this, SLOT(slotMessage(PackageKit::Client::MessageType,QString)));
    connect(transaction, SIGNAL(errorCode(PackageKit::Client::ErrorType,QString)),
            this, SLOT(slotErrorCode(PackageKit::Client::ErrorType,QString)));
}

void PKPlugin::slotRequireRestart(PackageKit::Client::RestartType type, Package*)
{
    StringTransactionQuestion *question = 0;
    switch (type) {
        case PackageKit::Client::RestartApplication:
            break;
        case PackageKit::Client::RestartNone:
            break;
        default:
            question = new StringTransactionQuestion(i18n("A restart is necessary to complete the transaction. Would you like to "
                                                          "restart the system now?<br /><i>Note: you will not be able to perform "
                                                          "other transactions until this has been done.</i>"),
                                                     TransactionQuestion::AnswerYes | TransactionQuestion::AnswerNo,
                                                     KIcon("system-reboot"), false);

            d->questions[question->id()] = "restart";
            newTransactionQuestion(question);
            break;
    }
}

void PKPlugin::slotEulaRequired(PackageKit::Client::EulaInfo info)
{
    QString txt = i18n("The package %1 requires accepting the following EULA from %2",
                       info.package->name(), info.vendorName);

    TransactionQuestion *question = new TransactionQuestion(new EulaWidgetFactory(txt, info.licenseAgreement),
                                                            TransactionQuestion::AnswerUser1 | TransactionQuestion::AnswerUser2);
    question->setUser1Icon(KIcon("dialog-ok-apply"));
    question->setUser1Text(i18n("Accept"));
    question->setUser2Icon(KIcon("dialog-cancel"));
    question->setUser2Text(i18n("Decline"));

    d->questions[question->id()] = "eula";
    d->lastEula = info;

    newTransactionQuestion(question);
}

void PKPlugin::slotMediaChangeRequired(PackageKit::Transaction::MediaType type, const QString &id, const QString &text)
{
    QIcon icon = PKStringsAndIcons::iconForMediaType(type);
    StringTransactionQuestion *question = new StringTransactionQuestion(i18n("The medium labeled '%1' must be inserted to "
                                                                             "continue the transaction.", text),
                                                                        TransactionQuestion::AnswerYes |
                                                                        TransactionQuestion::AnswerCancel, icon);
    d->questions[question->id()] = "media";

    newTransactionQuestion(question);
}

void PKPlugin::slotMessage(PackageKit::Client::MessageType type, const QString &msg)
{
    addTransactionMessage(i18nc("%1 is the generic message, %2 the detailed one", "%1, %2",
                                PKStringsAndIcons::stringForMessage(type), msg));
}

void PKPlugin::slotErrorCode(PackageKit::Client::ErrorType error, const QString &details)
{
    QPair<Shaman::TransactionInterface::Error, QString> repr = PKStringsAndIcons::stringForError(error, details);
    setTransactionError(repr.first, repr.second);
}

void PKPlugin::slotStateChanged(Shaman::Backend::State newstate)
{
    if (newstate == Shaman::Backend::Idling && d->stopBgTransactions) {
        d->stopBgTransactions = false;
        // Try restarting. If it's already over, it should just pass by
        if (backend()->packageGroups().isEmpty()) {
            reloadNextPackageGroup();
        }
    }
}

Q_EXPORT_PLUGIN2(shaman_backend, PKPlugin)

#include "PKPlugin.moc"
