/* This file is part of the KDE project

   Copyright © 2010 David Palacio <dpalacio@uninorte.edu.co>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "AptBackend.h"
#include "DebPackage.h"

#include <kicon.h>
#include <kdebug.h>

#include "shamantransactioninterface.h"
#include "shamanquery.h"
#include <QTimer>

AptBackend::AptBackend()
    : Shaman::BackendInterface()
{
    m_backend = new QApt::Backend();
}

AptBackend::~AptBackend()
{
    delete m_backend;
}

void AptBackend::transactionQuestionAnswered(Shaman::TransactionQuestion* question)
{

}

void AptBackend::init()
{
    setCapabilities(/*Transaction capabilities*/
                    Shaman::BackendCapabilities::InstallPackagesCapability |
                    Shaman::BackendCapabilities::RemovePackagesCapability |
                    Shaman::BackendCapabilities::UpgradePackagesCapability |
                    Shaman::BackendCapabilities::UpdateRepositoriesCapability |
                    Shaman::BackendCapabilities::UpgradeSystemCapability,
                    /*Shaman::BackendCapabilities::DistUpgradeCapability |
                    Shaman::BackendCapabilities::InstallPackageFromFileCapability,*/
                    /*Search capabilities*/
                    Shaman::BackendCapabilities::BasicSearchFromBackendCapability,
                    /*Package Grouping capabilities*/
                    Shaman::BackendCapabilities::GroupPackagesByPackageGroupsCapability,
                    //Shaman::BackendCapabilities::GroupPackagesByRepositoryCapability,
                    /*Information capabilities*/
                    Shaman::BackendCapabilities::ShowRequiredByCapability |
                    Shaman::BackendCapabilities::ShowFileListCapability);

    Shaman::ProcessQueueOption::List options;
    options << new Shaman::ProcessQueueOption(i18n("Purge"),
                                              i18n("Purge configuration files when removing packages. "
                                                   "Valuable data might be lost."),
                                              QVariant("purge"));
    setAvailableProcessQueueOptions(options);
    KAboutPerson person1(ki18n("David Palacio"), ki18n("Maintainer"), "dpalacio@uninorte.edu.co", "http://bakalacio.blogspot.com/");
    KAboutPerson person2(ki18n("Jonathan Thomas"), ki18n("Maintainer"), "echidnaman@kubuntu.org");
    setProperties(i18n("QApt backend"), i18n("A backend using the LibQApt library"), KIcon("applications-other"),
                  KIcon("application-x-deb"), QList<KAboutPerson>() << person1 << person2, QStringList());
    setState(Shaman::Backend::Initializing);
    m_backend->init();

    connect(m_backend, SIGNAL(errorOccurred(QApt::ErrorCode, const QVariantMap&)),
            this, SLOT(slotErrorOccurred(QApt::ErrorCode, const QVariantMap&)));
    connect(m_backend, SIGNAL(packageChanged()),
            this, SLOT(refreshAdditionalTargets()));
    connect(m_backend, SIGNAL(workerEvent(QApt::WorkerEvent)),
            this, SLOT(slotWorkerEvent(QApt::WorkerEvent)));
    connect(m_backend, SIGNAL(downloadProgress(int, int, int)),
            this, SLOT(slotSetTransactionProgress(int, int, int)));
    connect(m_backend, SIGNAL(downloadMessage(int, const QString&)),
            this, SLOT(slotSetOperationDescription(int, const QString&)));
    setState(Shaman::Backend::Idling);
}

bool AptBackend::aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate)
{
    if (((oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||
         (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) &&
        backend()->operationMode() == Shaman::Base::CachedMode)
    {
        // Time to reload, guys.
        QTimer::singleShot(0, this, SLOT(reload()));
        // If we're initializing, just don't reject the state transition
        if (oldstate != Shaman::Backend::Initializing)
        {
            // Otherwise do.
            return false;
        }
    }

    // Else we're fine with defaults
    return Shaman::BackendInterface::aboutToChangeState(oldstate, newstate);
}

void AptBackend::processQueue(const Shaman::AbstractPackage::List& packages, const Shaman::ProcessQueueOption::List &options)
{
    bool purge;

    foreach (Shaman::ProcessQueueOption * const option, options) {
        if (option->name() == "Purge") {
            purge = true;
        }
    }

    foreach (Shaman::AbstractPackage *package, packages) {
        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
            case Shaman::AbstractPackage::Upgrade:
                m_backend->markPackageForInstall(package->name());
                break;
            case Shaman::AbstractPackage::Uninstall:
                // FIXME: Also handle purge here
                m_backend->markPackageForRemoval(package->name());
                break;
            default:
                break;
        }
    }

    kDebug() << "asking to commit changes";
    m_backend->commitChanges();
}

void AptBackend::searchPackages(Shaman::Query *query)
{
    QApt::PackageList searchResults;
    Shaman::AbstractPackage::List list;

    searchResults = m_backend->search(query->query().pattern());

    foreach (QApt::Package *pkg, searchResults) {
        list << findPackageByName(pkg->name());
    }

    searchCompleted(query->id(), list);
    setState(Shaman::Backend::Idling);
}

bool AptBackend::cancelQueueProcessing()
{
// TODO: implement in libqapt
//     m_backend->cancelOperationUpdate();
//     setState(Shaman::Backend::Idling);
// 
    return false;
}

void AptBackend::slotErrorOccurred(QApt::ErrorCode code, const QVariantMap& args)
{
    kDebug() << args;
    QString message;
    Shaman::TransactionInterface::Error error = Shaman::TransactionInterface::UnknownError;

    if (code == QApt::InitError) {
        message = i18n("Shaman was unable to initialize the package system. "
                       "Your configuration may be broken.");
    } else if (code == QApt::LockError) {
        message = i18n("Another package manager seems to be using the package "
                       "system at this time. You must close all other package "
                       "managers before you will be able to install, remove or "
                       "upgrade any packages.");
    } else if (code == QApt::DiskSpaceError) {
        QString drive = args["DirectoryString"].toString();
        kDebug() << drive;
        message = i18n("Shaman has detected that you do not have enough disk "
                       "space in the directory at %1 to continue with this "
                       "operation.", drive);
    } else if (code == QApt::FetchError) {
        //FIXME: Not necessarily fatal, should be a warning
        message = i18n("An error occurred while fetching packages: ");

        QString failedItem = args["FailedItem"].toString();
        QString errorText = args["ErrorText"].toString();
        if (!failedItem.isEmpty() && !errorText.isEmpty()) {
             message.append("\n\n");
             message.append(failedItem);
             message.append('\n');
             message.append(errorText);
        }
    } else if (code == QApt::CommitError) {
        message = i18n("An error occurred while committing changes:");

        QString errorText = args["ErrorText"].toString();
        if (!errorText.isEmpty()) {
             message.append("\n\n");
             message.append(errorText);
        }
    } else if (code == QApt::AuthError) {
        message = i18n("Shaman will not be able to continue this operation "
                       "since authorization was not provided.");
    } else if (code == QApt::WorkerDisappeared) {
        message = i18n("It looks like Shaman's internal helper crashed or disappeared. "
                       "This is a bug with the QApt library. Please report this crash "
                       "to the QApt developers.");
    } else if (code == QApt::UntrustedError) {
        QStringList untrustedItems = args["UntrustedItems"].toStringList();
        if (untrustedItems.size() == 1) {
            message = i18n("The %1 package has not been verified by its author. "
                           "Downloading untrusted packages has been disallowed "
                           "by your current configuration.",
                           untrustedItems.first());
        } else if (untrustedItems.size() > 1) {
            QString failedItemString;
            foreach (const QString &string, untrustedItems) {
                failedItemString.append("- " + string + '\n');
            }
            message = i18n("The following packages have not been verified by "
                        "their authors:"
                        "\n%1\n"
                        "Downloading untrusted packages has "
                        "been disallowed by your current configuration.",
                        failedItemString);
        }
    }

    setTransactionError(error, message);
}

void AptBackend::slotWorkerEvent(QApt::WorkerEvent event)
{
    switch (event) {
        case QApt::CacheUpdateStarted:
            setTransactionState(Shaman::TransactionInterface::Downloading);
            break;
        case QApt::CacheUpdateFinished:
            setState(Shaman::Backend::Idling);
            break;
        case QApt::PackageDownloadStarted:
            setTransactionState(Shaman::TransactionInterface::Downloading);
            break;
        case QApt::PackageDownloadFinished:
            break;
        case QApt::CommitChangesStarted:
            break;
        case QApt::CommitChangesFinished:
            setState(Shaman::Backend::Idling);
    }
}

void AptBackend::updateRepositories()
{
    m_backend->updateCache();
    setTransactionState(Shaman::TransactionInterface::Processing);
}

bool AptBackend::cancelRepositoriesUpdate()
{
    m_backend->cancelDownload();

    return true;
}

void AptBackend::reload()
{
    setState(Shaman::Backend::Reloading);
    setPackages(Shaman::AbstractPackage::List());
    setPackageGroups(Shaman::PackageGroup::List());

    QHash<QString, Shaman::AbstractPackage*> packageList;
    Shaman::PackageGroup::List groupList;

    QApt::PackageList packages = m_backend->availablePackages();

    QHash<QString,Shaman::PackageGroup*> sectionCollection;

    // Populate packages
    foreach (QApt::Package *package, packages) {
        DebPackage *p = new DebPackage(this, package);

        packageList[p->name()] = p;
    }

    foreach (const QApt::Group &group, m_backend->availableGroups()) {
        Shaman::PackageGroup *g = new Shaman::PackageGroup(group, this);
        groupList.append(g);

        foreach (QApt::Package *pkg, packages) {
            if (pkg->section() == group) {
                g->append(packageList[pkg->name()]);
            }
        }
    }

    setPackages(packageList.values());
    setPackageGroups(groupList);

    setState(Shaman::Backend::Idling);
}

Shaman::AbstractPackage *AptBackend::findPackageByName(const QString &name)
{
    Shaman::AbstractPackage *ret = 0;

    foreach (Shaman::AbstractPackage *ent, backend()->packages()) {
        if (ent->name() == name) {
            ret = ent;
            break;
        }
    }

// I don't think the QApt backend leaves any package uncached, since it's always populated by reload
//     if (!ret && backend()->operationMode() == Shaman::Base::UncachedMode) {
//         // We probably need to add it to the cache. Find it out first.
//         Aqpm::Package *package = Aqpm::Backend::instance()->package(name, QString());
//         if (package) {
//             Shaman::AbstractPackage *p = createShamanPackageFrom(package, Aqpm::Backend::instance()->upgradeablePackages());
//             ret = p;
//             packagesRetrieved(Shaman::AbstractPackage::List() << p);
//         }
//     }

    return ret;
}

void AptBackend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    bool purge;

    foreach (Shaman::ProcessQueueOption * const option, options) {
        if (option->name() == "Purge") {
            purge = true;
        }
    }

    foreach (Shaman::AbstractPackage *package, backend()->queuedPackages()) {
        switch (package->type()) {
            case Shaman::AbstractPackage::LocalPackageType:
                break;
            case Shaman::AbstractPackage::BackendPackageType:
                switch (package->action()) {
                    case Shaman::AbstractPackage::Install:
                    case Shaman::AbstractPackage::Upgrade:
                        m_backend->markPackageForInstall(package->name());
                        break;
                    case Shaman::AbstractPackage::Uninstall:
                        // FIXME: support purge here
                        m_backend->markPackageForRemoval(package->name());
                        break;
                    default:
                        break;
                }
                break;
            default:
                kWarning() << "I got a package which is not from local fs or from backend, there's something screwed";
                break;
        }
    }
    setState(Shaman::Backend::Idling);
}

void AptBackend::refreshAdditionalTargets()
{
    kDebug() << "Marking additional targets";
    Shaman::AbstractPackage *p;

    foreach (QApt::Package *backendPackage, m_backend->availablePackages()) {
        int state = backendPackage->state();
        if (state & QApt::Package::ToInstall) {
            p = findPackageByName(backendPackage->name());
            bool manualInstall = (p->action() == Shaman::AbstractPackage::Install);
            bool manualUpgrade = (p->action() == Shaman::AbstractPackage::Upgrade);
            // Currently the only way we can upgrade a single package is by marking
            // it for install. We can show it as an upgrade to the user if we know
            // better, though.
            if (!(manualInstall || manualUpgrade)) {
                p->setAction(Shaman::AbstractPackage::TransactionDependencyInstall);
                if (state & QApt::Package::Upgradeable) {
                    p->setAction(Shaman::AbstractPackage::TransactionDependencyUpgrade);
                }
            } else {
                if (state & QApt::Package::Upgradeable) {
                    p->setAction(Shaman::AbstractPackage::Upgrade);
                }
            }
        } else if (state & QApt::Package::ToRemove) {
            p = findPackageByName(backendPackage->name());
            bool manualRemove = (p->action() == Shaman::AbstractPackage::Uninstall);
            if (!manualRemove) {
                p->setAction(Shaman::AbstractPackage::TransactionDependencyUninstall);
            }
        }
    }
}

void AptBackend::slotSetTransactionProgress(int percent, int speed, int ETA)
{
    manualProgressReporter()->setTransactionProgress(percent);
}

void AptBackend::slotSetOperationDescription(int flag, const QString &message)
{
    QString fullMessage;

    switch(flag) {
      case QApt::DownloadFetch:
          fullMessage = i18n("Downloading: %1", message);
          break;
      case QApt::HitFetch:
          fullMessage = i18n("Checking: %1", message);
          break;
      case QApt::IgnoredFetch:
          fullMessage = i18n("Ignored: %1", message);
          break;
      default:
          fullMessage = message;
    }
    // FIXME: Doesn't put anything in the details pane of the Update Database dialog... shouldn't it do so?
    addTransactionMessage(message);
}

Q_EXPORT_PLUGIN2(shaman_backend, AptBackend)

#include "AptBackend.moc"
