/* This file is part of the KDE project

   Copyright (C) 2008 - 2010 Dario Freddi <drf@kde.org>
                 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "PluginLoader_p.h"

#include <config-shaman.h>
#include "shamanabstractplugin_p.h"
#include "shamancoreplugin_p.h"
#include "shamanproviderplugin.h"
#include "shamanbackend_p.h"
#include "shamanbackendinterface_p.h"
#include "shamanabstractpluginloader.h"
#include <backend-config_p.h>
#ifdef USE_SCRIPTED_BACKEND
#include "scripting/backend/ScriptBackendInterface.h"
#endif

#include <shamanmacros.h>

#include <KServiceTypeTrader>
#include <KPluginInfo>
#include <KMessageBox>
#include <KDebug>
#include <QEventLoop>
#include <QCoreApplication>

#ifndef KDE_USE_FINAL
#ifndef USE_SCRIPTED_BACKEND
Q_IMPORT_PLUGIN(shaman_backend)
#endif
#endif

namespace Shaman {
class PluginLoader::Private
{
    public:
        Private(KSharedConfig::Ptr cfg)
         : config(cfg)
        {
        }
        ~Private()
        {}

        Q_DECLARE_PUBLIC(PluginLoader)
        PluginLoader *q_ptr;

        void initBackend();
        void loadPlugins();
        void cacheServiceList();

        KSharedConfig::Ptr config;
        QList<CorePlugin*> corePlugins;
        QList<Provider*> providerPlugins;
        QHash<QString, KConfigGroup> configInterfaces;
        QHash<QString, AbstractPluginLoader*> pluginLoaders;

        KService::List offers;
};

PluginLoader::PluginLoader(KSharedConfig::Ptr config, QObject * parent)
  : QObject(parent),
    d(new Private(config))
{
    d->q_ptr = this;
    d->cacheServiceList();
}

PluginLoader::~PluginLoader()
{
    delete d;
}

void PluginLoader::registerPluginLoader(AbstractPluginLoader *loader)
{
    d->pluginLoaders.insert(loader->pluginType(), loader);
}

void PluginLoader::Private::cacheServiceList()
{
    offers = KServiceTypeTrader::self()->query("Shaman/Plugin");
}

void PluginLoader::Private::loadPlugins()
{
    Q_Q(PluginLoader);

    KConfigGroup plugins(config, "Plugins");
    foreach (KService::Ptr offer, offers) {
        KPluginInfo info(offer);
        info.load(plugins);

        QString configClass = offer->property("X-KDE-Shaman-plugin-config-class").toString();
        if (!configClass.isEmpty()) {
            configInterfaces.insert(configClass,
                                    KConfigGroup(Base::instance()->configurationFile(), offer->desktopEntryName()));
        }

        if( !info.isPluginEnabled() ) {
            kDebug() << offer->desktopEntryName() << "is not enabled";
            continue;
        }

        QString ptype = offer->property("X-KDE-Shaman-plugin-type").toString();

        // Do we have a loader?
        if (!pluginLoaders.contains(ptype)) {
            kWarning() << "No loaders available for type " << ptype;
            continue;
        }

        AbstractPluginLoader *loader = pluginLoaders[ptype];
        AbstractPlugin *plugin = loader->loadInstance(offer, q->parent());

        if (!plugin) {
            kWarning() << offer->desktopEntryName() << "failed to load";
            continue;
        }

        // Now initialize
        plugin->d_func()->setPluginName(offer->name());
        plugin->init();

        // Check if it's a core or a provider plugin
        CorePlugin *corePlugin = qobject_cast< Shaman::CorePlugin* >(plugin);
        if (corePlugin) {
            corePlugins.append(corePlugin);
        }

        ProviderPlugin *providerPlugin = qobject_cast< Shaman::ProviderPlugin* >(plugin);
        if (providerPlugin) {
            providerPlugins.append(providerPlugin->provider());
        }

        emit q->pluginCreated(plugin);
    }

    kDebug() << "Plugins loaded. " << corePlugins.count() << "core, " << providerPlugins.count()
             << "provider";
}

QList< Provider* > PluginLoader::providers() const
{
    return d->providerPlugins;
}

void PluginLoader::loadPlugins()
{
    // Load backend first
    d->initBackend();

    // Then all the rest
    d->loadPlugins();

    // Done
    emit pluginsLoaded();
}

void PluginLoader::Private::initBackend()
{
    Q_Q(PluginLoader);

    BackendInterface *backend = 0;

#ifndef USE_SCRIPTED_BACKEND
    QObjectList l = QPluginLoader::staticInstances();
    foreach(QObject *o, l) {
        backend = qobject_cast<BackendInterface *>(o);
        if (backend) {
            kDebug() << "BackendInterface successfully Initialized";
            break;
        }
    }
#else
    ScriptBackend * scriptPlugin = new ScriptBackend(q->parent(), QVariantList());
    scriptPlugin->loadScript(QString(BACKEND_PATH));
    kDebug(5001) << "Backend-Plugin script" << BACKEND_PATH;
    backend = scriptPlugin;
#endif

    if (!backend) {
        // We would end up crashing stuff. Display a meaningful messagebox and flee
        KMessageBox::error(0, i18n("The package management backend could not be loaded. Please check your installation"));
        QCoreApplication::instance()->quit();
        return;
    }

    // Create the backend and inject into it the interface
    Backend *b = new Backend(backend, q);
    
    emit q->backendPluginCreated(b);//We need to emit this before initialization, as we access Base::instance()->core()->backend() during it
    // Now trigger initialization
    QEventLoop e;
    backend->setState(Shaman::Backend::Initializing);
    connect(b, SIGNAL(stateChanged(Shaman::Backend::State)), &e, SLOT(quit()));

    QMetaObject::invokeMethod(backend, "init", Qt::QueuedConnection);
    kDebug() << "Backend init routine started";
    // Wait for the backend to change state
    while (b->state() != Shaman::Backend::Idling) {
        e.exec();
    }

    kDebug() << "Backend init routine ended";
}

QHash< CorePlugin*, const char* > PluginLoader::hooksFor(CorePlugin::Hook hook)
{
    QHash< CorePlugin*, const char* > rethash;
    foreach (CorePlugin *plugin, d->corePlugins) {
        if (plugin->d_func()->memberForHook(hook)) {
            rethash.insert(plugin, plugin->d_func()->memberForHook(hook));
        }
    }

    return rethash;
}
KConfigGroup PluginLoader::configGroupFor(const QString &interface) const
{
    kDebug() << "Checking interface " << interface << ", I have " << d->configInterfaces.keys();
    if (d->configInterfaces.contains(interface)) {
        return d->configInterfaces[interface];
    } else {
        return KConfigGroup();
    }
}

}

#include "PluginLoader_p.moc"
