/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanabstractpackage_p.h"
#include "shamanmetapackage.h"
#include "shamanglobalpackagewatcher.h"
#include "shamanpackagewatcher_p.h"

#include <KDebug>

namespace Shaman {

AbstractPackage::AbstractPackage(AbstractPackagePrivate& dd, QObject* parent)
    : d_ptr(&dd)
{
    qRegisterMetaType<AbstractPackage*>();
    qRegisterMetaType<Shaman::AbstractPackage::List>();
    qRegisterMetaType<Shaman::AbstractPackage::Action>("Shaman::AbstractPackage::Action");
    qRegisterMetaType<Shaman::AbstractPackage::State>("Shaman::AbstractPackage::State");
}

AbstractPackage::~AbstractPackage()
{
    foreach (MetaPackage *package, metaPackages()) {
        package->removePackage(this);
    }
    // Notify any watchers around to detach
    foreach (PackageWatcher *watcher, d_func()->watchers) {
        watcher->d->detachPackage();
    }
    delete d_ptr;
}

QString AbstractPackage::name() const
{
    Q_D(const AbstractPackage);
    return d->name;
}

AbstractPackage::Type AbstractPackage::type() const
{
    Q_D(const AbstractPackage);
    return d->type;
}

MetaPackage::List AbstractPackage::metaPackages() const
{
    Q_D(const AbstractPackage);
    return d->metapackages;
}

void AbstractPackage::setIcon(const QIcon& icon)
{
    Q_D(AbstractPackage);
    d->icon = icon;
}

QIcon AbstractPackage::icon() const
{
    Q_D(const AbstractPackage);
    return d->icon;
}

AbstractPackage::Properties AbstractPackage::properties() const
{
    Q_D(const AbstractPackage);
    return d->properties;
}

void AbstractPackage::setProperties(AbstractPackage::Properties properties)
{
    Q_D(AbstractPackage);
    d->properties = properties;
}

void AbstractPackage::setDescription(const QString &desc)
{
    Q_D(AbstractPackage);
    d->description = desc;
    if (d->values & Description) {
        d->values ^= Description;
        EMIT_VALUE_CACHED(Description);
    }
}

QString AbstractPackage::description()
{
    Q_D(AbstractPackage);
    if (d->values & Description) {
        cacheValue(Description);
    }
    return d->description;
}

void AbstractPackage::setVersion(const QString &version)
{
    Q_D(AbstractPackage);
    d->version = version;
}

QString AbstractPackage::version() const
{
    Q_D(const AbstractPackage);
    return d->version;
}

void AbstractPackage::setAvailableVersion(const QString &version)
{
    Q_D(AbstractPackage);
    d->availableVersion = version;
}

QString AbstractPackage::availableVersion() const
{
    Q_D(const AbstractPackage);
    if (d->availableVersion.isEmpty()) {
        return d->version;
    }
    return d->availableVersion;
}

void AbstractPackage::setSize(KIO::filesize_t size)
{
    Q_D(AbstractPackage);
    d->size = size;
    if (d->values & Size) {
        d->values ^= Size;
        EMIT_VALUE_CACHED(Size);
    }
}

KIO::filesize_t AbstractPackage::size()
{
    Q_D(AbstractPackage);
    if (d->values & Size) {
        cacheValue(Size);
    }
    return d->size;
}

void AbstractPackage::setMaintainer(const QString &name)
{
    Q_D(AbstractPackage);
    d->maintainer = name;
    if (d->values & Maintainer) {
        d->values ^= Maintainer;
        EMIT_VALUE_CACHED(Maintainer);
    }
}

QString AbstractPackage::maintainer()
{
    Q_D(AbstractPackage);
    if (d->values & Maintainer) {
        cacheValue(Maintainer);
    }
    return d->maintainer;
}

void AbstractPackage::setHomepage(const KUrl &url)
{
    Q_D(AbstractPackage);
    d->homepage = url;
    if (d->values & Homepage) {
        d->values ^= Homepage;
        EMIT_VALUE_CACHED(Homepage);
    }
}

KUrl AbstractPackage::homepage()
{
    Q_D(AbstractPackage);
    if (d->values & Homepage) {
        cacheValue(Homepage);
    }
    return d->homepage;
}

void AbstractPackage::setLicense(const QString &license)
{
    Q_D(AbstractPackage);
    d->license = license;
    if (d->values & License) {
        d->values ^= License;
        EMIT_VALUE_CACHED(License);
    }
}

QString AbstractPackage::license()
{
    Q_D(AbstractPackage);
    if (d->values & License) {
        cacheValue(License);
    }
    return d->license;
}

void AbstractPackage::setState(State state)
{
    Q_D(AbstractPackage);
    if (d->state == state) {
        return;
    }

    d->state = state;

    foreach(PackageWatcher *watcher, d->watchers) {
        emit watcher->stateChanged(d->state);
    }
    emit GlobalPackageWatcher::instance()->stateChanged(this, d->state);
}

Shaman::AbstractPackage::State AbstractPackage::state() const
{
    Q_D(const AbstractPackage);
    return d->state;
}

void AbstractPackage::setAction(const Action &action)
{
    Q_D(AbstractPackage);
    if (d->action == action || (action == Install && d->state == Installed)
                            || (action == Uninstall && d->state == NotInstalled)) {
        return;
    }

    d->action = action;
    foreach(PackageWatcher *watcher, d->watchers) {
        emit watcher->actionChanged(d->action);
    }
    emit GlobalPackageWatcher::instance()->actionChanged(this, d->action);
}

Shaman::AbstractPackage::Action AbstractPackage::action() const
{
    Q_D(const AbstractPackage);
    return d->action;
}

bool AbstractPackage::isProviderInstalled()
{
    foreach (AbstractPackage *package, providers())
    {
        if (package->state() == AbstractPackage::Installed)
            return true;
    }
    return false;
}

Shaman::AbstractPackage::List AbstractPackage::providers()
{
    Q_D(AbstractPackage);
    if (d->values & Providers) {
        cacheValue(Providers);
    }
    return d->providers;
}

Shaman::AbstractPackage::List AbstractPackage::dependencies()
{
    Q_D(AbstractPackage);
    if (d->values & Dependencies) {
        cacheValue(Dependencies);
    }
    return d->dependencies;
}

Shaman::AbstractPackage::List AbstractPackage::requiredBy()
{
    Q_D(AbstractPackage);
    if (d->values & RequiredBy) {
        cacheValue(RequiredBy);
    }
    return d->dependenciesOnPackage;
}

Shaman::AbstractPackage::List AbstractPackage::conflictsWith()
{
    Q_D(AbstractPackage);
    if (d->values & ConflictsWith) {
        cacheValue(ConflictsWith);
    }
    return d->conflicts;
}

QStringList AbstractPackage::fileList()
{
    Q_D(AbstractPackage);
    if (d->values & FileList) {
        cacheValue(FileList);
    }
    return d->fileList;
}

void AbstractPackage::setConflictsWith(const Shaman::AbstractPackage::List& packages)
{
    Q_D(AbstractPackage);
    d->conflicts = packages;
    if (d->values & ConflictsWith) {
        d->values ^= ConflictsWith;
        EMIT_VALUE_CACHED(ConflictsWith);
    }
}

void AbstractPackage::setDependencies(const Shaman::AbstractPackage::List& packages)
{
    Q_D(AbstractPackage);
    d->dependencies = packages;
    if (d->values & Dependencies) {
        d->values ^= Dependencies;
        EMIT_VALUE_CACHED(Dependencies);
    }
}

void AbstractPackage::setProviders(const Shaman::AbstractPackage::List& packages)
{
    Q_D(AbstractPackage);
    d->providers = packages;
    if (d->values & Providers) {
        d->values ^= Providers;
        EMIT_VALUE_CACHED(Providers);
    }
}

void AbstractPackage::setRequiredBy(const Shaman::AbstractPackage::List& packages)
{
    Q_D(AbstractPackage);
    d->dependenciesOnPackage = packages;
    if (d->values & RequiredBy) {
        d->values ^= RequiredBy;
        EMIT_VALUE_CACHED(RequiredBy);
    }
}

void AbstractPackage::setFileList(const QStringList& files)
{
    Q_D(AbstractPackage);
    d->fileList = files;
    if (d->values & FileList) {
        d->values ^= FileList;
        EMIT_VALUE_CACHED(FileList);
    }
}

void AbstractPackage::setChangelog(const QString& changelog)
{
    Q_D(AbstractPackage);
    d->changelog = changelog;
    if (d->values & Changelog) {
        d->values ^= Changelog;
        EMIT_VALUE_CACHED(Changelog);
    }
}

QString AbstractPackage::changelog()
{
    Q_D(AbstractPackage);
    if (d->values & Changelog) {
        cacheValue(Changelog);
    }
    return d->changelog;
}

AbstractPackage::UpdateType AbstractPackage::updateType() const
{
    Q_D(const AbstractPackage);
    return d->updateType;
}

void AbstractPackage::setUpdateType(AbstractPackage::UpdateType type)
{
    Q_D(AbstractPackage);
    d->updateType = type;
}

void AbstractPackage::setUnCachedValues(Shaman::AbstractPackage::CachedValues values)
{
    Q_D(AbstractPackage);
    d->values = values;
}

AbstractPackage::CachedValues AbstractPackage::cachedValues() const
{
    Q_D(const AbstractPackage);
    return ~(d->values);
}

void AbstractPackage::addCustomValue(const QString& name, WidgetFactory *factory)
{
    Q_D(AbstractPackage);
    d->customValues.insert(name, factory);
    d->values ^= CustomValues;
    EMIT_VALUE_CACHED(CustomValues);
}

void AbstractPackage::removeCustomValue(const QString& name)
{
    Q_D(AbstractPackage);
    d->customValues.remove(name);
    EMIT_VALUE_CACHED(CustomValues);
}

QHash< QString, WidgetFactory* > AbstractPackage::customValues()
{
    Q_D(AbstractPackage);
    if (d->values & CustomValues) {
        cacheValue(CustomValues);
    }

    return d->customValues;
}

}
