/* This file is part of the KDE project

   Copyright (C) 2008-2009 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef ABSTRACTPACKAGE_H
#define ABSTRACTPACKAGE_H

#include <QtCore/QObject>
#include "shamanmacros.h"
#include <KUrl>
#include <kio/global.h>

namespace Shaman {

class PackageGroup;

class Repository;

class WidgetFactory;
class MetaPackage;

class AbstractPackagePrivate;
/**
 * \class AbstractPackage Shaman/AbstractPackage
 *
 * \brief The base class for packages in LibShaman
 *
 * \c AbstractPackage is the base class for packages in LibShaman. You are not interested in using this
 * class directly, since it's completely abstract and inheritable just privately: you probably want to
 * use one of its abstract superclasses, such as \c ProviderPackage, \c BackendPackage or \c LocalPackage.
 *
 * AbstractPackage carries a lot of information common to all packages, such as name, size, a set of properties,
 * which for example define if a package is debug information, and more. The most interesting feature in
 * AbstractPackage is its native caching support
 *
 * \par Caching support in AbstractPackage
 *
 * Many backends are not able to provide all the information AbstractPackage and its superclasses are able to
 * expose quickly: for this reason, AbstractPackage implements an easy to use caching system, which is completely
 * transparent and requires almost no additional work on the programmer side, both if he's implementing a superclass
 * or using LibShaman in an application.
 *
 * \par Implementing caching support in an AbstractPackage superclass
 *
 * The key functions to caching support are two functions: one is \c setUnCachedValues, which you will have to
 * call in the constructor of your package, and the other is \c cacheValue, which you will have to reimplement.
 * Let's see a small example where dependencies and Changelog are uncached.
 *
 * \code
 * MyPackage::MyPackage([...])
 * {
 *   [...]
 *   setUnCachedValues(Dependencies | ChangeLog);
 *   [...]
 * }
 *
 * void MyPackage::cacheValue(CachedValue value)
 * {
 *   switch (value) {
 *       case Dependencies:
 *           [...] // Find out the dependencies
 *           setDependencies(<what you found out>);
 *           break;
 *       case Changelog:
 *           // The changelog is retrieved asynchronously
 *           connect(this, SIGNAL(changeLogRetrieved(QString)), this, SLOT(slotChangeLogRetrieved(QString)));
 *           break;
 *       default:
 *           // We're not interested in anything else
 *           break;
 *   }
 * }
 *
 * void MyPackage::slotChangeLogRetrieved(const QString &changelog)
 * {
 *   setChangelog(changelog);
 * }
 * \endcode
 *
 * As you can see, the caching operation is completely transparent and happens through the set* operators, which
 * will take care of populating the cache accordingly and eventually emit \c valueCached.
 *
 * LibShaman frontend developers should listen to valueCached if they're interested in knowing when a value will
 * get cached. \c cacheValue can be called explicitly, but will also be triggered whenever one requests an uncached
 * property. For example, calling \c dependencies() the first time in the previous example would trigger cacheValue.
 * However, due to the nature of the caching system, it's possible that the value returned would be empty, so you
 * should take care of updating your visualization every time \c valueCached is called. Or you can monitor what is
 * already cached through \c cachedValues.
 *
 * \par The custom property system
 *
 * Many times, backends are able to provide custom information for their packages: for this reason, \c AbstractPackage
 * implements a custom values system through \c WidgetFactory. Backends can define their own widgets for advanced
 * properties (a good use case is the package's action history), which will be eventually exposed by the frontend.
 *
 * @see BackendPackage
 * @see ProviderPackage
 * @see LocalPackage
 * @see WidgetFactory
 */
class SHAMAN_EXPORT AbstractPackage
{
    Q_DECLARE_PRIVATE(AbstractPackage)
    public:
        /**
         * \enum State
         *
         * This Enum describes the State of the Backend.
         * @see setState()
         * @see state()
         */
        enum State {
            Installed,
            NotInstalled,
            Upgradeable
        };
        /**
         * \enum Action
         *
         * This Enum describes the Action which is set to the Package.
         * @see setAction()
         * @see action()
         */
        enum Action {
            NoAction,
            Install,
            Uninstall,
            Upgrade,
            TransactionDependencyInstall,
            TransactionDependencyUninstall,
            TransactionDependencyUpgrade
        };

        enum CachedValue {
            NoValue = 0,
            Description = 1,
            Size = 2,
            Maintainer = 4,
            Homepage = 8,
            License = 16,
            Providers = 32,
            Dependencies = 64,
            RequiredBy = 128,
            ConflictsWith = 256,
            FileList = 512,
            Changelog = 1024,
            AdditionalValue1 = 2048,
            AdditionalValue2 = 4096,
            AdditionalValue3 = 8192,
            AdditionalValue4 = 16384,
            CustomValues = 32768
            // There's no room for anything else
        };
        Q_DECLARE_FLAGS(CachedValues, CachedValue)
        Q_FLAGS(CachedValues)

        enum UpdateType {
            LowPriorityUpdate = 0,
            FeatureEnhancementUpdate = 1,
            StandardUpdate = 2,
            BugFixUpdate = 3,
            ImportantUpdate = 4,
            SecurityUpdate = 5,
        };

        enum Type {
            AbstractType = 0,
            BackendPackageType = 1,
            LocalPackageType = 2,
            ProviderPackageType = 3,
            MetaPackageType = 4
            // Add new package types here as they roll
        };

        enum Property {
            NoProperties = 0,
            DebugInformationProperty = 1,
            UnstableProperty = 2,
            DevelopmentPackageProperty = 4
        };
        Q_DECLARE_FLAGS(Properties, Property)

        /**
         * \class List Package.h
         *
         * This class is a typedef for a QList<Package*>.
         */
        typedef QList<AbstractPackage*> List;
        virtual ~AbstractPackage();

        QHash<QString, WidgetFactory*> customValues();

        QList<MetaPackage*> metaPackages() const;

    public:
        /**
         * @returns the package-name
         */
        QString name() const;

        virtual QIcon icon() const;

        /**
         * @returns the Type
         */
        Type type() const;

        /**
         * @returns the Properties
         */
        Properties properties() const;

        /**
         * @returns the Description of the package
         */
        QString description();

        /**
         * @returns a list of all PackageGroups which are containing this package
         */
        virtual QList<Shaman::PackageGroup*> packageGroups() = 0;

        /**
         * @returns a list of all Repositories which are containing this package
         */
        virtual QList<Shaman::Repository*> repositories() = 0;

        /**
         * @returns the Version of the Package
         */
        QString version() const;

        /**
         * @returns the available Version of the Package
         */
        QString availableVersion() const;

        /**
         * @returns the Size of the Package
         */
        KIO::filesize_t size();

        /**
         * @returns the Package-Maintainer of the Package
         */
        QString maintainer();

        /**
         * @returns the homepage of the Package
         */
        KUrl homepage();

        /**
         * @returns the license of the Package
         */
        QString license();

        /**
         * Sets the State of the Package
         * @param state the new state
         */
        void setState(State state);

        /**
         * @returns the State of the Package
         */
        State state() const;

        /**
         * Sets the action for the Package
         * @param action the new Action of the Package
         * @example setAction(Package::Install) adds the package to the Install-Queue
         */
        void setAction(const Action &action);

        /**
         * @returns the current Action of the Package
         */
        Action action() const;

        /**
         * @returns true if a Provider for this Package is already installed
         */
        bool isProviderInstalled();

        /**
         * @returns a list of all Providers for the Package
         */
        List providers();

        /**
         * @returns a list of Dependencies of the Package
         */
        List dependencies();

        /**
         * @returns a list of Dependencies on the Package
         */
        List requiredBy();

        /**
         * @returns a list of conflicting Packages
         */
        List conflictsWith();

        /**
         * @returns the changelog
         */
        QString changelog();

        /**
         * @returns a list of file which are included in the package
         */
        QStringList fileList();

        UpdateType updateType() const;

        virtual void cacheValue(CachedValue value) = 0;

        CachedValues cachedValues() const;

    protected:
        void setUnCachedValues(CachedValues values);

        void setIcon(const QIcon &icon);

        /**
         * Sets the Properties
         * @param properties the new Properties
         */
        void setProperties(Properties properties);

        /**
         * Sets the Description of the Package
         * @param desc the new Description
         */
        void setDescription(const QString &desc);

        /**
         * Sets the Version of the Package
         * @param version the new Version
         */
        void setVersion(const QString &version);

        /**
         * Sets the available Version of the Package
         * @param version the available Version
         */
        void setAvailableVersion(const QString &version);

        /**
         * Set the Size of the Package
         * @param size the Size
         */
        void setSize(KIO::filesize_t size);

        /**
         * Sets the Package-Maintainer
         * @param name the Maintainer
         */
        void setMaintainer(const QString &name);

        /**
         * Sets the Homepage of the Package
         * @param url the new Url
         */
        void setHomepage(const KUrl &url);

        /**
         * Sets the License of the Package
         * @param license the License
         */
        void setLicense(const QString &license);

        /**
         * @returns a list of all Providers for the Package
         */
        void setProviders(const List &packages);

        /**
         * @returns a list of Dependencies of the Package
         */
        void setDependencies(const List &packages);

        /**
         * @returns a list of Dependencies on the Package
         */
        void setRequiredBy(const List &packages);

        /**
         * @returns a list of conflicting Packages
         */
        void setConflictsWith(const List &packages);

        void setFileList(const QStringList &files);

        void setChangelog(const QString &changelog);

        void setUpdateType(UpdateType type);

        void addCustomValue(const QString &name, WidgetFactory* factory);

        void removeCustomValue(const QString &name);

    protected:
        AbstractPackage(AbstractPackagePrivate &dd, QObject *parent = 0);

        AbstractPackagePrivate * const d_ptr;

        friend class MetaPackage;
        friend class PackageWatcher;
};

}

Q_DECLARE_METATYPE(Shaman::AbstractPackage*)
Q_DECLARE_METATYPE(Shaman::AbstractPackage::List)
Q_DECLARE_METATYPE(Shaman::AbstractPackage::Action)
Q_DECLARE_METATYPE(Shaman::AbstractPackage::State)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::AbstractPackage::CachedValues)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::AbstractPackage::Properties)

#endif // ABSTRACTPACKAGE_H
