/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanbackend_p.h"

#include <QEvent>
#include <QCoreApplication>
#include <QThread>

#include <KDebug>
#include "shamanbase.h"
#include "shamanbase_p.h"
#include "shamantransactioninterface_p.h"
#include "HooksEventLoop_p.h"
#include "pluginsystem/PluginLoader_p.h"
#include "shamancoreplugin.h"
#include "shamanbackendinterface.h"
#include <KIcon>
#include "shamanbackendinterface_p.h"
#include "shamanbackendpackage.h"
#include "shamanpackagewatcher.h"

namespace Shaman {

BackendCapabilities::BackendCapabilities()
    : d(new Private)
{
}

BackendCapabilities::~BackendCapabilities()
{
    delete d;
}

BackendCapabilities::InformationCapabilities BackendCapabilities::informationCapabilities() const
{
    return d->informationC;
}

BackendCapabilities::PackageGroupingCapabilities BackendCapabilities::packageGroupingCapabilities() const
{
    return d->packageGroupingC;
}

BackendCapabilities::SearchCapabilities BackendCapabilities::searchCapabilities() const
{
    return d->searchC;
}

BackendCapabilities::TransactionCapabilities BackendCapabilities::transactionCapabilities() const
{
    return d->transC;
}

////////////////

QList< KAboutPerson > BackendProperties::authors() const
{
    return d->authors;
}

QStringList BackendProperties::configurationModules() const
{
    return d->configurationModules;
}

QString BackendProperties::description() const
{
    return d->description;
}

QIcon BackendProperties::icon() const
{
    return d->icon;
}

QString BackendProperties::name() const
{
    return d->name;
}

QIcon BackendProperties::packageIcon() const
{
    return d->packageIcon;
}

BackendProperties::BackendProperties()
    : d(new Private)
{

}

////////////////

ProcessQueueOption::ProcessQueueOption(const QString& name, const QString& description, const QVariant& data)
        : d(new Private)
{
    d->name = name;
    d->description = description;
    d->data = data;
}

ProcessQueueOption::~ProcessQueueOption()
{
    delete d;
}

QVariant ProcessQueueOption::data() const
{
    return d->data;
}

QString ProcessQueueOption::description() const
{
    return d->description;
}

QString ProcessQueueOption::name() const
{
    return d->name;
}

QUuid ProcessQueueOption::uuid() const
{
    return d->uuid;
}

////////////////

Shaman::Backend::Backend(BackendInterface *interface, QObject *parent)
  : QObject(parent),
    d(new BackendPrivate(interface))
{
    d->q_ptr = this;
    // Inject the interface
    interface->d->backend = this;

    qRegisterMetaType<Shaman::Backend::Error>("Shaman::Backend::Error");
    qRegisterMetaType<Shaman::Backend::State>("Shaman::Backend::State");
    qRegisterMetaType<Shaman::TransactionInterface::State>();
    qRegisterMetaType<Shaman::TransactionInterface::Error>();
}

Shaman::Backend::~Backend()
{
    delete d;
}

TransactionInterface * BackendPrivate::transactionInterface()
{
    if (tInterface.isNull()) {
        Q_Q(Backend);
        tInterface = new TransactionInterface(q);
    }
    return tInterface.data();
}

Shaman::Backend::State Shaman::Backend::state()
{
    return d->state;
}

Shaman::Backend::Error Shaman::Backend::error()
{
    return d->error;
}

const BackendCapabilities *Shaman::Backend::capabilities() const
{
    return d->capabilities;
}

Shaman::Repository::List Shaman::Backend::repositories() const
{
    return d->repos;
}

Shaman::PackageGroup::List Shaman::Backend::packageGroups() const
{
    return d->groups;
}

Shaman::AbstractPackage::List Shaman::Backend::packages() const
{
    return d->packages;
}

QList< DistUpgrade* > Backend::availableDistUpgrades() const
{
    return d->distUpgrades;
}

Shaman::AbstractPackage::List Shaman::Backend::queuedPackages() const
{
    Shaman::AbstractPackage::List retlist;
    foreach (Shaman::AbstractPackage *p, packages()) {
        if (p->action() == AbstractPackage::Install ||
            p->action() == AbstractPackage::Uninstall ||
            p->action() == AbstractPackage::Upgrade) {
            retlist.append(p);
        }
    }
    return retlist;
}

void Shaman::BackendPrivate::setState(Shaman::Backend::State newstate)
{
    kDebug() << state << newstate;
    if (state == newstate) {
        // Pass by, we don't care
        return;
    }

    Q_Q(Backend);

    if (newstate == Backend::Reloading) {
        upgradeablePackages.clear();
        emit q->upgradeablePackagesChanged(upgradeablePackages);
    }

    if (state == Backend::MarkingAdditionalTargets && newstate == Backend::Idling) {
        // We have all of our targets marked now. Let's cache the size for each one of them
        Shaman::AbstractPackage::List::const_iterator i;
        for (i = packages.constBegin(); i != packages.constEnd(); ++i) {
            if ((*i)->action() != Shaman::AbstractPackage::NoAction) {
                // Let's see if it's a package
                Shaman::BackendPackage *p = dynamic_cast< Shaman::BackendPackage* >(*i);
                if (p) {
                    if (!(p->cachedValues() & p->cachedValueForAdditionalCacheValue(Shaman::BackendPackage::DownloadSize))) {
                        CacheEventLoop e(BackendPackage::cachedValueForAdditionalCacheValue(Shaman::BackendPackage::DownloadSize));
                        PackageWatcher watcher(p);
                        e.connect (&watcher, SIGNAL(valueCached(Shaman::AbstractPackage::CachedValue)),
                                   &e, SLOT(requestQuit(Shaman::AbstractPackage::CachedValue)));
                        p->cacheValue(BackendPackage::cachedValueForAdditionalCacheValue(Shaman::BackendPackage::DownloadSize));
                        e.exec();
                    }
                }
            }
        }
    }

    if (state == Backend::OnTransaction) {
        // Something has just ended, before changing the state, let's run the hooks
        // Let's queue the hook run
        runHooksWhenBackIdle = true;
    }

    if (newstate == Backend::Idling && runHooksWhenBackIdle) {
        //runHooksWhenBackIdle = false;
        // Go with the hooks!
        // But - first check the transaction errors: if any, skip the hooks
        if (Base::instance()->core()->transactionInterface()->lastError() != TransactionInterface::NoError) {
            // In this case, skip the hooks

            // Set the transaction state to NoTransaction to declare the end of the operation :)
            setTransactionState(Shaman::TransactionInterface::NoTransaction);
            state = newstate;
            emit q->stateChanged(state);
        }

        setTransactionState(Shaman::TransactionInterface::RunningPostHooks);
        CorePlugin::Hook hook = CorePlugin::NoHooks;
        if (transactionType == "queue") {
            hook = CorePlugin::PostTransactionHook;
        } else if (transactionType == "repoupdate") {
            hook = CorePlugin::PostUpdateRepositoriesHook;
        }

        QHash<CorePlugin*, const char *> hooks = Base::instance()->d_func()->pluginLoader->hooksFor(hook);
        QHash<CorePlugin*, const char *>::const_iterator i;
        for (i = hooks.constBegin(); i != hooks.constEnd(); ++i) {
            HooksEventLoop e(hook, q);
            bool result;
            if (hook == CorePlugin::PostTransactionHook) {
                result = QMetaObject::invokeMethod(i.key(), i.value(), Qt::QueuedConnection,
                                                   Q_ARG(Shaman::AbstractPackage::List, transactionInterface()->packages()));
            } else {
                result = QMetaObject::invokeMethod(i.key(), i.value(), Qt::QueuedConnection,
                                                   Q_ARG(Shaman::Repository::List, repos));
            }

            if (result) {
                i.key()->connect(i.key(), SIGNAL(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)),
                                 &e, SLOT(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)));
                transactionInterface()->d->newTransactionMessageFromBackend(i18n("Running post transaction hook from %1...",
                                                                               i.key()->name()));
                e.exec();

                switch(e.result()) {
                case CorePlugin::Fatal:
                    transactionInterface()->d->newTransactionMessageFromBackend(i18n("The hook failed to run"));
                    return;
                case CorePlugin::SuccessWithWarning:
                    transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully with warnings"));
                    break;
                default:
                    transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully"));
                    break;
                }
            }
        }

        // Set the transaction state to NoTransaction to declare the end of the operation :)
        setTransactionState(Shaman::TransactionInterface::NoTransaction);
    }

    // Let's actually query the backend for the state transition
    if (!interface->aboutToChangeState(state, newstate)) {
        // The backend rejected the transition. End this here then.
        return;
    }
    state = newstate;
    emit q->stateChanged(state);
}

void Shaman::BackendPrivate::setError(Shaman::Backend::Error error, const QString &details)
{
    Q_Q(Backend);
    error = error;
    if (error != Backend::NoError) {
        emit q->errorTriggered(error, details);
    }
}

void Shaman::Backend::updateRepositories()
{
    // Reset the transaction interface first
    d->transactionInterface()->d->reset();
    // Pretransaction hooks
    d->setTransactionState(Shaman::TransactionInterface::RunningPreHooks);
    QHash<CorePlugin*, const char *> hooks = Base::instance()->d_func()->pluginLoader->hooksFor(CorePlugin::PreUpdateRepositoriesHook);
    QHash<CorePlugin*, const char *>::const_iterator i;
    for (i = hooks.constBegin(); i != hooks.constEnd(); ++i) {
        HooksEventLoop e(Shaman::CorePlugin::PreUpdateRepositoriesHook, this);
        if (QMetaObject::invokeMethod(i.key(), i.value(), Qt::QueuedConnection,
            Q_ARG(Shaman::Repository::List, repositories()))) {
            connect(i.key(), SIGNAL(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)), 
                    &e, SLOT(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)));
            d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Running pre update hook from %1...",
                                                                                i.key()->name()));
            e.exec();

            switch(e.result()) {
                case CorePlugin::Fatal:
                    d->setError(HookError, i18n("The hook from the plugin %1 failed to run and requested to abort the "
                                                "transaction", i.key()->name()));
                    d->interface->setTransactionError(Shaman::TransactionInterface::HookError,
                                                      i18n("A hook from the plugin %1 failed to run "
                                                           "and requested to abort the transaction", i.key()->name()));
                    return;
                case CorePlugin::SuccessWithWarning:
                    d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully with warnings"));
                    break;
                default:
                    d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully"));
                    break;
            }
        }
    }

    d->transactionType = "repoupdate";
    d->setState(OnTransaction);
    QMetaObject::invokeMethod(d->interface, "updateRepositories", Qt::QueuedConnection);
}

void Shaman::Backend::processQueue(const ProcessQueueOption::List &options)
{
    Shaman::AbstractPackage::List list;
    foreach (Shaman::AbstractPackage * package, packages()) {
        if (package->action() != Shaman::AbstractPackage::NoAction) {
            list.append(package);
        }
    }

    // Reset the transaction interface first
    d->transactionInterface()->d->reset();
    // Push stuff upon it
    d->transactionInterface()->d->setPackages(list);

    // Pretransaction hooks
    d->setTransactionState(Shaman::TransactionInterface::RunningPreHooks);
    QHash<CorePlugin*, const char *> hooks = Base::instance()->d_func()->pluginLoader->hooksFor(CorePlugin::PreTransactionHook);
    QHash<CorePlugin*, const char *>::const_iterator i;
    for (i = hooks.constBegin(); i != hooks.constEnd(); ++i) {
        kDebug() << "Hook from " << i.key()->name() << i.value();
        HooksEventLoop e(Shaman::CorePlugin::PreTransactionHook, this);
        if (QMetaObject::invokeMethod(i.key(), i.value(), Qt::QueuedConnection, Q_ARG(Shaman::AbstractPackage::List, list))) {
            kDebug() << "Invoked " << i.key()->name() << i.value();
            connect(i.key(), SIGNAL(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)), 
                    &e, SLOT(hookCompleted(CorePlugin::Hook,CorePlugin::HookResult)));
            d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Running pre transaction hook from %1...",
                                                                                i.key()->name()));
            e.exec();

            switch(e.result()) {
                case CorePlugin::Fatal:
                    d->setError(HookError, i18n("The hook from the plugin %1 failed to run and requested to abort the "
                                                "transaction", i.key()->name()));
                    d->interface->setTransactionError(Shaman::TransactionInterface::HookError,
                                                      i18n("A hook from the plugin %1 failed to run and requested to "
                                                           "abort the transaction", i.key()->name()));
                    return;
                case CorePlugin::SuccessWithWarning:
                    d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully with warnings"));
                    break;
                default:
                    d->transactionInterface()->d->newTransactionMessageFromBackend(i18n("Hook run successfully"));
                    break;
            }
        }
    }

    d->transactionType = "queue";

    d->setState(OnTransaction);
    // Keep it async
    d->interface->processQueue(list, options);
}

bool Backend::cancelQueueProcessing()
{
    return d->interface->cancelQueueProcessing();
}

bool Backend::cancelRepositoriesUpdate()
{
    return d->interface->cancelRepositoriesUpdate();
}


// Transaction interface stuff

void Shaman::BackendPrivate::setTransactionState(Shaman::TransactionInterface::State state)
{
    transactionInterface()->d->setTransactionState(state);
}

const Shaman::BackendProperties* Backend::properties() const
{
    return d->properties;
}

void Backend::markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options)
{
    // Reset the transaction interface first
    d->transactionInterface()->d->reset();

    d->setState(Backend::MarkingAdditionalTargets);
    d->interface->markAdditionalTargetsForQueue(options);
}

void Backend::unmarkAdditionalTargets()
{
    Shaman::AbstractPackage::List pkgs = packages();
    Shaman::AbstractPackage::List::const_iterator i;
    for (i = pkgs.constBegin(); i != pkgs.constEnd(); ++i) {
        if ((*i)->action() == Shaman::AbstractPackage::TransactionDependencyInstall ||
            (*i)->action() == Shaman::AbstractPackage::TransactionDependencyUninstall ||
            (*i)->action() == Shaman::AbstractPackage::TransactionDependencyUpgrade) {
            (*i)->setAction(Shaman::AbstractPackage::NoAction);
        }
    }
}

Shaman::AbstractPackage::List Backend::upgradeablePackages() const
{
    return d->upgradeablePackages;
}

void BackendPrivate::addUpgradeablePackage(Shaman::AbstractPackage *package)
{
    Q_Q(Backend);
    upgradeablePackages.append(package);
    if (q->packages().contains(package)) {
        emit q->upgradeablePackagesChanged(upgradeablePackages);
    }
}

void BackendPrivate::removeUpgradeablePackage(Shaman::AbstractPackage *package)
{
    Q_Q(Backend);
    upgradeablePackages.removeOne(package);
    if (q->packages().contains(package)) {
        emit q->upgradeablePackagesChanged(upgradeablePackages);
    }
}

ProcessQueueOption::List Backend::availableProcessQueueOptions() const
{
    return d->availablePQOptions;
}

AbstractPackage* Backend::packageFromFile(const KUrl& path)
{
    return d->interface->packageFromFile(path);
}

Base::OperationMode Backend::operationMode() const
{
    return Shaman::Base::instance()->d_func()->mode;
}

}

#include "shamanbackend.moc"
#include "shamanbackend_p.moc"
