/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANBACKEND_H
#define SHAMANBACKEND_H

#include "shamanmacros.h"
#include "shamanabstractplugin.h"
#include "shamanabstractpackage.h"
#include "shamanrepository.h"
#include "shamanpackagegroup.h"

#include "shamantransactioninterface.h"

#include <KAboutData>
#include <QtCore/quuid.h>
#include "shamanbase.h"

class KAboutData;
class QEvent;

/** This is the namespace for all classes in libshaman.*/
namespace Shaman {

class DistUpgrade;

class SHAMAN_EXPORT BackendCapabilities
{
    public:
        virtual ~BackendCapabilities();

        enum TransactionCapability {
            UnknownTransactionCapabilities = 0,
            InstallPackagesCapability = 1,
            RemovePackagesCapability = 2,
            UpgradePackagesCapability = 4,
            UpdateRepositoriesCapability = 8,
            UpgradeSystemCapability = 16,
            DistUpgradeCapability = 32,
            InstallPackageFromFileCapability = 64,
        };
        Q_DECLARE_FLAGS(TransactionCapabilities, TransactionCapability)

        enum SearchCapability {
            UnknownSearchCapabilities = 0,
            BasicSearchFromViewCapability = 1,
            ExtendedSearchFromViewCapability = 2,
            BasicSearchFromBackendCapability = 4,
            ExtendedSearchFromBackendCapability = 8
        };
        Q_DECLARE_FLAGS(SearchCapabilities, SearchCapability)

        enum PackageGroupingCapability {
            UnknownPackageGroupingCapabilities = 0,
            GroupPackagesByRepositoryCapability = 1,
            GroupPackagesByPackageGroupsCapability = 2
        };
        Q_DECLARE_FLAGS(PackageGroupingCapabilities, PackageGroupingCapability)

        enum InformationCapability {
            UnknownInformationCapabilities = 0,
            ShowFileListCapability = 1,
            ShowDependsOnCapability = 2,
            ShowRequiredByCapability = 4,
            ShowConflictsWithCapability = 8,
            ShowProvidesCapability = 16,
            ShowUpdateType = 32,
            ShowPackageChangelog = 64
        };
        Q_DECLARE_FLAGS(InformationCapabilities, InformationCapability)

        TransactionCapabilities transactionCapabilities() const;

        SearchCapabilities searchCapabilities() const;

        PackageGroupingCapabilities packageGroupingCapabilities() const;

        InformationCapabilities informationCapabilities() const;

    private:
        BackendCapabilities();

        class Private;
        Private * const d;

        friend class BackendInterface;
};

class SHAMAN_EXPORT BackendProperties
{
    public:
        QString name() const;
        QString description() const;
        QIcon icon() const;
        QIcon packageIcon() const;
        QList<KAboutPerson> authors() const;
        QStringList configurationModules() const;

    private:
        BackendProperties();

        class Private;
        Private * const d;

        friend class BackendInterface;
};

class SHAMAN_EXPORT ProcessQueueOption
{
    public:
        typedef QList<ProcessQueueOption*> List;

        explicit ProcessQueueOption(const QString &name, const QString &description, const QVariant &data = QVariant());
        ~ProcessQueueOption();

        QUuid uuid() const;
        QString name() const;
        QString description() const;
        QVariant data() const;

    private:
        class Private;
        Private * const d;
};

class BackendInterface;
class BackendPrivate;
/**
 * \class Backend shamanbackend.h
 * \brief Shaman's Backend class.
 *
 * This class it the base for all Backends. It has State and Error Properties.
 * To tell the other Plugins about Transaction-Progress, the TransactionInterface should be used.
 * The Plugin-Author can access it via transactionInterface().
 *
 * To implement a complete Backend, the Author also needs to reimplement the Package class.
 * The Plugin should set its Packages with setPackages(), its PackageGroups with setPackageGroups() and
 * its Repositories with setRepositories().
 *
 * The shamanbackend.has native threading support, though you might want to implement some threads in
 * your plugin. Anyway, consider that every method you are writing will be made available for
 * asynchronous calls, so implement threading only if it is strictly necessary. If while testing your
 * plugin you see the GUI blocked, open a bug report against Shaman before trying to implement
 * threading. In the end, use threading only if you need concurrent calls in your plugin.
 */
class SHAMAN_EXPORT Backend : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(Backend)
    Q_ENUMS(State)
    Q_ENUMS(Error)
    public:
        /**
         * \enum State
         *
         * This enum describes the State of the Backend.
         * @see setState()
         * @see state()
         * @see stateChanged()
         */
        enum State
        {
            /**
             * The Backend is idling.
             */
            Idling = 0,
            /**
             * The Backend is currently intitializing itself.
             */
            Initializing = 1,
            /**
             * The Backend is currently reloading itself.
             */
            Reloading = 2,
            /**
             * The backend is currently performing a search on packages
             */
            Searching = 3,
            /**
             * The backend is currently marking additional targets for the transaction
             */
            MarkingAdditionalTargets = 4,
            /**
             * The backend is currently waiting for a lock on the package management system.
             * Until this state persists, no more transactions or requests can be queued.
             */
            WaitingForLock = 5,
            /**
             * The Backend is on a transaction. This is usually done in updateRepositories() and processQueue().
             * After the backend will turn back on idle from OnTransaction, it will be automatically reloaded.
             */
            OnTransaction = 6,
            /**
             * The Backend is broken. This should be set when, for example, the dbus interface of the
             * Package Manager Backend or a wrong library version was found.
             */
            Broken = 7
        };

        /**
         * \enum Error
         *
         * This enum describes the Error which occurred in the Backend.
         * @see setError()
         * @see error()
         * @see errorTriggered()
         */
        enum Error
        {
            /**
             * This is the default state of the Backend. Everything is ok, no Error triggered.
             */
            NoError = 0,
            /**
             * This should be set when an error got triggered while initializing itself.
             */
            InitializationError = 1,
            /**
             * This should be set when an error inside a transaction has been triggered.
             * Usually after updateRepositories(), processQueue() or addditionalTargetsForQueue() got called.
             */
            TransactionError = 2,
            /**
             * This is automatically set whenever a transaction gets aborted due to an hook
             * in a plugin
             */
            HookError = 3
        };

        /**
         * The destructor of the Backend.
         */
        virtual ~Backend();

    public Q_SLOTS:

        /**
         * @returns the current State of the Backend
         * @see setState()
         */
        State state();

        /**
         * @returns the Error which is set to the Backend. If everything is ok, it returns Shaman::Backend::NoError.
         * @see setError()
         */
        Error error();

        const BackendProperties *properties() const;

        /**
         * @returns capabilities of the current backend.
         * @see setCapabilities()
         */
        const BackendCapabilities *capabilities() const;

        Shaman::ProcessQueueOption::List availableProcessQueueOptions() const;

        /**
         * @returns all available Repositories
         */
        Shaman::Repository::List repositories() const;

        /**
         * @returns all available Package-Groups
         */
        Shaman::PackageGroup::List packageGroups() const;

        /**
         * @returns all Packages
         */
        Shaman::AbstractPackage::List packages() const;

        /**
         * @returns all packages currently in queue
         */
        Shaman::AbstractPackage::List queuedPackages() const;

        /**
         * @returns all currently upgradeable packages
         */
        Shaman::AbstractPackage::List upgradeablePackages() const;

        QList<Shaman::DistUpgrade*> availableDistUpgrades() const;

        /**
         * Always call this function before processQueue. It
         * adds additional information to the Packages for the upcoming queue, and helps discovering
         * failures before the actual transaction takes place. Also, some backends might require a simulation/resolving
         * before the actual queue is processed.
         * \par
         * During this stage, the state of the backend will switch to \c MarkingAdditionalTargets, and the backend will
         * be able to stream questions, errors, and will likely show the progress of the operation through \c operationProgress
         */
        void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options);

        /**
         * Cancels the Queue-Processing, else does nothing.
         */
        bool cancelQueueProcessing();

        /**
         * Cancels the repository update process, else does nothing.
         */
        bool cancelRepositoriesUpdate();

        /**
         * Updates the Database of the Backend
         */
        void updateRepositories();

        /**
         * Asks the backend to process the packages marked for queue. Be sure to call this function after
         * \c markAdditionalTargetsForQueue has been called and did not report errors.
         *
         * @param packages the packages explicitly selected by the user to be processed.
         * @param options a list of processing options for the current queue
         *
         * @see markAdditionalTargetsForQueue
         */
        void processQueue(const ProcessQueueOption::List &options = ProcessQueueOption::List());

        AbstractPackage *packageFromFile(const KUrl &path);

        void unmarkAdditionalTargets();

        Shaman::Base::OperationMode operationMode() const;

    Q_SIGNALS:
        /**
         * This signal will be emitted emitted when the current State of the Backend changed.
         * @param state the current State of the Backend
         * @see state()
         */
        void stateChanged(Shaman::Backend::State state);

        /**
         * This signal will be emitted when an error occurred.
         * @param error the Error which occurred
         * @see error()
         */
        void errorTriggered(Shaman::Backend::Error error, const QString &details);

        /**
         * This gets automatically emitted when the Repositories of the Backend changed.
         * @param repositories the current Repositories
         * @see setRepositories()
         * @see repositories()
         */
        void repositoriesChanged(Shaman::Repository::List repositories);

        /**
         * This gets automatically emitted when the PackageGroups of the Backend changed.
         * @param groups the current PackageGroups
         * @see packageGroups()
         * @see setPackageGroups()
         */
        void packageGroupsChanged(Shaman::PackageGroup::List groups);

        /**
         * This gets automatically emitted when the Packages of the Backend changed.
         * @param packages the current Packages
         * @see packages()
         * @see setPackages()
         */
        void packagesChanged(Shaman::AbstractPackage::List packages);

        void upgradeablePackagesChanged(Shaman::AbstractPackage::List packages);

        void operationProgress(const QString &operation, int progress);

        void newDistUpgradeAvailable(Shaman::DistUpgrade *upgrade);

    private:
        Backend(Shaman::BackendInterface *interface, QObject *parent);

        BackendPrivate * const d;

        friend class BackendPrivate;
        friend class BackendInterface;
        friend class CoreInterface;
        friend class TransactionInterface;
        friend class TransactionQuestion;
        friend class PluginLoader;
        friend class ManualProgressReporter;
        friend class BackendPackage;
};
}

Q_DECLARE_METATYPE(Shaman::Backend::State)
Q_DECLARE_METATYPE(Shaman::Backend::Error)
Q_DECLARE_METATYPE(Shaman::ProcessQueueOption::List)

Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::BackendCapabilities::TransactionCapabilities)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::BackendCapabilities::SearchCapabilities)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::BackendCapabilities::InformationCapabilities)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::BackendCapabilities::PackageGroupingCapabilities)

#endif
