/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANBACKEND_P
#define SHAMANBACKEND_P

#include "shamanbackend.h"

#include <QEvent>
#include <QEventLoop>

namespace Shaman {

class BackendPrivate
{
    public:
        BackendPrivate(BackendInterface *iface)
          : state(Shaman::Backend::Idling),
            error(Shaman::Backend::NoError),
            capabilities(0),
            properties(0),
            interface(iface),
            runHooksWhenBackIdle(false)
        {
        }
        ~BackendPrivate()
        {}

        enum ThreadedFunction {
            RegisterThreadMetatypes = 1001,
            Init = 1002,
            ProcessQueue = 1003,
            UpdateDatabase = 1004
        };

        Q_DECLARE_PUBLIC(Backend)
        Backend *q_ptr;

        TransactionInterface *transactionInterface();
        void addUpgradeablePackage(Shaman::AbstractPackage *package);
        void removeUpgradeablePackage(Shaman::AbstractPackage *package);
        void setTransactionState(Shaman::TransactionInterface::State state);
        void setError(Shaman::Backend::Error error, const QString &details);
        void setState(Shaman::Backend::State state);

        Shaman::Backend::State state;
        Shaman::Backend::Error error;
        BackendCapabilities *capabilities;
        Shaman::PackageGroup::List groups;
        Shaman::Repository::List repos;
        Shaman::AbstractPackage::List packages;
        Shaman::AbstractPackage::List upgradeablePackages;
        QPointer<TransactionInterface> tInterface;
        QString details;
        QString transactionType;
        Shaman::AbstractPackage::List currentQueuePackages;
        BackendProperties *properties;
        BackendInterface *interface;
        QList<Shaman::DistUpgrade*> distUpgrades;
        Shaman::ProcessQueueOption::List availablePQOptions;
        bool runHooksWhenBackIdle;
};

class BackendCapabilities::Private
{
    public:
        Private() {}

        TransactionCapabilities transC;
        SearchCapabilities searchC;
        PackageGroupingCapabilities packageGroupingC;
        InformationCapabilities informationC;
};

class BackendProperties::Private
{
    public:
        Private() {}

        QString name;
        QString description;
        QIcon icon;
        QIcon packageIcon;
        QList<KAboutPerson> authors;
        QStringList configurationModules;
};

class ProcessQueueOption::Private
{
    public:
        Private() : uuid(QUuid::createUuid()) {}

        QUuid uuid;
        QString name;
        QString description;
        QVariant data;
};

class CacheEventLoop : public QEventLoop
{
    Q_OBJECT
    public:
        explicit CacheEventLoop(Shaman::AbstractPackage::CachedValue value, QObject* parent = 0)
         : QEventLoop(parent), m_value(value), m_cached(false) {}
         virtual ~CacheEventLoop() {}

         int exec(ProcessEventsFlags flags = AllEvents) {
             if (m_cached) {
                 return 0;
             } else {
                 return QEventLoop::exec(flags);
             }
         }

    public slots:
        void requestQuit(Shaman::AbstractPackage::CachedValue value) {
            if (m_value == value) {
                m_cached = true;
                quit();
            }
        }

    private:
        Shaman::AbstractPackage::CachedValue m_value;
        bool m_cached;
};

}

#endif
