/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
   Copyright (C) 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "shamanbackendinterface_p.h"
#include "shamanbackend_p.h"
#include "shamanbase.h"
#include "shamantransactioninterface_p.h"
#include "shamancoreinterface_p.h"

#include <KIcon>
#include <QSet>
#include "shamanquery.h"

namespace Shaman {

ManualProgressReporter::ManualProgressReporter(Backend* parent)
    : QObject(parent)
    , d(new Private(parent))
{

}

ManualProgressReporter::~ManualProgressReporter()
{
    delete d;
}

void ManualProgressReporter::setDownloadSpeed(qint64 bytesPerSecond)
{
    d->q->d->transactionInterface()->d->setDownloadSpeed(bytesPerSecond);
}

void ManualProgressReporter::setRemainingTime(const QTime& dueToFinishAt)
{
    d->q->d->transactionInterface()->d->setRemainingTime(dueToFinishAt);
}

void ManualProgressReporter::setTotalDownloadedSize(qint64 downloadedBytes)
{
    d->q->d->transactionInterface()->d->setTotalDownloadedSize(downloadedBytes);
}

void ManualProgressReporter::setTotalDownloadSize(qint64 totalBytes)
{
    d->q->d->transactionInterface()->d->setTotalDownloadSize(totalBytes);
}

void ManualProgressReporter::setTransactionProgress(int percent)
{
    d->q->d->transactionInterface()->d->setTransactionProgress(percent);
}

////////////////

BackendInterface::BackendInterface(QObject* parent)
    : QObject(parent)
    , d(new Private)
{
    d->q_ptr = this;
}

BackendInterface::~BackendInterface()
{
    qDeleteAll(d->backend->d->packages);
    qDeleteAll(d->backend->d->repos);
    qDeleteAll(d->backend->d->groups);
}

Backend* BackendInterface::backend() const
{
    return d->backend;
}

ManualProgressReporter* BackendInterface::manualProgressReporter()
{
    if (d->manualProgressReporter.isNull()) {
        d->manualProgressReporter = new ManualProgressReporter(d->backend);
    }

    return d->manualProgressReporter;
}

void BackendInterface::setState(Backend::State state)
{
    d->backend->d->setState(state);
}

void BackendInterface::setError(Backend::Error error, const QString& details)
{
    d->backend->d->setError(error, details);
}

void BackendInterface::setCapabilities(BackendCapabilities::TransactionCapabilities transactionCapabilities,
                                    BackendCapabilities::SearchCapabilities searchCapabilities,
                                    BackendCapabilities::PackageGroupingCapabilities packageGroupingCapabilities,
                                    BackendCapabilities::InformationCapabilities informationCapabilities)
{
    delete d->backend->d->capabilities;
    d->backend->d->capabilities = new BackendCapabilities();
    d->backend->d->capabilities->d->transC = transactionCapabilities;
    d->backend->d->capabilities->d->searchC = searchCapabilities;
    d->backend->d->capabilities->d->packageGroupingC = packageGroupingCapabilities;
    d->backend->d->capabilities->d->informationC = informationCapabilities;
}

void BackendInterface::setPackages(const Shaman::AbstractPackage::List &packages)
{
    // Delete the unused items
    QSet<Shaman::AbstractPackage*> toDelete = d->backend->d->packages.toSet().subtract(packages.toSet());
    qDeleteAll(toDelete);

    // Now set the packages
    d->backend->d->packages = packages;
    emit d->backend->packagesChanged(packages);
}

void BackendInterface::packagesRetrieved(const Shaman::AbstractPackage::List& packages)
{
    d->backend->d->packages.append(packages);
    emit d->backend->packagesChanged(packages);
}

void BackendInterface::setRepositories(const Shaman::Repository::List &repos)
{
    // Delete the unused items
    QSet<Shaman::Repository*> toDelete = d->backend->d->repos.toSet().subtract(repos.toSet());
    qDeleteAll(toDelete);

    // Now set the repositories
    d->backend->d->repos = repos;
    emit d->backend->repositoriesChanged(repos);
}

void BackendInterface::setPackageGroups(const Shaman::PackageGroup::List &groups)
{
    // Delete the unused items
    QSet<Shaman::PackageGroup*> toDelete = d->backend->d->groups.toSet().subtract(groups.toSet());
    qDeleteAll(toDelete);

    // Now set the package groups
    d->backend->d->groups = groups;
    emit d->backend->packageGroupsChanged(groups);
}

int BackendInterface::Private::generateNewQuestionToken()
{
    ++lastToken;
    return lastToken;
}

// Transaction interface stuff

void BackendInterface::setPackageDownloadProgress(AbstractPackage *package, qint64 downloadedBytes, qint64 totalBytes)
{
    d->backend->d->transactionInterface()->d->setPackageDownloadProgress(package, downloadedBytes, totalBytes);
}

void BackendInterface::setPackageTransactionProgress(AbstractPackage *package, int percent)
{
    d->backend->d->transactionInterface()->d->setPackageTransactionProgress(package, percent);
}

void BackendInterface::setTransactionError(Shaman::TransactionInterface::Error error, const QString& details)
{
    d->backend->d->transactionInterface()->d->setTransactionError(error, details);
}

void BackendInterface::setTransactionState(Shaman::TransactionInterface::State state)
{
    d->backend->d->transactionInterface()->d->setTransactionState(state);
}

void BackendInterface::addTransactionMessage(const QString& message)
{
    d->backend->d->transactionInterface()->d->newTransactionMessageFromBackend(message);
}

void BackendInterface::newTransactionQuestion(TransactionQuestion* question)
{
    d->backend->d->transactionInterface()->d->newTransactionQuestion(question);
}

void BackendInterface::searchCompleted(const QUuid &searchUuid,const Shaman::AbstractPackage::List &packages)
{
    Shaman::Base::instance()->core()->d_func()->queryCompleted(this, searchUuid, packages);
}

void BackendInterface::setProperties(const QString& name, const QString& description, const QIcon& icon, const QIcon&
packageIcon, const QList< KAboutPerson >& authors, const QStringList& configurationModules)
{
    delete d->backend->d->properties;
    d->backend->d->properties = new BackendProperties;
    d->backend->d->properties->d->name = name;
    d->backend->d->properties->d->description = description;
    d->backend->d->properties->d->icon = icon;
    d->backend->d->properties->d->packageIcon = packageIcon;
    d->backend->d->properties->d->authors = authors;
    d->backend->d->properties->d->configurationModules = configurationModules;
}

void BackendInterface::showOperationProgress(const QString& operation, int progress)
{
    /*if (Base::instance()->statusBar()) {
        if (progress < 100) {
            Base::instance()->statusBar()->showMessage(operation, KIcon("system-software-update").pixmap(16,16), -1);
        } else {
            Base::instance()->statusBar()->showMessage(operation, KIcon("dialog-ok").pixmap(16,16), 10);
        }

        Base::instance()->statusBar()->setProgress(progress);
    }*/

    // Now emit the signal from the Backend as well
    emit d->backend->operationProgress(operation, progress);
}

void BackendInterface::newDistUpgrade(DistUpgrade* distUpgrade)
{
    d->backend->d->distUpgrades.append(distUpgrade);
    emit d->backend->newDistUpgradeAvailable(distUpgrade);
}

AbstractPackage* BackendInterface::packageFromFile(const KUrl& path)
{
    Q_UNUSED(path)
    return 0;
}

void BackendInterface::setAvailableProcessQueueOptions(const Shaman::ProcessQueueOption::List& options)
{
    d->backend->d->availablePQOptions = options;
}

MetaPackage* BackendInterface::metaPackage(const QString& name)
{
    return Shaman::Base::instance()->core()->d_func()->metaPackage(name);
}

bool BackendInterface::aboutToChangeState(Backend::State oldstate, Backend::State newstate)
{
    Q_UNUSED(oldstate)
    Q_UNUSED(newstate)

    // Always accept state changes
    return true;
}

void BackendInterface::searchPackages(Query* query)
{
    searchCompleted(query->id(), AbstractPackage::List());
}

}

#include "shamanbackendinterface.moc"
