/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef BACKENDINTERFACE_H
#define BACKENDINTERFACE_H

#include <QObject>
#include "shamanbackend.h"
#include "shamantransactionquestion.h"

namespace Shaman {

class Query;

class MetaPackage;

class ManualProgressReporter : public QObject {

    Q_OBJECT
    Q_DISABLE_COPY(ManualProgressReporter)
    public:
        virtual ~ManualProgressReporter();

    public Q_SLOTS:
        /**
         * Sets the global progress for a transaction. Normally you would not need to call this function
         * if your backend reports accurate per-package information: libshaman will do the necessary internal
         * computations required. Although, some backends do not provide enough information to retrieve an
         * accurate value of the total progress: in this case, you can use this function.
         * \par
         * As everything else in Shaman, this function is referred to the state in which the backend is. This
         * means that if your backend is in the transaction state \c Downloading, this function will set the
         * progress for the \c Downloading operation, as one might expect
         *
         * \note Calling this function in the \c Downloading state will invalidate information about
         *
         * @param percent the progress of the current transaction in percent
         */
        void setTransactionProgress(int percent);

        void setTotalDownloadedSize(qint64 downloadedBytes);

        void setTotalDownloadSize(qint64 totalBytes);

        void setDownloadSpeed(qint64 bytesPerSecond);

        void setRemainingTime(const QTime &dueToFinishAt);

    private:
        ManualProgressReporter(Backend* parent);

        class Private;
        Private * const d;

        friend class BackendInterface;
};

/**
 * \class BackendInterface shamanbackendinterface.h
 * \brief A interface to implement for creating a backend for Shaman
 *
 * BackendInterface is an interface that should be implemented by a package management backend.
 * This class is not exported publicly and it's only meant to serve for implementing a new backend.
 * Hence, this class is interesting only for backend developers.
 * If you're developing an application based on libshaman, you are probably looking for \c Backend instead.
 *
 * \par The basics
 * Shaman's backends are based on a light, but powerful state machine and on a few functions to reimplement.
 * BackendInterface is task based and centered on a very high-level abstraction: the principle is that
 * the backends should handle their internals in the best possible way without limitations, and be required only to
 * implement some common operations (processing packages, updating repositories, etc). Also, Shaman lets every backend
 * specify its capabilities, to avoid developers to implement and go through stuff they are not interested in,
 * but still providing the best possible UI by morphing it based on what the backend is capable of.
 *
 * \par Implementing a basic backend
 * To implement a basic backend, you would have to reimplement this class and \c Package. It is extremely important to
 * set the capabilities correctly, to avoid libshaman querying your backend for information it does not provide.
 * More than implementing the basic required actions (initialization, queue processing, repositories update and reload),
 * you would probably need to deal with the most important component of a backend, the state machine, and the transaction
 * progress report.
 *
 * \par The state machine
 * The state machine is the most important part when creating or using a backend. The state machine triggers events,
 * notifies about state, and handles most of the backend logic. There are two main state machines: the backend
 * state machine, accessible through \c setState, and the transaction state machine, accessible through \c setTransactionState.
 * When setting a new state, it is actually not guaranteed (quite the opposite) that the backend will switch immediately
 * to the next state: for example, setting the state to \c Idling when the backend is \c OnTransaction will cause
 * the backend to perform the post transaction hooks, hence the state will switch to \c Idling when this operation will have
 * finished. For this reason, it might make sense (if needed) to monitor the state in your backend through
 * \c Backend::stateChanged to track the backend state. This applies also to TransactionState.
 * \par
 * For this reason, Shaman has a concept of "state transitions", which will make your life easier and allow you to perform a
 * task right before the state change happens. This can be done inside \c aboutToChangeState, where, for example,
 * you might want to handle cache population if you're in cached mode (see below). \c aboutToChangeState will be called right
 * before the actual state transition takes place, letting you hook into the process and eventually block it. For example:
 * \code
 * if (oldstate == OnTransaction && newstate == Idling) {
 *   if (operationMode() == Cached) {
 *     // We need to call our reload function here
 *     myReloadFunction();
 *     // We actually want to avoid the state transition here, since our reload function will change the state to Reloading
 *     // let's reject the event then
 *     return false;
 *   }
 * }
 * \endcode
 * Just like with Qt event system, you might accept or reject the state transition. The default implementation always returns
 * true.
 * \par
 * The state machine is updated automatically whenever needed. For example, when processQueue gets called, it is guaranteed
 * that the backend's state will be \c OnTransaction. Hence, you will need to manually call \c setState only for
 * \c Idling, \c Reloading, \c WaitingForLock and \c Broken. More details can be found in \c setState and \c setTransactionState
 * documentation.
 *
 * \par Operation modes and cache
 * Shaman can operate in two different modes: cached and uncached. In the cached mode, the backend is expected to always provide
 * all the available packages to the frontend through \c setPackages. More insight on this will be given in the next paragraph.
 * In uncached mode, the backend has to create new package objects, if required, when asked by the interface. Usually, this can
 * happen only when searching for packages and (eventually) when marking additional targets for queue.
 * So the only places in your implementation where you will have to take care of this will be in the reloading functions,
 * \c search and \c markAdditionalTargetsForQueue.
 *
 * \par Reloading packages
 * Shaman has a particular state, \c Reloading, which is meant to be used while the backend is busy loading the package cache, if
 * required (see paragraph above).
 * While the backend is in the \c Reloading state it will not allow anything else to be done and the frontend
 * will likely be in a waiting state. It's up to you deciding what to reload while in the reloading state:
 * it is required that at least the package and repository list should be filled when getting back to \c Idling,
 * so that the user will be able to perform most of the operations.
 * \par
 * Depending on the performances of your backend, you might choose to perform the other operations in the background.
 * For example, the Aqpm backend, being very fast, handles reloading all the information while in the \c Reloading
 * state, whereas the PackageKit one reloads only the repositories and packages list, and handles loading upgradeable
 * packages and groups after getting back to \c Idling.
 * \par
 * A reload, though, can happen only if the backend is not performing any other operation (it is either \c Idling or in a state
 * transition, see \c aboutToChangeState). Usually, for example, you might want to reload packages after a transaction inside
 * \c aboutToChangeState, when oldstate == \c OnTransaction and newstate == \c Idling. Read more in the documentation for
 * \c aboutToChangeState.
 *
 * \par Reporting transactions' progress
 * Shaman strives to automate wherever possible to let developers code only the strictly necessary. Although, when
 * dealing with transactions' progress, some backends might be not able to provide enough or not accurate enough information
 * to let Shaman track progress efficiently. In case every information your backend provides is reliable, Shaman will
 * be able to provide the user rich progress information with extremely few information from your side:
 * \c setPackageDownloadProgress, \c setPackageTransactionProgress and the \c Repository state machine. However, if you
 * want to bypass automation for any reason, you can use \c manualProgressReporter to feed Shaman with lower level information.
 * In this case, Shaman will prevent automatic information from being generated automatically and will report your information.
 *
 * \see Backend
 */
class BackendInterface : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(BackendInterface)
    public:
        virtual ~BackendInterface();

    protected Q_SLOTS:
        /**
         * Reimplement this function in your backend. Calling this function should retrieve
         * the additional targets required for processing the queue (the queue can be obtained through \c queuedPackages)
         * \par
         * The function is designed to be asynchronous and it's part of Shaman's state machine, so you
         * will have to set the state to \c Idling when you're finished (it is automatically set to \c MarkingAdditionalTargets
         * when this method will be called)
         * \par
         * Inside this function, you would have to compute additional targets and set the action of the relevant packages to
         * TransactionDependency{Install,Uninstall,Upgrade} depending on the requirements. You can also ask questions in
         * this phase, throughout \c newTransactionQuestion.
         * \par
         * In case there was an error during the dependency solving, you can handle this by calling \c setTransactionError,
         * to let the frontend abort the operation earlier.
         *
         */
        virtual void markAdditionalTargetsForQueue(const Shaman::ProcessQueueOption::List &options) = 0;

        /**
         * This function, if reimplemented, lets you hook into Shaman's state machine and perform operations before the state
         * is changed and propagated. Within this function, you are also able to prevent state changes by rejecting the
         * transition (returning false). This must be done \b ONLY if you know what you're doing: the only sensible use case
         * for rejecting a transition is that a particular state change in your backend triggers an operation that changes the
         * state itself. The most explainatory example is a backend operating in cached mode that needs to reload packages after
         * a transaction: in this case you would be notified of the transition \c OnTransaction -> \c Idling, but inside this
         * function you'll be switching state to \c Reloading. For this reason, you don't want the transition \c OnTransaction ->
         * \c Idling to happen, as you'll take care of setting the backend back to \c Idling whenever your reloading ends.
         * \par
         * Despite appearing complex, this kind of design is extremely flexible, as it allows you to make the State machine
         * extremely more flexible and reliable for the frontend, and also to make yourself sure that some conditions will
         * be met whenever the backend will reach a certain state.
         *
         * \note Be aware that interrupting a transition from the \c Initializing state might break things in the interface and
         *       cause delayed loading. A good practice is the following:
         * \code
         * if ((oldstate == Shaman::Backend::Initializing && newstate == Shaman::Backend::Idling) ||
         *     (oldstate == Shaman::Backend::OnTransaction && newstate == Shaman::Backend::Idling)) {
         *   // Time to reload, guys.
         *   QTimer::singleShot(0, this, SLOT(reload()));
         *   // If we're initializing, just don't reject the state transition
         *   if (oldstate != Shaman::Backend::Initializing) {
         *     // Otherwise do.
         *     return false;
         *   }
         * }
         * \endcode
         * This guarantees you that the interface will cope with the fact that the initialization has completed and a reload is
         * following
         *
         * \p oldstate The state the backend is exiting from
         * \p newstate The state the backend is entering in
         *
         * \returns \c true if the transition is accepted. In this case the signal \c Backend::stateChanged will be emitted
         *          right after.
         *          \c false if the transition is not accepted. In this case, the state will remain set to \c oldstate. Please
         *          consider what was written above before returning \c false.
         */
        virtual bool aboutToChangeState(Shaman::Backend::State oldstate, Shaman::Backend::State newstate);

        /**
         * Reimplement this function only if you specified in the backend capabilities \c BasicSearchFromBackendCapability
         * or \c ExtendedSearchFromBackendCapability. Perform the search in your backend and emit
         * \c searchCompleted carrying a list of matching packages when done. This function is meant to
         * be asynchronous and it's part of Shaman's state machine, so you
         * will have to set the state to \c Idling when you're finished (it is automatically set to \c Searching
         * when this method will be called)
         *
         * @param query the query string
         * @param type the type of the search
         */
        virtual void searchPackages(Query *query);

        /**
         * In this method's reimplementation you will have to update the repositories of your backend.
         * This function is meant to be asynchronous and it's part of Shaman's state machine, so you
         * will have to set the state to \c Idling when you're finished (it is automatically set to \c OnTransaction
         * when this method will be called)
         */
        virtual void updateRepositories() = 0;

        /**
         * Cancels the Database-Update if one is processing, else does nothing.
         */
        virtual bool cancelRepositoriesUpdate() = 0;

        /**
         * \brief Processes the current queue, as represented in \c packages.
         *
         * In this method's reimplementation you will have to process the given packages with the given options.
         * This function is meant to be asynchronous and it's part of Shaman's state machine, so you
         * will have to set the state to \c Idling when you're finished (it is automatically set to \c OnTransaction
         * when this method will be called)
         *
         * Please note that \c packages includes only a list of packages explicitly marked by the user, it
         * does not include the dependencies found through \c markAdditionalTargetsForQueue. You can retrieve them,
         * if needed, by iterating the packages list returned by \c packages().
         * \par
         * To check what needs to be done with each package in the list, just check the \c Action associated with
         * each \c Package
         *
         * @param packages the packages explicitly selected by the user to be processed.
         * @param options a list of processing options for the current queue
         */
        virtual void processQueue(const Shaman::AbstractPackage::List & packages,
                                  const Shaman::ProcessQueueOption::List &options) = 0;

        /**
         * Cancels the Queue-Processing, else does nothing.
         */
        virtual bool cancelQueueProcessing() = 0;

        /**
         * Emit this signal when \c searchPackages has completed, carrying the result, and the original query and type
         *
         * @see searchPackages
         */
        void searchCompleted(const QUuid &searchUuid,const Shaman::AbstractPackage::List &packages);

        /**
         * \brief Initializes the backend and makes it ready to be used
         *
         * This function is guaranteed to be the very first function to be called after the constructor, in which
         * you will need to get the backend up and running and ready to be queried. Inside this function, you are also
         * required to set the capabilities of your backend and, if you like, its about data.
         * \par
         * This function will be called only once - it is like a sort of delayed constructor. This function is meant to
         * be asynchronous and it's part of Shaman's state machine, so you
         * will have to set the state to \c Idling when you're finished (it is automatically set to \c Initializing
         * when this method will be called)
         *
         * \see setCapabilities
         * \see setAboutData
         */
        virtual void init() = 0;

        /**
         * Reimplement this function inside your backend. This function will be called when the question passed as
         * a parameter will be answered.
         *
         * \note it is safe (and advised) to delete the TransactionQuestion object after processing it
         *
         * @param question the question object carrying the answer
         *
         * @see newTransactionQuestion
         * @see TransactionQuestion
         */
        virtual void transactionQuestionAnswered(Shaman::TransactionQuestion *question) = 0;

        virtual AbstractPackage *packageFromFile(const KUrl &path);

        /**
         * \brief Sets the current state of the Backend
         *
         * This function is the heart of Shaman's state machine. You should call this function only when you need
         * to change the backend state manually. See \c Backend::State to learn more about what each state does.
         *
         * \note It is not guaranteed that upon calling this function the state of the backend will immediately switch.
         *       If you want to track the backend's state effectively you would have to connect to \c Backend::stateChanged.
         *
         * The only states that \b REQUIRE you to set them manually are \c Idling, \c WaitingForLock, \c Reloading and \c Broken.
         * All the others are automatically set when needed, so you mostly have to set the state back to \c Idling when an
         * operation that triggered a state change ends. Every method in BackendInterface that expects you to set the
         * backend back to \c Idling at the end of the operation explicitly states it in its documentation.
         *
         * \warning It is strongly unrecommended to set a state which is not in the three ones mentioned above manually
         *
         * @param state the new State
         *
         * \see Backend::State
         */
        void setState(Shaman::Backend::State state);

        /**
         * If an error occurred, this is the right function to inform other plugins.
         * @param error the Error which occurred
         * @param details details about the error
         */
        void setError(Shaman::Backend::Error error, const QString &details);

        /**
         * Sets the Packages of the Backend. This is needed while initializing. All the packages previously
         * in the cache (not contained in \c packages) will be freed.
         *
         * \note This has to be used \b ONLY in cached mode. Use \c packagesRetrieved instead.
         * @param packages the Packages of the Backend
         */
        void setPackages(const Shaman::AbstractPackage::List &packages);

        /**
         * Call this function whenever the backend caches new packages. This function has to be used when the backend
         * is in uncached mode and retrieved some packages, for example through a search or while marking additional targets.
         */
        void packagesRetrieved(const Shaman::AbstractPackage::List &packages);

        /**
         * Sets the Repositories of the Backend. This is needed while initializing.
         * @param repos the Repositories of the Backend
         */
        void setRepositories(const Shaman::Repository::List &repos);

        /**
         * Sets the PackageGroups of the Backend. This is needed while initializing
         * @param groups the PackageGroups of the Backend
         */
        void setPackageGroups(const Shaman::PackageGroup::List &groups);

        // Transaction setters
        /**
         * This sets a new State of the transaction
         * @param state the new State
         */
        void setTransactionState(Shaman::TransactionInterface::State state);

        /**
         * This sets a new Error to the TransactionInterface.
         * @param error the Error which occurred
         */
        void setTransactionError(Shaman::TransactionInterface::Error error, const QString &details = QString());

        /**
         * Sets the progress for the current Package. If your backend reports information correctly,
         * this function together with \c setPackageDownloadProgress is the only one you will need to
         * call to get advanced progress reporting for free. All the other needed information will be
         * computed internally by Shaman.
         *
         * @param percent the progress of the current Package in percent
         */
        void setPackageTransactionProgress(Shaman::AbstractPackage *package, int percent);

        /**
         * Sets the download progress for a package. If your backend reports information correctly,
         * this function together with \c setPackageTransactionProgress is the only one you will need to
         * call to get advanced progress reporting for free. All the other needed information will be
         * computed internally by Shaman.
         *
         * @param bytespersecond the current Download-Speed in Bytes/s
         */
        void setPackageDownloadProgress(Shaman::AbstractPackage *package, qint64 downloadedBytes, qint64 totalBytes);

        /**
         * This function returns the manual progress reporter for the backend. As the name says, it should be
         * used only if your transaction is not able to provide accurate information through \c Package::size,
         * \c Repository::Status, \c setPackageDownloadProgress and \c setPackageTransactionProgress.
         * If those four information are reliable, you won't need to access and use this object.
         * Otherwise, through it you can partly or totally bypass Shaman's automatic progress computation
         * and report progress in form of percentage and some additional info.
         */
        ManualProgressReporter *manualProgressReporter();

        ////
        /**
         * \brief Shows some feedback about an operation to the user.
         *
         * This will likely be shown in the statusbar of the application. This function is useful for giving feedback for
         * lenghty operations (such as reloading). This is \b not useful for transactions, you should refer to the dedicated
         * functions for that. The most common usage for this function is when reloading or searching from the backend.
         *
         * \param operation A description of the operation in progress
         * \param progress The progress of the operation, ranging from 0 to 100. A value < 0 means that progress
         *                 is not known, a value >=100 means that the operation has finished running
         *
         * @see reload
         */
        void showOperationProgress(const QString &operation, int progress);

        /**
         * Call this function when the backend streams additional details about the operation in progress
         *
         * @param message the message
         */
        void addTransactionMessage(const QString &message);

        /**
         * Call this function when the backend requires user intervention and needs the user to answer
         * a question. The question is handled asynchronously. To learn more about transaction questions, please
         * see the documentation for the \c TransactionQuestion class.
         *
         * @see transactionQuestionAnswered
         * @see TransactionQuestion
         */
        void newTransactionQuestion(Shaman::TransactionQuestion *question);

        void newDistUpgrade(Shaman::DistUpgrade *distUpgrade);

    protected:
        /**
         * BackendInterface's constructor. The constructor is meant to stay empty, please move anything you
         * would put inside it into \c init.
         *
         * \see init
         */
        BackendInterface(QObject* parent = 0);

        /**
         * \returns the main backend object
         */
        Backend *backend() const;

        /**
         * Sets the capabilities of the backend. This function is crucial, since the library should
         * know what your backend is capable of. This function is meant to be called once, and inside \c init.
         * Failure in doing so might lead to undefined behavior.
         *
         * @param capabilities a set of Shaman::Backend::Capability
         *
         * \see init
         */
        void setCapabilities(BackendCapabilities::TransactionCapabilities transactionCapabilities,
                             BackendCapabilities::SearchCapabilities searchCapabilities,
                             BackendCapabilities::PackageGroupingCapabilities packageGroupingCapabilities,
                             BackendCapabilities::InformationCapabilities informationCapabilities);

        /**
         * Sets a \c KAboutData for your backend. This function is meant to be called once, and inside \c init.
         * Failure in doing so might lead to undefined behavior.
         *
         * \note Setting an about data is advised but not compulsory
         *
         * \param data an about data for your backend
         *
         * \see init
         */
        void setProperties(const QString &name, const QString &description,
                           const QIcon &icon, const QIcon &packageIcon,
                           const QList<KAboutPerson> &authors, const QStringList &configurationModules);

        void setAvailableProcessQueueOptions(const Shaman::ProcessQueueOption::List &options);

        Shaman::MetaPackage *metaPackage(const QString &name);

    private:
        class Private;
        Private * const d;

        friend class Backend;
        friend class BackendPrivate;
        friend class PluginLoader;
        friend class TransactionInterface;
        friend class TransactionQuestion;
        friend class BackendPackage;
        friend class LocalPackage;
        friend class CoreInterface;
};

}

Q_DECLARE_INTERFACE(Shaman::BackendInterface, "org.kde.shaman.BackendInterface/0.1")

#endif // BACKENDINTERFACE_H
