/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamanbackendpackage_p.h"

#include "shamanbackendinterface.h"
#include "shamanpackagegroup.h"
#include "shamanrepository.h"
#include "shamanbase.h"
#include "shamancoreinterface.h"
#include "shamanbackend_p.h"
#include "shamanglobalpackagewatcher.h"
#include "shamanpackagewatcher.h"

#include <KDebug>

namespace Shaman {

BackendPackage::BackendPackage(const QString &name, BackendInterface *parent)
  : AbstractPackage(*(new BackendPackagePrivate(name)), parent)
{
    qRegisterMetaType<BackendPackage*>();
    qRegisterMetaType<Shaman::BackendPackage::List>();
    qRegisterMetaType<Shaman::BackendPackage::Action>("Shaman::Package::Action");
    qRegisterMetaType<Shaman::BackendPackage::State>("Shaman::Package::State");
}

BackendPackage::BackendPackage(const QString &name, const QString &description, const QString &version , const State &state,
                 BackendInterface *parent)
  : AbstractPackage(*(new BackendPackagePrivate(name, description, version, state)), parent)
{
    qRegisterMetaType<BackendPackage*>();
    qRegisterMetaType<Shaman::BackendPackage::List>();
    qRegisterMetaType<Shaman::BackendPackage::Action>("Shaman::Package::Action");
    qRegisterMetaType<Shaman::BackendPackage::State>("Shaman::Package::State");
}

BackendPackage::~BackendPackage()
{
}

QIcon BackendPackage::icon() const
{
    Q_D(const BackendPackage);
    if (d->icon.isNull()) {
        return Base::instance()->core()->backend()->properties()->packageIcon();
    } else {
        return d->icon;
    }
}

void BackendPackage::setDownloadSize(KIO::filesize_t size)
{
    Q_D(BackendPackage);
    d->downloadSize = size;
    if (d->values & cachedValueForAdditionalCacheValue(DownloadSize)) {
        d->values ^= cachedValueForAdditionalCacheValue(DownloadSize);
        EMIT_VALUE_CACHED(cachedValueForAdditionalCacheValue(DownloadSize));
    }
}

KIO::filesize_t BackendPackage::downloadSize()
{
    Q_D(BackendPackage);
    if (d->values & cachedValueForAdditionalCacheValue(DownloadSize)) {
        cacheValue(cachedValueForAdditionalCacheValue(DownloadSize));
    }
    return d->downloadSize;
}

void BackendPackage::setState(State state)
{
    Q_D(BackendPackage);
    if (d->state == state) {
        return;
    }

    if (d->state == Upgradeable) {
        Base::instance()->core()->backend()->d->removeUpgradeablePackage(this);
    } else if (state == Upgradeable) {
        Base::instance()->core()->backend()->d->addUpgradeablePackage(this);
    }

    d->state = state;

    foreach(PackageWatcher *watcher, d->watchers) {
        emit watcher->stateChanged(d->state);
    }
    emit GlobalPackageWatcher::instance()->stateChanged(this, d->state);
}

Shaman::PackageGroup::List BackendPackage::packageGroups()
{
    Backend * backend = Shaman::Base::instance()->core()->backend();
    if (!backend)
        return Shaman::PackageGroup::List();
    Shaman::PackageGroup::List list;
    foreach (PackageGroup * group, backend->packageGroups())
    {
        if (group->contains(this)) {
            list.append(group);
        }
    }
    return list;
}

Shaman::Repository::List BackendPackage::repositories()
{
    Backend * backend = Shaman::Base::instance()->core()->backend();
    if (!backend)
        return Shaman::Repository::List();
    Shaman::Repository::List list;
    foreach (Repository * repo, backend->repositories())
    {
        if (repo->contains(this))
            list.append(repo);
    }
    return list;
}

BackendPackage::AdditionalCacheValue BackendPackage::additionalValueForValue(AbstractPackage::CachedValue value)
{
    if (value == AbstractPackage::AdditionalValue1) {
        return BackendPackage::DownloadSize;
    } else {
        return BackendPackage::NoValue;
    }
}

AbstractPackage::CachedValue BackendPackage::cachedValueForAdditionalCacheValue(BackendPackage::AdditionalCacheValue value)
{
    if (value == DownloadSize) {
        return AbstractPackage::AdditionalValue1;
    } else {
        return AbstractPackage::NoValue;
    }
}

}
