/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANBACKENDPACKAGE_H
#define SHAMANBACKENDPACKAGE_H

#include "shamanmacros.h"
#include <QObject>
#include <QString>
#include <KUrl>

#include <kio/global.h>
#include <QIcon>
#include "shamanabstractpackage.h"

namespace Shaman {
class BackendInterface;
class PackageGroup;
class Repository;
class WidgetFactory;
class BackendPackagePrivate;
/** \class Package Package.h
 * \brief Shaman's Package Class.
 *
 * This is the generic Package class of Shaman. In order to use it, every plugin needs a reimplementation
 * of this.
 * The Backend should set the following properties directly after initializing: State, Name, Version, PackageGroups and Repository. Others,
 * for example providers() have to be reimplemented.
 *
 * \code
 * class FooPackage : public Shaman::Package
 * {
 *     Q_OBJECT
 *     public:
 *         FooPackage()
 *           : Package(i18n("foo"), i18n("This is a foo package"), "1.0alpha6", Package::NotInstalled)
 *         {
 *         }
 *
 *         ~FooPackage()
 *         {}
 *
 *         virtual Shaman::Package::List dependencies()
 *         {
 *             return Shaman::Package::List();
 *         }
 * }
 * \endcode
 *
 * This is an example implementation inside a Backend with a reimplementation of dependencies().
 */
class SHAMAN_EXPORT BackendPackage : public AbstractPackage
{
    Q_DECLARE_PRIVATE(BackendPackage)
    public:
        enum AdditionalCacheValue {
            NoValue = 0,
            DownloadSize = 1
        };
        Q_DECLARE_FLAGS(AdditionalCacheValues, AdditionalCacheValue)
        /**
         * \class List Package.h
         *
         * This class is a typedef for a QList<Package*>.
         */
        typedef QList<BackendPackage*> List;
        virtual ~BackendPackage();

    public:
        QIcon icon() const;

        KIO::filesize_t downloadSize();

        /**
         * @returns a list of all PackageGroups which are containing this package
         */
        virtual QList<Shaman::PackageGroup*> packageGroups();

        /**
         * @returns a list of all Repositories which are containing this package
         */
        virtual QList<Shaman::Repository*> repositories();

        static AdditionalCacheValue additionalValueForValue(CachedValue value);

        static CachedValue cachedValueForAdditionalCacheValue(AdditionalCacheValue value);

    protected:
        BackendPackage(const QString &name, BackendInterface *parent);
        BackendPackage(const QString &name, const QString &description, const QString &version, const State &state,
                       BackendInterface *parent);

        void setDownloadSize(KIO::filesize_t size);

        void setState(State state);
};
}

Q_DECLARE_METATYPE(Shaman::BackendPackage*)
Q_DECLARE_METATYPE(Shaman::BackendPackage::List)
Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::BackendPackage::AdditionalCacheValues)

#endif
