/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef BASE_H
#define BASE_H

#include <QtCore/QObject>

#include "shamanmacros.h"

class QWidget;
class QDockWidget;
class QAction;
class QStatusBar;

namespace Shaman {

class AbstractPluginLoader;

class GuiInterface;
class CoreInterface;
class StatusBarInterface;
class Backend;
class Provider;

class BasePrivate;
/** \class Base shamanbase.h
 * \brief The base class of libshaman.
 * If a Program wants to use libshaman, this class needs to be initialized.
 * The Program should create the class, then connect the signals to slots, set a Status-Bar using setStatusBar() and then
 * call init().
 *
 * \code
 * Shaman::Base * shaman = new Shaman::Base(this);
 * connect(shaman, SIGNAL(createMainWidget(QWidget*)), SLOT(createMainWidget(QWidget*)));
 * connect(shaman, SIGNAL(createDockWdiget(Qt::DockWidgetArea, QDockWidget*)), SLOT(createDockWidget(Qt::DockWidgetArea, QDockWidget*)));
 * connect(shaman, SIGNAL(createAction(QAction*)), SLOT(createAction(QAction*)));
 * shaman->setStatusBar(statusBar());
 * shaman->init();
 * \endcode
 *
 * This creates a Shaman-base-class, connects all signals to slots, calls setStatusBar() and calls init() after it.
 */
class SHAMAN_EXPORT Base : public QObject
{
    Q_OBJECT
    Q_ENUMS(OperationMode)
    Q_DISABLE_COPY(Base)
    Q_DECLARE_PRIVATE(Base)
    BasePrivate * const d_ptr;

    friend class Backend;
    friend class BackendPrivate;
    friend class PluginConfig;
    friend class BaseHelper;

    Base(QObject * parent = 0);

    Q_PRIVATE_SLOT(d_func(), void __k__slotBackendCreated(Shaman::Backend *backend))
    Q_PRIVATE_SLOT(d_func(), void __k__slotProviderCreated(Shaman::Provider *provider))
    Q_PRIVATE_SLOT(d_func(), void __k__slotSetProviderPlugins())
    Q_PRIVATE_SLOT(d_func(), void __k__initCompleted())
    public:
        /**
         * \enum OperationMode
         * This enum describes the operation modes of the backend
         */
        enum OperationMode
        {
            UncachedMode = 0,
            CachedMode = 1
        };

        static Base *instance();

        ~Base();

        /**
         * @returns the CoreInterface of the initialized libshaman
         */
        CoreInterface * core();

        /**
         * @returns the configuration file that is used by libshaman
         * @see setConfigurationFile()
         */
        KSharedConfig::Ptr configurationFile();

        /**
         * Sets the configuration file that will be used by libshaman.
         * Plugin configuration will be read from this file.
         * @param bar the StatusBar
         * @see configurationFile()
         */
        void setConfigurationFile(KSharedConfig::Ptr cfg);

        /**
         * Informs all plugins that the configuration got changed.
         */
        void setConfigurationChanged();

        void registerPluginLoader(AbstractPluginLoader *loader);

    public Q_SLOTS:
        /**
         * Initializes LibShaman. Calling this function will load all plugins specified with
         * setPlugins() (defaults to all plugins available) and initialize the library for being
         * ready to use.
         */
        void init(OperationMode mode = UncachedMode);

    Q_SIGNALS:
        /**
         * The configuration file changed
         */
        void configChanged();

        void initCompleted();
};
}

#endif
