/* This file is part of the KDE project

   Copyright (C) 2008-2010 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamancoreinterface_p.h"

#include "shamanbackend_p.h"
#include "shamanabstractpackage.h"
#include "shamanmacros_p.h"
#include "shamanquery_p.h"
#include "shamanbackendinterface.h"

#include <QSet>
#include <KDebug>
#include "shamanprovider_p.h"
#include "shamanproviderplugin.h"

namespace Shaman {

void CoreInterfacePrivate::queryCompleted(QObject *sender, const QUuid &uuid, const Shaman::AbstractPackage::List &packages)
{
    // Get the query from the query pool first
    Query *query = QueryPool::instance()->fromUuid(uuid);
    if (!query) {
        return;
    }

    query->d_func()->partialResult.append(packages);
    query->d_func()->availableTargets.removeOne(sender);

    if (query->d_func()->availableTargets.isEmpty()) {
        // Process the result
        QSet<MetaPackage*> retset;
        foreach (AbstractPackage *p, query->d_func()->partialResult) {
            retset.unite(p->metaPackages().toSet());
        }
        query->d_func()->metapackages = retset.toList();
        query->d_func()->setState(Shaman::Query::Completed);
    }
}

void CoreInterfacePrivate::cleanUnusedMetaPackages()
{
    QHash<QString, MetaPackage*>::iterator i;
    for (i = metaPackages.begin(); i != metaPackages.end(); ++i) {
        if (i.value()->packages().isEmpty()) {
            MetaPackage *toRemove = i.value();
            i = metaPackages.erase(i);
            delete toRemove;
        }
    }
}

MetaPackage* CoreInterfacePrivate::metaPackage(const QString& name)
{
    if (!metaPackages.contains(name)) {
        // Create a new one and add it to the cache
        Q_Q(CoreInterface);
        MetaPackage *package = new MetaPackage(name, q);
        metaPackages[name] = package;
    }

    return metaPackages.value(name);
}

AbstractPackage::List CoreInterfacePrivate::packagesFor(QObject* object) const
{
    return packages.value(object);
}

void CoreInterfacePrivate::__k__packagesChanged(const Shaman::AbstractPackage::List& packages)
{
    kDebug() << "Packages changed for" << sender();
    setPackagesFor(sender(), packages);
}

void CoreInterfacePrivate::setPackagesFor(QObject* object, const Shaman::AbstractPackage::List& pkgs)
{
    Q_Q(CoreInterface);
    packages[object] = pkgs;
    PERFORMANCE_MONITOR_START("Adding metapackages")
    // Now fill metapackages
    foreach (Shaman::AbstractPackage *package, pkgs) {
        if (package->metaPackages().isEmpty()) {
            // Create a new metaPackage for it
            metaPackage(package->name())->addPackage(package);
        }
    }
    PERFORMANCE_MONITOR_STOP("Adding metapackages")
    emit q->metaPackagesChanged(q->metaPackages());
}

////////

CoreInterface::CoreInterface(QObject * parent)
  : QObject(parent),
    d_ptr(new CoreInterfacePrivate(this))
{
}

CoreInterface::~CoreInterface()
{
}

Backend * CoreInterface::backend()
{
    Q_D(CoreInterface);
    return d->backend;
}

TransactionInterface* CoreInterface::transactionInterface()
{
    Q_D(CoreInterface);
    return d->backend->d->transactionInterface();
}

void CoreInterfacePrivate::setBackend(Backend *newbackend)
{
    backend = newbackend;
    // Watch over the backend
    if (!backend->packages().isEmpty()) {
        setPackagesFor(backend, backend->packages());
    }
    connect(newbackend, SIGNAL(packagesChanged(Shaman::AbstractPackage::List)),
            this, SLOT(__k__packagesChanged(Shaman::AbstractPackage::List)));
}

Provider::List CoreInterface::providers() const
{
    Q_D(const CoreInterface);
    return d->providers;
}

void CoreInterfacePrivate::setProviders(Provider::List newProviderPlugins)
{
    foreach (Provider *p, providers) {
        disconnect(p, SIGNAL(availablePackagesChanged(Shaman::AbstractPackage::List)),
                   this, SLOT(__k__packagesChanged(Shaman::AbstractPackage::List)));
    }
    providers = newProviderPlugins;
    foreach (Provider *p, providers) {
        if (!p->availablePackages().isEmpty()) {
            setPackagesFor(p, p->availablePackages().toList());
        }
        connect(p, SIGNAL(availablePackagesChanged(Shaman::AbstractPackage::List)),
                this, SLOT(__k__packagesChanged(Shaman::AbstractPackage::List)));
    }

    Q_Q(CoreInterface);
    emit q->providersChanged(providers);
}

void CoreInterface::setCurrentPackage(AbstractPackage *package)
{
    Q_D(CoreInterface);
    d->currentPackage = package;
    emit currentPackageChanged(package);
}

AbstractPackage * CoreInterface::currentPackage()
{
    Q_D(CoreInterface);
    return d->currentPackage;
}

MetaPackage::List CoreInterface::metaPackages() const
{
    Q_D(const CoreInterface);

    return d->metaPackages.values();
}

void CoreInterface::performQuery(Query* query)
{
    Q_D(CoreInterface);
    query->d_func()->setState(Shaman::Query::Running);
    query->d_func()->availableTargets.clear();

    // Add targets list first
    if (query->targets() & Shaman::Query::BackendTarget || query->targets() & Shaman::Query::AllTargets) {
        query->d_func()->availableTargets.append(d->backend->d->interface);
    }
    if (query->targets() & Shaman::Query::ProvidersTarget || query->targets() & Shaman::Query::AllTargets) {
        foreach (Provider *p, providers()) {
            query->d_func()->availableTargets.append(p->d_func()->plugin);
        }
    }

    // Then spawn the queries
    if (query->targets() & Shaman::Query::BackendTarget || query->targets() & Shaman::Query::AllTargets) {
        // Inject the query on the backend
        d->backend->d->interface->searchPackages(query);
    }
    if (query->targets() & Shaman::Query::ProvidersTarget || query->targets() & Shaman::Query::AllTargets) {
        // Inject the query on the providers
        foreach (Provider *p, providers()) {
            p->d_func()->plugin->search(query);
        }
    }
}

}

#include "shamancoreinterface.moc"
#include "shamancoreinterface_p.moc"
