/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANCOREINTERFACE_H
#define SHAMANCOREINTERFACE_H

#include "shamanmacros.h"
#include <QObject>
#include "shamanprovider.h"
#include "shamanmetapackage.h"

namespace Shaman {

class Query;
class TransactionInterface;
class Backend;
class AbstractPackage;

class CoreInterfacePrivate;
/**
 * \class CoreInterface shamancoreinterface.h
 * \brief Shaman's CoreInterface class.
 *
 * This class is the connection from the Gui-Plugins to the Core,
 * aka Backend-Layer.
 *
 * \code
 * foreach (Shaman::Package * package, backend()->packages())
 * {
 *     package->setAction(Shaman::Package::Install);
 * }
 * \endcode
 *
 * This markes all Packages for Installing
 */
class SHAMAN_EXPORT CoreInterface : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(CoreInterface)
    Q_DECLARE_PRIVATE(CoreInterface)
    CoreInterfacePrivate * const d_ptr;

    friend class BasePrivate;
    friend class BackendInterface;
    friend class ProviderPlugin;

    Q_PRIVATE_SLOT(d_func(), void __k__packagesChanged(const Shaman::AbstractPackage::List &packages))

    public:
        CoreInterface(QObject * parent);
        ~CoreInterface();

    public Q_SLOTS:
        /**
         * @returns the current selected Package
         * @see setCurrentPackage()
         */
        AbstractPackage * currentPackage();

        /**
         * Returns the current Backend. The type of it is the BackendInterface and not Backend,
         * cause the Interface handles some threading stuff in it.
         * @returns the current BackendInterface
         */
        Backend * backend();

        /**
         * Returns the current TransactionInterface, which can be used to monitor the transaction's progress
         * @returns the current TransactionInterface
         */
        TransactionInterface * transactionInterface();

        /**
         * Returns a list of all loaded Provider-Plugins
         * @returns a list of ProviderPlugins
         */
        Provider::List providers() const;

        /**
         * Sets the current Package. This function should be used by the Plugin which is providing the PackagesView.
         * @see currentPackage()
         * @param package the current Package
         */
        void setCurrentPackage(AbstractPackage *package);

        Shaman::MetaPackage::List metaPackages() const;

        void performQuery(Shaman::Query *query);

    Q_SIGNALS:
        /**
         * This gets emitted when the list of ProviderPlugins changed.
         * @see providerPlugins()
         */
        void providersChanged(const Shaman::Provider::List &providerPlugins);

        /**
         * This gets emitted when the current Package changed.
         * @see currentPackage()
         * @see setCurrentPackage()
         */
        void currentPackageChanged(Shaman::AbstractPackage * package);

        void metaPackagesChanged(const Shaman::MetaPackage::List &metaPackages);
};
}
#endif

