/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef COREPLUGIN_H
#define COREPLUGIN_H

#include <shamanabstractplugin.h>
#include "shamanmacros.h"

namespace Shaman {

class CoreInterface;

class CorePluginPrivate;
class SHAMAN_EXPORT CorePlugin : public AbstractPlugin
{
    Q_OBJECT
    Q_DISABLE_COPY(CorePlugin)
    Q_DECLARE_PRIVATE(CorePlugin)
    Q_ENUMS(Hook)
    Q_ENUMS(HookResult)

    friend class PluginLoader;
    friend class GuiPlugin;

    public:
        enum Hook {
            NoHooks = 0,
            PreTransactionHook = 1,
            PostTransactionHook = 2,
            PreUpdateRepositoriesHook = 4,
            PostUpdateRepositoriesHook = 8,
            PostBackendReloadedHook = 16
        };
        Q_DECLARE_FLAGS(Hooks, Hook)
        Q_FLAGS(Hooks)
        enum HookResult {
            Unknown = 0,
            Success = 1,
            SuccessWithWarning = 2,
            Error = 3,
            Fatal = 4
        };

        CorePlugin(QObject *parent);
        ~CorePlugin();

    public Q_SLOTS:
        /**
         * This method gives the Plugin access to the CoreInterface.
         * @returns the CoreInterface
         */
        CoreInterface * core();

        /**
         * \returns the hooks registered by this plugin
         */
        Hooks registeredHooks() const;

    protected:
        /**
         * Registers a hook, and connects it to the function you specified in \c member.
         * The functions should have a matching signature for the hook. In particular,
         * transaction hooks should accept a Shaman::Package::List; database hooks should accept
         * a Shaman::Repository::List, and backend reloaded hooks should accept no arguments. A working example is:
         * \code
         * void preTransactionHook(const Shaman::Package::List &packages);
         * void postUpdateDatabaseHook(const Shaman::Repository::List &databases);
         * void backendReloaded();
         * \endcode
         * The hooks will be guaranteed to be called one at a time. Pre- hooks are able to abort the 
         * action before it takes place, if needed.
         * \par
         * To register one of the hooks above, simply do the following:
         * \code
         * registerHook(CorePlugin::PreTransactionHook, "preTransactionHook");
         * \endcode
         * \par
         * Inside the registered hook you will NEED to emit the signal \c hookCompleted when you're
         * done, otherwise you'll freeze Shaman. This is done to let you perform long actions asynchronously
         * without freezing the UI.
         * \par
         * \note You are able to register a single hook for each type. Calling \c registerHook two times
         *       for the same hook type will cause the first registration to be overwritten
         *
         * \param hook The hook to be registered
         * \param member The function that will serve as the hook
         *
         * \see hookCompleted
         */
        void registerHook(Hook hook, const char *member);
        void hookMessage(const QString &message);
        /**
         * Unregisters a previously registered hook
         *
         * \param hook The hook to unregister
         */
        void unregisterHook(Hook hook);
        void unregisterAllHooks();

    Q_SIGNALS:
        /**
         * Emit this signal when an hook, registered with \c registerHook, has finished running.
         * Emitting this signal is compulsory, otherwise Shaman will wait forever for the hook to finish.
         *
         * \param result The result of the hook. If it is Fatal and the hook is a pre- one, the transaction
         *               will be aborted
         */
        void hookCompleted(CorePlugin::Hook hook, CorePlugin::HookResult result);

        /**
         * Emitted whenever a hook gets registered. You do not need to call this signal: it is handled
         * internally
         */
        void hookRegistered(CorePlugin::Hook hook, const char *member);

        /**
         *
         */
        void hookUnregistered(CorePlugin::Hook hook);

    protected:
        CorePlugin(CorePluginPrivate &dd, QObject *parent = 0);
};
}

Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::CorePlugin::Hooks)

#endif // COREPLUGIN_H
