/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef DISTUPGRADE_H
#define DISTUPGRADE_H

#include <QtCore/QObject>
#include <QtGui/QIcon>
#include "shamantransactioninterface.h"

namespace Shaman {

class TransactionQuestion;

class WidgetFactory;
class DistUpgradePrivate;
/**
 * \class DistUpgrade shamandistupgrade.h
 *
 * \brief A class representing a dist upgrade
 *
 * \par What is a dist upgrade?
 *
 * From Shaman point of view, a dist upgrade is a special transaction in which everything is customizable
 * by the backend. A dist upgrade can consist in anything: most of the time is a special upgrade which you
 * can't represent with a common transaction. For this reason, it implements its own extendable state machine,
 * its own progress report, and some more things to get feedback from the user.
 *
 * \par Handling a dist upgrade
 *
 * To handle a dist upgrade you want to listen to \c Shaman::Backend::newDistroUpgradeAvailable. Once you get the 
 * DistUpgrade object, you might need to check if it requires a setup or not (in most cases it will). If it does,
 * create the widget from the given \c WidgetFactory. When setup is over, start the dist upgrade by calling \c start()
 * and passing as the argument the widget created for the setup. If there was no need for setups, just pass it 0 or no arguments.
 * To monitor progress, monitor (as usual) the state changes and listen to the signals. Each state can represent a download
 * or miscellaneous progress. Watch out for errors, and when then state gets back to \c NotRunning, you're done.
 *
 * \note You can't start a DistUpgrade twice: you have to consider the object invalid as it returns back to \c NotRunning. If
 * there has been an error, the backend will take care of creating a new DistUpgrade.
 */
class DistUpgrade : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(DistUpgrade)
    Q_DECLARE_PRIVATE(DistUpgrade)
    Q_ENUMS(State)
    public:
        enum State {
            NotRunning = 0,
            CustomState1 = 1,
            CustomState2 = 2,
            CustomState3 = 3,
            CustomState4 = 4,
            CustomState5 = 5,
            CustomState6 = 6,
            CustomState7 = 7
        };
        Q_DECLARE_FLAGS(States, State)

        enum ViewType {
            ProgressView,
            DownloadView
        };

        DistUpgrade(QObject* parent = 0);
        virtual ~DistUpgrade();

        QString name() const;
        QString description() const;

        States availableStates() const;

        State state() const;

        QString customStateName(State customState) const;
        QString customStateIconOrMovieName(State customState) const;
        ViewType customStateViewType(State customState) const;

        WidgetFactory *details() const;

        bool needsSetup();
        QList<WidgetFactory*> setup() const;

    public Q_SLOTS:
        virtual void start(QWidget *setupResult = 0) = 0;
        virtual void questionAnswered(Shaman::TransactionQuestion *question) = 0;

    Q_SIGNALS:
        void stateChanged(State state);
        void errorTriggered(Shaman::TransactionInterface::Error error, const QString &details);

        void entityDownloadProgressChanged(const QString &name, qint64 bytesDownloaded, qint64 totalBytes);
        void downloadProgressChanged(qint64 bytesDownloaded, qint64 totalBytes);
        void progressChanged(int percent);

        void newQuestion(Shaman::TransactionQuestion *question);
        void newMessage(const QString &message, Shaman::TransactionInterface::MessageLevel level);

    protected:
        void setName(const QString &name);
        void setDescription(const QString &description);
        void setAvailableStates(States states);
        void setState(State state);

        void setCustomStateName(State customState, const QString &name);
        void setCustomStateIconOrMovieByName(State customState, const QString &name);
        void setCustomStateViewType(State customState, ViewType type);

        void setDetails(WidgetFactory *details);
        void setSetup(WidgetFactory *setup);
        void setSetup(QList<WidgetFactory*> setup);

    private:
        DistUpgradePrivate * const d_ptr;
};

}

#endif // DISTUPGRADE_H
