/* This file is part of the KDE project

   Copyright (C) 2008-2010 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef SHAMANMACROS_H
#define SHAMANMACROS_H

/** \file shamanmacros.h
    \brief Contains Macros used in Shaman

    This contains some macros that help in exporting/importing symbols and plugins
*/

#include <kdemacros.h>
#include <KPluginFactory>
#include <KPluginLoader>

#define SHAMAN_EXPORT __attribute__ ((visibility("default")))

#define SHAMAN_PLUGIN_EXPORT( c ) \
    K_PLUGIN_FACTORY( ShamanFactory, registerPlugin< c >(); ) \
    K_EXPORT_PLUGIN( ShamanFactory("c") )

#define SHAMAN_PLUGIN_CONFIG_EXPORT( c ) \
    K_PLUGIN_FACTORY( ShamanFactory, registerPlugin< c >(); ) \
    K_EXPORT_PLUGIN( ShamanFactory( "c" ) )

#define SHAMAN_PLUGIN_IMPORT( C ) qobject_cast<AbstractPlugin *>(C)
#define SHAMAN_GUI_PLUGIN_IMPORT( C ) qobject_cast<GuiPlugin *>(C)
#define SHAMAN_SCRIPTED_PLUGINS_INSTALL_DIR "${SCRIPTED_PLUGINS_INSTALL_DIR}"

// Versioning stuff
/// @brief LibShaman version as string at compile time.
#define LIBSHAMAN_VERSION_STRING "${CMAKE_LIBSHAMAN_VERSION_STRING}"

/// @brief The major LibShaman version number at compile time
#define LIBSHAMAN_VERSION_MAJOR ${CMAKE_LIBSHAMAN_VERSION_MAJOR}

/// @brief The minor LibShaman version number at compile time
#define LIBSHAMAN_VERSION_MINOR ${CMAKE_LIBSHAMAN_VERSION_MINOR}

/// @brief The LibShaman release version number at compile time
#define LIBSHAMAN_VERSION_RELEASE ${CMAKE_LIBSHAMAN_VERSION_RELEASE}

/// @brief The name of the backend used by LibShaman
#define LIBSHAMAN_BACKEND_NAME ${SHAMAN_BACKEND}

/**
 * \brief Create a unique number from the major, minor and release number of a %LibShaman version
 *
 * This function can be used for preprocessing. For version information at runtime
 * use the version methods in the Shaman namespace.
 */
#define LIBSHAMAN_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))

/**
 * \brief %LibShaman Version as a unique number at compile time
 *
 * This macro calculates the %LibShaman version into a number. It is mainly used
 * through LIBSHAMAN_IS_VERSION in preprocessing. For version information at runtime
 * use the version methods in the Shaman namespace.
 */
#define LIBSHAMAN_VERSION \
    LIBSHAMAN_MAKE_VERSION(LIBSHAMAN_VERSION_MAJOR,LIBSHAMAN_VERSION_MINOR,LIBSHAMAN_VERSION_RELEASE)

/**
 * \brief Check if the %LibShaman version matches a certain version or is higher
 *
 * This macro is typically used to compile conditionally a part of code:
 * \code
 * #if LIBSHAMAN_IS_VERSION(2,1)
 * // Code for LibShaman 2.1
 * #else
 * // Code for LibShaman 2.0
 * #endif
 * \endcode
 *
 * For version information at runtime
 * use the version methods in the LibShaman namespace.
 */
#define LIBSHAMAN_IS_VERSION(a,b,c) ( LIBSHAMAN_VERSION >= LIBSHAMAN_MAKE_VERSION(a,b,c) )


namespace Shaman {
    /**
     * @brief Returns the major number of LibShaman's version, e.g.
     * 1 for %LibShaman 1.0.2.
     * @return the major version number at runtime.
     */
    SHAMAN_EXPORT unsigned int versionMajor();

    /**
     * @brief Returns the minor number of LibShaman's version, e.g.
     * 0 for %LibShaman 1.0.2.
     * @return the minor version number at runtime.
     */
    SHAMAN_EXPORT unsigned int versionMinor();

    /**
     * @brief Returns the release of LibShaman's version, e.g.
     * 2 for %LibShaman 1.0.2.
     * @return the release number at runtime.
     */
    SHAMAN_EXPORT unsigned int versionRelease();

    /**
     * @brief Returns the %LibShaman version as string, e.g. "1.0.2".
     *
     * On contrary to the macro LIBSHAMAN_VERSION_STRING this function returns
     * the version number of LibShaman at runtime.
     * @return the %LibShaman version. You can keep the string forever
     */
    SHAMAN_EXPORT const char* versionString();
}

#endif /*SHAMANMACROS_H*/
