/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANREPOSITORY_H
#define SHAMANREPOSITORY_H

#include "shamanpackagegroup.h"
#include "shamanmacros.h"
#include <QObject>
#include <KUrl>
#include <QString>

namespace Shaman {
/** \class Repository shamanrepository.h
 * \brief Shaman's Repository class.
 *
 * This is Shaman's Repository class. It does not need to be reimplemented.
 * Basically it's a Package-Group, but extended by a State-Enum and the Url-Property.
 * When updating the Repository-Database the Backend should set the current State of it.
 *
 * \code
 * Repository * repository = new Repository("Repository", KUrl("http://server.com/repo/i686"), this);
 * repository->append(package);
 * \endcode
 *
 * This creates an empty repository with the Name "Repository" and the Url "http://server.com/repo/i686".
 * A package gets appended afterwards.
 */
class SHAMAN_EXPORT Repository : public PackageGroup
{
    Q_OBJECT
    Q_ENUMS(State)
    Q_PROPERTY(QUrl url READ url WRITE setUrl)
    public:
        /**
         * \enum State
         * This enum describes the state of the Repository.
         * @see setState()
         */
        enum State
        {
            Idling,
            Downloading,
            Updating,
            UpdatingFinished,
            Error
        };
        typedef QList<Repository*> List;
        Repository(const QString &name, const KUrl &url, QObject *parent);
        Repository(const QString &name, const QIcon &icon, const KUrl &url, QObject *parent);
        virtual ~Repository();

    public Q_SLOTS:
        /**
         * Sets the Url of the Repository.
         * @param url the new Url
         */
        void setUrl(const KUrl &url);

        /**
         * @return the Url of the Repository
         */
        KUrl url() const;

        /**
         * Sets the state of the Repository 
         * @param status the new Status
         */
        void setState(State status);

        /**
         * @returns the State of the Repository
         */
        State state();

    Q_SIGNALS:
        /**
         * This signal gets automatically emitted when the Repository-State changed.
         * The Backend should use setState().
         * @see setState()
         */
        void stateChanged(Repository::State state);

    private:
        class Private;
        Private *d;
};
}
Q_DECLARE_METATYPE(Shaman::Repository::State)
Q_DECLARE_METATYPE(Shaman::Repository*)
Q_DECLARE_METATYPE(Shaman::Repository::List)

#endif
