/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "shamantransactioninterface_p.h"
#include <klocalizedstring.h>
#include "shamanbase.h"
#include "shamancoreinterface.h"
#include "shamanbackend.h"
#include <KDebug>
#include "shamanbackend_p.h"
#include "shamanbackendinterface.h"
#include "shamanbackendpackage.h"

namespace Shaman {

TransactionInterface::TransactionInterface(QObject * parent)
  : QObject(parent),
    d(new Private())
{
    d->q_ptr = this;
    qRegisterMetaType<Shaman::TransactionInterface::State>();
    qRegisterMetaType<Shaman::TransactionInterface::Error>();
    qRegisterMetaType<TransactionQuestion*>();
}

TransactionInterface::~TransactionInterface()
{
    delete d;
}

void TransactionInterface::Private::setPackageDownloadProgress(Shaman::AbstractPackage *package, qint64 downloadedBytes, qint64
totalBytes)
{
    Q_Q(TransactionInterface);
    if (automaticComputations & TotalDownloadedSize) {
        totalDownloaded += downloadedBytes - downloadStatus[package];
    }
    downloadStatus[package] = downloadedBytes;
    if (automaticComputations & DownloadSpeed) {
        // Compute speed changes
        qint64 newspeed = (totalDownloaded / downloadTime.elapsed()) * 1000;
        if (downloadSpeed != newspeed) {
            downloadSpeed = newspeed;
            emit q->downloadSpeedChanged(downloadSpeed);
            if (automaticComputations & RemainingTime) {
                // If the speed changed, the remaining time is quite likely to have changed as well
                remainingTime = QTime::currentTime();
                if (downloadSpeed) {
                    remainingTime.addSecs((totalBytes - downloadedBytes) / downloadSpeed);
                }
                emit q->remainingTimeChanged(remainingTime);
            }
        }
    }
    emit q->packageDownloadProgressChanged(package, downloadedBytes, totalBytes);
    emit q->downloadProgressChanged(totalDownloaded, totalDownloadSize);
}

void TransactionInterface::Private::setPackages(const Shaman::AbstractPackage::List& newpackages)
{
    transactionMessages.clear();
    packages = newpackages;
    totalDownloadSize = 0;
    totalDownloaded = 0;
    Shaman::AbstractPackage::List dlpackages;
    // Cache the total download size
    foreach (Shaman::AbstractPackage *package, packages) {
        if (package->action() == Shaman::AbstractPackage::Install ||
            package->action() == Shaman::AbstractPackage::TransactionDependencyInstall ||
            package->action() == Shaman::AbstractPackage::Upgrade ||
            package->action() == Shaman::AbstractPackage::TransactionDependencyUpgrade) {
            BackendPackage *p = dynamic_cast< Shaman::BackendPackage* >(package);
            if (p) {
                totalDownloadSize += p->downloadSize();
                dlpackages.append(p);
            }
        }
    }
    // Cache the percentage influence of each package
    if (dlpackages.size() < 100 && dlpackages.size() != 0) {
        packagePercentageInfluence = 100 / dlpackages.size();
    } else {
        // Simply 0
        packagePercentageInfluence = 0;
    }
}

void TransactionInterface::Private::setPackageTransactionProgress(Shaman::AbstractPackage *package, int percent)
{
    Q_Q(TransactionInterface);

    emit q->packageTransactionProgressChanged(package, percent);

    if (!transactionPackages.contains(package)) {
        transactionPackages.append(package);

        QString message;

        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
                message = i18n("Installing %1...", package->name());
                break;
            case Shaman::AbstractPackage::Uninstall:
                message = i18n("Removing %1...", package->name());
                break;
            case Shaman::AbstractPackage::Upgrade:
                message = i18n("Upgrading %1...", package->name());
                break;
            case Shaman::AbstractPackage::TransactionDependencyInstall:
                message = i18n("Installing %1 as a dependency...", package->name());
                break;
            case Shaman::AbstractPackage::TransactionDependencyUninstall:
                message = i18n("Removing %1 as a dependency...", package->name());
                break;
            case Shaman::AbstractPackage::TransactionDependencyUpgrade:
                message = i18n("Upgrading %1 as a dependency...", package->name());
                break;
            default:
                break;
        }

        if (!message.isEmpty()) {
            transactionMessages.append(qMakePair<QString, MessageLevel>(message, TransactionStatusLevel));
            emit q->newMessage(message, TransactionStatusLevel);
        }
    }

    int newTransactionProgress = 0;

    if (percent >= 100) {
        newTransactionProgress = (transactionPackages.size() * 100) / (packages.size());

        QString message;

        switch (package->action()) {
            case Shaman::AbstractPackage::Install:
            case Shaman::AbstractPackage::TransactionDependencyInstall:
                message = i18n("%1 has been installed successfully", package->name());
                break;
            case Shaman::AbstractPackage::Uninstall:
            case Shaman::AbstractPackage::TransactionDependencyUninstall:
                message = i18n("%1 has been removed successfully", package->name());
                break;
            case Shaman::AbstractPackage::Upgrade:
            case Shaman::AbstractPackage::TransactionDependencyUpgrade:
                message = i18n("%1 has been upgraded successfully", package->name());
                break;
            default:
                break;
        }

        if (!message.isEmpty()) {
            transactionMessages.append(qMakePair<QString, MessageLevel>(message, TransactionStatusLevel));
            emit q->newMessage(message, TransactionStatusLevel);
        }
    } else {
        if (transactionPackages.size() > 0 && automaticComputations & TransactionProgress) {
            newTransactionProgress = ((transactionPackages.size() - 1) * 100) / (packages.size());
            newTransactionProgress += (packagePercentageInfluence * percent) / 100;
        }
    }

    if (transactionProgress != newTransactionProgress && automaticComputations & TransactionProgress) {
        transactionProgress = newTransactionProgress;
        emit q->transactionProgressChanged(transactionProgress);
    }
}

void TransactionInterface::Private::setTransactionError(Shaman::TransactionInterface::Error newerror, const QString& details)
{
    Q_Q(TransactionInterface);
    error = newerror;
    errorDetails = details;
    emit q->errorTriggered(error, details);
    // Append a message as well
    emit q->newMessage(i18n("There has been an error: %1", details), TransactionStatusLevel);
}

void TransactionInterface::Private::setTransactionState(Shaman::TransactionInterface::State newstate)
{
    Q_Q(TransactionInterface);
    if (state == NoTransaction && newstate != NoTransaction) {
        elapsedTime.restart();
    }

    if (state != Downloading && newstate == Downloading) {
        // Reset the download starting time
        downloadTime = QTime::currentTime();
    }

    if (state != newstate) {
        state = newstate;
        // Reload also the automation as well
        automaticComputations = TotalDownloadedSize | DownloadSpeed | RemainingTime | TransactionProgress;
        emit q->stateChanged(newstate);
    }
}

void TransactionInterface::Private::newTransactionMessageFromBackend(const QString& message)
{
    Q_Q(TransactionInterface);
    transactionMessages.append(qMakePair<QString, MessageLevel>(message, BackendMessageLevel));
    emit q->newMessage(message, BackendMessageLevel);
}

void TransactionInterface::Private::newTransactionMessageFromHook(const QString& message)
{
    Q_Q(TransactionInterface);
    transactionMessages.append(qMakePair<QString, MessageLevel>(message, HookMessageLevel));
    emit q->newMessage(message, HookMessageLevel);
}

void TransactionInterface::Private::newTransactionQuestion(Shaman::TransactionQuestion *question)
{
    Q_Q(TransactionInterface);
    transactionQuestions.append(question);
    emit q->newTransactionQuestion(question);
}

void TransactionInterface::Private::setTotalDownloadedSize(qint64 downloadedBytes)
{
    Q_Q(TransactionInterface);
    automaticComputations ^= TotalDownloadedSize;
    if (totalDownloaded != downloadedBytes) {
        totalDownloaded = downloadedBytes;
        emit q->downloadProgressChanged(totalDownloaded, totalDownloadSize);
    }
}

void TransactionInterface::Private::setTotalDownloadSize(qint64 totalBytes)
{
    Q_Q(TransactionInterface);
    totalDownloadSize = totalBytes;
    emit q->downloadProgressChanged(totalDownloaded, totalDownloadSize);
}

void TransactionInterface::Private::setDownloadSpeed(qint64 bytesPerSecond)
{
    Q_Q(TransactionInterface);
    automaticComputations ^= DownloadSpeed;
    if (downloadSpeed != bytesPerSecond) {
        downloadSpeed = bytesPerSecond;
        emit q->downloadSpeedChanged(downloadSpeed);
        if (automaticComputations & RemainingTime) {
            // If the speed changed, the remaining time is quite likely to have changed as well
            remainingTime = QTime::currentTime();
            if (downloadSpeed) {
                remainingTime.addSecs((totalDownloadSize - totalDownloaded) / downloadSpeed);
            }
            emit q->remainingTimeChanged(remainingTime);
        }
    }
    downloadSpeed = bytesPerSecond;
}

void TransactionInterface::Private::setRemainingTime(const QTime &dueToFinishAt)
{
    Q_Q(TransactionInterface);
    automaticComputations ^= RemainingTime;
    remainingTime = dueToFinishAt;
    emit q->remainingTimeChanged(remainingTime);
}

void TransactionInterface::Private::reset()
{
    // Restore it back to default
    state = TransactionInterface::NoTransaction;
    error = TransactionInterface::NoError;
    automaticComputations = TotalDownloadedSize | DownloadSpeed | RemainingTime | TransactionProgress;
    errorDetails.clear();
    packages.clear();
    transactionMessages.clear();
    packagePercentageInfluence = 0;
    totalDownloadSize = 0;
    totalDownloaded = 0;
    elapsedTime = QTime();
    downloadTime = QTime();
    remainingTime = QTime();
    downloadStatus.clear();
    transactionPackages.clear();
    transactionProgress = 0;
    transactionQuestions.clear();
    if (!downloadReliable) {
        Q_Q(TransactionInterface);
        downloadReliable = true;
        emit q->downloadProgressReliabilityChanged(downloadReliable);
    }
}

void TransactionInterface::Private::setTransactionProgress(int newTransactionProgress)
{
    Q_Q(TransactionInterface);
    automaticComputations ^= TransactionProgress;
    if (transactionProgress != newTransactionProgress) {
        transactionProgress = newTransactionProgress;
        emit q->transactionProgressChanged(transactionProgress);
    }
    if (state == Downloading) {
        automaticComputations ^= DownloadSpeed;
        automaticComputations ^= TotalDownloadedSize;
        downloadReliable = false;
        emit q->downloadProgressReliabilityChanged(downloadReliable);
    }
}

Shaman::TransactionInterface::State TransactionInterface::state()
{
    return d->state;
}

Shaman::TransactionInterface::Error TransactionInterface::lastError() const
{
    return d->error;
}

QList< QPair< QString, Shaman::TransactionInterface::MessageLevel > > TransactionInterface::transactionMessages() const
{
    return d->transactionMessages;
}

Shaman::AbstractPackage::List TransactionInterface::downloadedPackages() const
{
    return d->downloadStatus.keys();
}

QPair< qint64, qint64 > TransactionInterface::downloadProgressAsBytes() const
{
    return qMakePair<qint64, qint64>(d->totalDownloaded, d->totalDownloadSize);
}

int TransactionInterface::downloadProgressAsPercentage() const
{
    return ((d->totalDownloaded * 100) / d->totalDownloadSize);
}

int TransactionInterface::downloadSpeed() const
{
    return d->downloadSpeed;
}

QTime TransactionInterface::elapsedTime() const
{
    return d->elapsedTime;
}

QString TransactionInterface::lastErrorDetails() const
{
    return d->errorDetails;
}

Shaman::AbstractPackage::List TransactionInterface::packages() const
{
    return d->packages;
}

Shaman::AbstractPackage::List TransactionInterface::processedPackages() const
{
    return d->transactionPackages;
}

QTime TransactionInterface::remainingTime() const
{
    return d->remainingTime;
}

int TransactionInterface::transactionProgress() const
{
    return d->transactionProgress;
}

QList<Shaman::TransactionQuestion*> TransactionInterface::transactionQuestions() const
{
    return d->transactionQuestions;
}

bool TransactionInterface::isDownloadProgressReliable() const
{
    return d->downloadReliable;
}

}

#include "shamantransactioninterface.moc"
