/* This file is part of the KDE project

   Copyright (C) 2008-2009 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef TRANSACTIONINTERFACE_H
#define TRANSACTIONINTERFACE_H

#include "shamanmacros.h"
#include "shamanabstractpackage.h"

#include <QObject>
#include <QTime>
#include <QIcon>
#include "shamantransactionquestion.h"

namespace Shaman {
/**
 * \class TransactionInterface shamantransactioninterface.h
 * \brief Shaman's TransactionInterface class.
 * 
 * This class is the Monitor of the Transaction-Progress when
 * processing the Queue. The Backend should try to set 
 * all information it can get to the TransactionInterface.
 * This class is directly used in the GuiPlugins.
 *
 */
class SHAMAN_EXPORT TransactionInterface : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(TransactionInterface)
    Q_ENUMS(State)
    Q_ENUMS(Error)
    Q_ENUMS(TransactionStatusLevel)
    Q_ENUMS(TransactionQuestionAnswer)
    public:
        /**
         * \enum State
         * This enum describes the current State of the Transaction.
         */
        enum State
        {
            /**
             * No Transaction is running.
             */
            NoTransaction = 0,
            RunningPreHooks = 1,
            /**
             * The Backend is current downloading the Packages.
             */
            Downloading = 2,
            /**
             * The Backend is current installing or removing Packages.
             */
            Processing = 3,
            /**
             * The Backend is current cleaning the install environment.
             */
            Cleaning = 4,
            ProcessingProvider = 5,
            RunningPostHooks = 6
        };
        /**
         * \enum Error
         * This enum describes the error which occurred in the Transaction.
         */
        enum Error
        {
            /**
             * No Error triggered.
             */
            NoError = 0,
            /**
             * The Download of the Packages failed.
             */
            DownloadFailedError = 1,
            /**
             * The processing of the Queue failed. This is a generic error when
             * none of the next ones will suit the description
             */
            ProcessingFailedError = 2,
            /**
             * The cleanup failed.
             */
            CleanupFailedError = 3,
            /**
             * A file conflict got detected.
             */
            FileConflictError = 4,
            /**
             * A dependency Conflict got detected.
             */
            DependencyConflictError = 5,
            /**
             *
             */
            DependenciesUnsatisfiedError = 6,

            BuildError = 7,
            /**
             * An error in the backend was found
             */
            BackendError = 8,
            /**
             * An error in the provider was found
             */
            ProviderError = 9,
            HookError = 10,
            CanceledByUser = 255,
            /**
             * An unknown Error occurred.
             */
            UnknownError = 256
        };

        enum MessageLevel {
            TransactionStatusLevel = 0,
            HookMessageLevel = 1,
            BackendMessageLevel = 2
        };

        /**
         * The descructor of the TransactionInterface.
         */
        ~TransactionInterface();

        /**
         * @returns the current State of the TransactionInterface
         */
        State state();

        /**
         * @returns the current Error of the TransactionInterface
         */
        Error lastError() const;

        /**
         * @returns details about the last errors, if any
         */
        QString lastErrorDetails() const;

        /**
         * @returns a list of Packages on which the Backend is working in this Transaction
         */
        Shaman::AbstractPackage::List packages() const;

        /**
         * @returns the total Progress of the Transaction in percent
         */
        int downloadProgressAsPercentage() const;

        QPair<qint64, qint64> downloadProgressAsBytes() const;

        Shaman::AbstractPackage::List downloadedPackages() const;

        /**
         * @returns the progress of the current Package in percent
         */
        int transactionProgress() const;

        Shaman::AbstractPackage::List processedPackages() const;

        /**
         * @returns the current Download-Speed in Bytes/s
         */
        int downloadSpeed() const;

        bool isDownloadProgressReliable() const;

        /**
         * @returns the already elapsed time of the Transaction
         */
        QTime elapsedTime() const;

        /**
         * @returns the remaining time of the Transaction
         */
        QTime remainingTime() const;

        QList< QPair<QString, Shaman::TransactionInterface::MessageLevel> > transactionMessages() const;

        QList<Shaman::TransactionQuestion*> transactionQuestions() const;

    Q_SIGNALS:
        /**
         * This signal gets automatically emitted when the State changed.
         */
        void stateChanged(Shaman::TransactionInterface::State state);
        /**
         * This signal gets automatically emitted when the Error changed.
         */
        void errorTriggered(Shaman::TransactionInterface::Error error, const QString &details);
        /**
         * This signal gets automatically emitted when the current Package changed.
         */
        void packageDownloadProgressChanged(Shaman::AbstractPackage *package, qint64 downloadedBytes, qint64 totalBytes);
        /**
         * This signal gets automatically emitted when the total Progress changed.
         */
        void downloadProgressChanged(qint64 downloadedBytes, qint64 totalBytes);
        /**
         * This signal gets automatically emitted when the progress of the transaction changes.
         */
        void transactionProgressChanged(int percent);
        void packageTransactionProgressChanged(Shaman::AbstractPackage *package, int percent);
        /**
         * This signal gets automatically emitted when the Download-Speed changed.
         */
        void downloadSpeedChanged(int bytespersecond);
        /**
         * This signal gets automatically emitted when the remaining Time changed.
         */
        void remainingTimeChanged(const QTime &time);
        void downloadProgressReliabilityChanged(bool isReliable);
        /**
         * Emitted whenever there's a new message describing the transaction in progress
         */
        void newMessage(const QString &message, Shaman::TransactionInterface::MessageLevel level);

        /**
         * This signal will be emitted whenever a new question is streamed from the backend
         *
         * @see TransactionQuestion
         */
        void newTransactionQuestion(Shaman::TransactionQuestion *question);

    private:
        /**
         * Constructs a new TransactionInterface.
         * @param parent the Parent of the TransactionInterface
         */
        TransactionInterface(QObject * parent);

        class Private;
        Private * const d;

        friend class Backend;
        friend class BackendPrivate;
        friend class BackendInterface;
        friend class CorePlugin;
        friend class ManualProgressReporter;
};
}

Q_DECLARE_METATYPE(Shaman::TransactionQuestion*)
Q_DECLARE_METATYPE(Shaman::TransactionInterface::State)
Q_DECLARE_METATYPE(Shaman::TransactionInterface::Error)

#endif
