/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#ifndef TRANSACTIONQUESTION_H
#define TRANSACTIONQUESTION_H

#include <QFlags>
#include <QMetaObject>
#include <QString>
#include <QIcon>
#include "shamanmacros.h"
#include <QtCore/quuid.h>

class QWidget;
namespace Shaman {

class WidgetFactory;

class TransactionQuestionPrivate;
/**
 * \class TransactionQuestion shamantransactionquestion.h
 * \brief Represents a transaction question
 *
 * \c TransactionQuestion is the way libshaman handles communication between the backend and the user during
 * transactions, if interaction is needed. As all the other components in libshaman, it is designed with the
 * maximum possible flexibility. In fact, TransactionQuestion is mostly based on QWidget
 *
 * \par TransactionQuestion and WidgetFactory
 * A question in libshaman is represented by a widget, and widgets are represented through \c WidgetFactory, that
 * allows to create an instance of a representation. Some backends and some systems (like debconf) might require
 * custom widgets and custom return values, hence the reason for making questions widget-based. If you're looking
 * into a very simple question though, you will probably fit yourself better with \c StringTransactionQuestion.
 *
 * \par How to stream a question with a custom widget from a backend
 * Suppose you want to stream a question carrying two radio buttons and a checkbox, that accepts Yes and Cancel as
 * answers. First you would have to create your WidgetFactory, let's call it MyWidgetFactory, and represent your widget
 * \code
 * QWidget * MyWidgetFactory::createInstance() const
 * {
 *  QWidget *w = new QWidget();
 *  QLabel *l = new QLabel("Hey answer this!");
 *  QRadioButton *b1 = new QRadioButton("choice 1");
 *  b1->setObjectName("choice1Radio");
 *  QRadioButton *b2 = new QRadioButton("choice 2");
 *  b2->setObjectName("choice2Radio");
 *  QCheckBox *cb = new QCheckBox("check me");
 *  cb->setObjectName("checkBoxCheck");
 *
 *  QVBoxLayout *lay = new QVBoxLayout();
 *  // Now add stuff to the layout
 *  w->setLayout(lay);
 *  return w;
 * }
 * \endcode
 * Note the \c setObjectName calls: they are extremely important. Done that, you can stream your question when needed with
 * \code
 * Shaman::TransactionQuestion *question = new Shaman::TransactionQuestion(new MyWidgetFactory,
 *                                                                         Shaman::TransactionQuestion::AnswerYes |
 *                                                                         Shaman::TransactionQuestion::AnswerCancel);
 * newTransactionQuestion(question);
 * \endcode
 * Ok. When the user answers your question, the callback "transactionQuestionAnswered" will be triggered in the backend.
 * You can handle the question in the following way:
 * \code
 * bool hasCheck = question->answerWidgetByName<QCheckBox*>("checkBoxCheck")->isChecked();
 * bool radio1Check = question->answerWidgetByName<QRadioButton*>("choice1Radio")->isChecked();
 * bool radio2Check = question->answerWidgetByName<QRadioButton*>("choice2Radio")->isChecked();
 * switch (question->answer) {
 *   // Do your thing
 * \endcode
 * As you can see, the widget will be parsed back based on the names you issued to the objects. This way, you will be able
 * to retrieve every information available in which you're interested. When you finished processing the question, you might
 * want to delete it
 *
 * \par Handling questions from an interface
 * You need to listen to \c TransactionInterface 's \c newTransactionQuestion signal, and display the question accordingly
 * by creating a widget through \c questionWidgetFactory. Once you get an answer, call \c setAnswerWidget giving back the
 * widget which was displayed (and possibly modified) in your dialog, and afterwards \c setAnswer specifying the answer given.
 * Be aware that after you call \c setAnswer you must consider the question object and the widget passed to \c setAnswerWidget
 * as invalid, since the backend might have deleted them.
 */
class SHAMAN_EXPORT TransactionQuestion
{
    Q_DECLARE_PRIVATE(TransactionQuestion)
    public:
        enum Answer {
            UnknownAnswer = 0,
            AnswerYes = 1,
            AnswerNo = 2,
            AnswerCancel = 4,
            AnswerUser1 = 8,
            AnswerUser2 = 16,
            AnswerUser3 = 32
        };
        Q_DECLARE_FLAGS(Answers, Answer)

        explicit TransactionQuestion(WidgetFactory *factory, Answers possibleAnswers);
        explicit TransactionQuestion(const QMetaObject &metaobject, Answers possibleAnswers);
        virtual ~TransactionQuestion();

        QUuid id() const;

        WidgetFactory *questionWidgetFactory() const;

        void setAnswerWidget(QWidget *widget);
        QWidget *answerWidget() const;
        template<class T> T *answerWidgetByName(const QString &name);

        void setAnswer(Answer answer);
        Answer answer() const;
        Answers possibleAnswers() const;

        void setUser1Text(const QString &text);
        QString user1Text() const;
        void setUser1Icon(const QIcon &icon);
        QIcon user1Icon() const;

        void setUser2Text(const QString &text);
        QString user2Text() const;
        void setUser2Icon(const QIcon &icon);
        QIcon user2Icon() const;

        void setUser3Text(const QString &text);
        QString user3Text() const;
        void setUser3Icon(const QIcon &icon);
        QIcon user3Icon() const;

    private:
        TransactionQuestionPrivate * const d_ptr;
};

/**
 * \class StringTransactionQuestion shamantransactionquestion.h
 * \brief Convenience class for simple questions and non modal questions
 *
 * You can use \c StringTransactionQuestion to deliver simple questions, in which you're just interested in knowing
 * the answer and you just have to display a string and optionally an icon. It is a representation of a question asked
 * through a messagebox. More than that, \c StringTransactionQuestion also gives the possibility to deliver non-modal
 * questions, that will be displayed, for example, in a notification, to avoid interrupting the user's workflow.
 *
 * \par About non-modal questions
 * Only questions that are not important to the transaction's operativity should be streamed as non-modals. In fact, this
 * kind of questions (due to their nature, think about a question being displayed in a persistent notification) can even
 * not be answered at all. A nice use case for a non-modal question is a restart request after a transaction.
 */
class SHAMAN_EXPORT StringTransactionQuestion : public TransactionQuestion
{
    public:
         explicit StringTransactionQuestion(const QString & text, Answers possibleAnswers,
                                            const QIcon &icon = QIcon(), bool modal = true);
         virtual ~StringTransactionQuestion();

         void setModal(bool modal);
         bool modal() const;

         QString text() const;

         QIcon icon() const;

    private:
        class Private;
        Private * const d;
};

}

Q_DECLARE_OPERATORS_FOR_FLAGS(Shaman::TransactionQuestion::Answers)

#endif // TRANSACTIONQUESTION_H
