/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "TransactionLogger.h"
#include <shamancoreinterface.h>
#include <KDateTime>
#include <KTar>
#include <klocalizedstring.h>

SHAMAN_PLUGIN_EXPORT(TransactionLogger)

TransactionLogger::TransactionLogger(QObject* parent, const QVariantList& )
        : CorePlugin(parent)
        , m_isOn(false)
{
    connect(core()->backend(), SIGNAL(stateChanged(Shaman::Backend::State)),
            this, SLOT(stateChanged(Shaman::Backend::State)));
}

TransactionLogger::~TransactionLogger()
{

}

void TransactionLogger::init()
{
    registerHook(PreUpdateRepositoriesHook, "preUpdateHook");
    registerHook(PreTransactionHook, "preTransactionHook");
    m_log = KSharedConfig::openConfig("shamantransactionlogrc");
}

void TransactionLogger::preTransactionHook(const Shaman::AbstractPackage::List &list)
{
    Q_UNUSED(list)

    if (config().readEntry("logTransactions", true)) {
        makeRoomForLog();
        m_startTime = KDateTime::currentLocalDateTime();
        m_type = "queue";
        m_isOn = true;
        hookMessage(i18n("Logging transaction"));
    }

    emit hookCompleted(PreTransactionHook, Success);
}

void TransactionLogger::preUpdateHook(const Shaman::Repository::List &list)
{
    Q_UNUSED(list)

    if (config().readEntry("logRepoUpdates", false)) {
        makeRoomForLog();
        m_startTime = KDateTime::currentLocalDateTime();
        m_type = "repoupdate";
        m_isOn = true;
        hookMessage(i18n("Logging repository update"));
    }

    emit hookCompleted(PreUpdateRepositoriesHook, Success);
}

void TransactionLogger::makeRoomForLog()
{
    KConfigGroup cfg(m_log, "Logs");
    if (cfg.groupList().size() >= config().readEntry("logSize", 100) && config().readEntry("logSize", 100) > 0) {
        // We need to make room
        if (config().readEntry("sizeExceededAction", "overwrite") == "overwrite") {
            // Ok, let's remove the first one
            KConfigGroup delgrp = cfg.group(cfg.groupList().first());
            delgrp.deleteGroup();
            delgrp.config()->sync();
            cfg.config()->sync();
        } else {
            // Ok, so we need to archive the group. Let's see where
            QString path = config().readEntry("archivePath", QString());
            path.append(QString("shaman_log_archive_%1").arg(KDateTime::currentLocalDateTime().toString()));
            KSharedConfig::Ptr archiveCfg = KSharedConfig::openConfig(path);
            cfg.copyTo(archiveCfg.data());
            archiveCfg.data()->sync();
            // Compressed?
            if (config().readEntry("archiveCompress", true)) {
                QString arpath = config().readEntry("archivePath", QString());
                arpath.append(QString("shaman_log_archive_%1.tgz").arg(KDateTime::currentLocalDateTime().toString()));
                KTar compressed(arpath);
                compressed.open(QIODevice::WriteOnly);
                compressed.addLocalFile(path, QString());
                compressed.close();
            }

            // Now, erase
            cfg.deleteGroup();
            cfg.config()->sync();
        }
    }
}

void TransactionLogger::stateChanged(Shaman::Backend::State state)
{
    if (state == Shaman::Backend::Idling && m_isOn) {
        // Finished!
        m_isOn = false;
        KConfigGroup cfg(m_log, "Logs");
        KConfigGroup log = cfg.group(QString::number(m_startTime.toTime_t()));
        log.writeEntry("EndTime", KDateTime::currentLocalDateTime().toTime_t());
        log.writeEntry("Type", m_type);
        QString transaction;

        QList< QPair< QString, Shaman::TransactionInterface::MessageLevel > > messages =
                                                        core()->transactionInterface()->transactionMessages();
        QList< QPair< QString, Shaman::TransactionInterface::MessageLevel > >::const_iterator i;
        for (i = messages.constBegin(); i != messages.constEnd(); ++i) {
            switch ((*i).second) {
                case TransactionInterface::TransactionStatusLevel:
                    transaction.append("<b>" + (*i).first + "</b>");
                    break;
                case TransactionInterface::HookMessageLevel:
                    transaction.append((*i).first);
                    break;
                case TransactionInterface::BackendMessageLevel:
                    transaction.append("<i>" + (*i).first + "</i>");
                    break;
                default:
                    transaction.append((*i).first);
                    break;
            }
            transaction.append("<br>");
        }

        log.writeEntry("TransactionLog", transaction);
        log.config()->sync();
    }
}

