/* This file is part of the KDE project

   Copyright (C) 2008 Wang Kai <fearee@gmail.com>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackageDetailDockWidget.h"

#include <shamanbackend.h>

#include <QHeaderView>
#include <KIcon>
#include <KLocale>
#include <KDebug>
#include <KAboutData>
#include <shamanwidgetfactory.h>
#include <QPainter>
#include <shamanpackagewatcher.h>

PackageDetailDockWidget::PackageDetailDockWidget(Shaman::Backend * backend, QWidget * parent)
  : QDockWidget(parent),
    m_backend(backend),
    depends(0),
    requiredBy(0),
    conflicts(0),
    providers(0),
    currentPackage(0)
{
    setupUi(this);
    connect(backend, SIGNAL(stateChanged(Shaman::Backend::State)),
            this, SLOT(backendStateChanged(Shaman::Backend::State)));
    reset();
    backendStateChanged(backend->state());
    ktitlewidget->setPixmap(backend->properties()->packageIcon().pixmap(64), KTitleWidget::ImageLeft);
    relationTreeWidget->header()->setResizeMode(QHeaderView::ResizeToContents);
    filelistSearchLine->searchLine()->setTreeWidget(filelistTreeWidget);
    if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowDependsOnCapability) {
        depends = new QTreeWidgetItem(relationTreeWidget, QStringList(i18n("Depends on")));
    }
    if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowRequiredByCapability) {
        requiredBy = new QTreeWidgetItem(relationTreeWidget, QStringList(i18n("Required by")));
    }
    if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowConflictsWithCapability) {
        conflicts = new QTreeWidgetItem(relationTreeWidget, QStringList(i18n("Conflicts with")));
    }
    if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowProvidesCapability) {
        providers = new QTreeWidgetItem(relationTreeWidget, QStringList(i18n("Provided by")));
    }
    connect(packageDetailKTabWidget, SIGNAL(currentChanged(int)), SLOT(slotCurrentChanged(int)));
}

PackageDetailDockWidget::~PackageDetailDockWidget()
{
}

void PackageDetailDockWidget::slotCurrentChanged(int idx)
{
    QString grps, repos, file;
    QStringList slist;

    if (m_backend->state() == Shaman::Backend::Reloading || m_backend->state() == Shaman::Backend::WaitingForLock || !currentPackage) {
        return;
    } else {
        if (m_pages[idx] == "details") {
            ktitlewidget->setText(currentPackage->name());
            ksqueezedtextlabel->setText(currentPackage->description());
            versionLabel->setText(currentPackage->version());
            availableVersionLabel->setText(currentPackage->availableVersion());
            if (currentPackage->packageGroups().size())
            {
                grps = currentPackage->packageGroups()[0]->name();
                for (int i = 1; i < currentPackage->packageGroups().size(); i++) {
                    grps += ", ";
                    grps += currentPackage->packageGroups()[i]->name();
                }
            }
            grpLabel->setText(grps);
            if (currentPackage->repositories().size())
            {
                repos = currentPackage->repositories()[0]->name();
                for (int i = 1; i < currentPackage->repositories().size(); i++) {
                    repos += ", ";
                    repos += currentPackage->repositories()[i]->name();
                }
            }
            repoLabel->setText(repos);
            stateChanged();
            actionChanged();
        } else if (m_pages[idx] == "filelist") {
            filelistTreeWidget->clear();
            //foreach (file, )
            //    new QTreeWidgetItem(filelistTreeWidget, QStringList(file));
            foreach (const QString &file, currentPackage->fileList())
            {
                QStringList splitted = file.split(QChar('/'));
                QTreeWidgetItem *parentItem = 0;
                foreach (const QString &spl, splitted)
                {
                    if (spl.isEmpty()) {
                        continue;
                    }
                    if (parentItem)
                    {
                        bool there = false;
                        int j = parentItem->childCount();
                        for (int i = 0; i != j; i++)
                        {
                            if (parentItem->child(i)->text(0) == spl)
                            {
                                there = true;
                                parentItem = parentItem->child(i);
                                break;
                            }
                        }
                        if (!there) {
                            parentItem = new QTreeWidgetItem(parentItem, QStringList(spl));
                        }
                    }
                    else
                    {
                        QList<QTreeWidgetItem*> list = filelistTreeWidget->findItems(spl, Qt::MatchExactly);
                        if (!list.isEmpty())
                        {
                            parentItem = list.first();
                        }
                        else
                        {
                            parentItem = new QTreeWidgetItem(filelistTreeWidget, QStringList(spl));
                        }
                    }
                }
            }
        } else if (m_pages[idx] == "relations") {
            if (depends) {
                fillItems(depends, currentPackage->dependencies());
            }
            if (requiredBy) {
                fillItems(requiredBy, currentPackage->requiredBy());
            }
            if (conflicts) {
                fillItems(conflicts, currentPackage->conflictsWith());
            }
            if (providers) {
                fillItems(providers, currentPackage->providers());
            }
        } else if (m_pages[idx] == "changelog") {
            changelogBrowser->setText(currentPackage->changelog());
        }
    }
}

void PackageDetailDockWidget::slotCurrentPackageChanged(Shaman::AbstractPackage *pkg)
{
    kDebug();
    if (!m_currentWatcher.isNull()) {
        m_currentWatcher.data()->deleteLater();
    }
    if (! pkg) {
        reset();
        return;
    }
    currentPackage = pkg;
    QPixmap packagePixmap = currentPackage->icon().pixmap(64);

    if (currentPackage->properties() & Shaman::AbstractPackage::DebugInformationProperty) {
        // Paint a very nice bug over the package :)
        // Let's create a painter to paint the action icon over the key icon
        QPainter painter(&packagePixmap);
        const int iconSize = packagePixmap.size().width();
        // the the emblem icon to size 32
        int overlaySize = 32;
        // try to load the action icon
        const QPixmap pixmap = KIconLoader::global()->loadIcon("tools-report-bug",
                                                               KIconLoader::NoGroup,
                                                               overlaySize,
                                                               KIconLoader::DefaultState,
                                                               QStringList(),
                                                               0,
                                                               true);
        // if we're able to load the icon, paint it over!
        if (!pixmap.isNull()) {
            QPoint startPoint;
            // bottom right corner
            startPoint = QPoint(iconSize - overlaySize - 2,
                                iconSize - overlaySize - 2);
            painter.drawPixmap(startPoint, pixmap);
        }
    }
    if (currentPackage->type() == Shaman::AbstractPackage::ProviderPackageType) {
        // Paint a very nice bug over the package :)
        // Let's create a painter to paint the action icon over the key icon
        QPainter painter(&packagePixmap);
        const int iconSize = packagePixmap.size().width();
        // the the emblem icon to size 32
        int overlaySize = 32;
        // try to load the action icon
        const QPixmap pixmap = KIconLoader::global()->loadIcon("run-build",
                                                               KIconLoader::NoGroup,
                                                               overlaySize,
                                                               KIconLoader::DefaultState,
                                                               QStringList(),
                                                               0,
                                                               true);
        // if we're able to load the icon, paint it over!
        if (!pixmap.isNull()) {
            QPoint startPoint;
            // lower right corner
            startPoint = QPoint(0,
                                iconSize - overlaySize - 2);
            painter.drawPixmap(startPoint, pixmap);
        }
    }

    kDebug() << "Type is " << currentPackage->type();
    ktitlewidget->setPixmap(packagePixmap, KTitleWidget::ImageLeft);

    // Create a watcher
    m_currentWatcher = new Shaman::PackageWatcher(currentPackage, this);

    connect(m_currentWatcher.data(), SIGNAL(actionChanged(Shaman::AbstractPackage::Action)), SLOT(actionChanged()));
    connect(m_currentWatcher.data(), SIGNAL(stateChanged(Shaman::AbstractPackage::State)), SLOT(stateChanged()));
    connect(m_currentWatcher.data(), SIGNAL(valueCached(Shaman::AbstractPackage::CachedValue)), this, SLOT(reloadCurrentPackage()));
    if (!(currentPackage->cachedValues() & Shaman::AbstractPackage::CustomValues)) {
        // If it's not cached, cache them now!
        currentPackage->cacheValue(Shaman::AbstractPackage::CustomValues);
    }

    // Check if we have an overaboundance of pages.
    QHash<int, QString>::iterator j = m_pages.begin();
    while (j != m_pages.end()) {
        if (j.value() != "details" && j.value() != "relations" && j.value() != "filelist" && j.value() != "changelog") {
            // Take it away
            packageDetailKTabWidget->removeTab(j.key());
            j = m_pages.erase(j);
        } else {
            ++j;
        }
    }
    // restore the tab pages
    if (packageDetailKTabWidget->count() == 1 || !packageDetailKTabWidget->count()) {
        m_pages.clear();
        packageDetailKTabWidget->removeTab(0);
        m_pages.insert(packageDetailKTabWidget->addTab(detailPage, i18n("Details")), "details");
        if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowFileListCapability) {
            m_pages.insert(packageDetailKTabWidget->addTab(filelistPage, i18n("Filelists")), "filelist");
        }
        if (depends || requiredBy || conflicts || providers) {
            m_pages.insert(packageDetailKTabWidget->addTab(relationPage, i18n("Relations")), "relations");
        }
        if (m_backend->capabilities()->informationCapabilities() & Shaman::BackendCapabilities::ShowPackageChangelog) {
            m_pages.insert(packageDetailKTabWidget->addTab(changelogTab, i18n("Changelog")), "changelog");
        }
    }
    reloadCurrentPackage();
}

void PackageDetailDockWidget::reloadCurrentPackage()
{
    // First check if there were page changes
    QHash<QString, Shaman::WidgetFactory*>::const_iterator i;
    for (i = currentPackage->customValues().constBegin(); i != currentPackage->customValues().constEnd(); ++i)
    {
        if (!m_pages.values().contains(i.key())) {
            m_pages.insert(packageDetailKTabWidget->addTab(i.value()->createInstance(), i.key()), i.key());
        }
    }
    slotCurrentChanged(packageDetailKTabWidget->currentIndex());
}

void PackageDetailDockWidget::actionChanged()
{
    switch (currentPackage->action()) {
        case Shaman::AbstractPackage::Install:
            actionLabel->setText(i18n("Install"));
            actionIconLabel->setPixmap(KIcon("list-add").pixmap(16));
            break;
        case Shaman::AbstractPackage::Uninstall:
            actionLabel->setText(i18n("Uninstall"));
            actionIconLabel->setPixmap(KIcon("list-remove").pixmap(16));
            break;
        case Shaman::AbstractPackage::Upgrade:
            actionLabel->setText(i18n("Upgrade"));
            actionIconLabel->setPixmap(KIcon("view-refresh").pixmap(16));
            break;
        case Shaman::AbstractPackage::NoAction:
            actionLabel->setText(i18n("NoAction"));
            actionIconLabel->setPixmap(KIcon("").pixmap(16));
            break;
    };
}

void PackageDetailDockWidget::stateChanged()
{
    switch (currentPackage->state()) {
        case Shaman::AbstractPackage::Installed:
            statusLabel->setText(i18n("Installed"));
            statusIconLabel->setPixmap(KIcon("user-online").pixmap(16));
            break;
        case Shaman::AbstractPackage::NotInstalled:
            statusLabel->setText(i18n("Not Installed"));
            statusIconLabel->setPixmap(KIcon("user-offline").pixmap(16));
            break;
        case Shaman::AbstractPackage::Upgradeable:
            statusLabel->setText(i18n("Upgradeable"));
            statusIconLabel->setPixmap(KIcon("user-invisible").pixmap(16));
            break;
    };
}

void PackageDetailDockWidget::reset()
{
    m_pages.clear();
    int tabpages = packageDetailKTabWidget->count();
    for (int i = 0; i < tabpages; i++)
        packageDetailKTabWidget->removeTab(0);
    packageDetailKTabWidget->addTab(helpPage, i18n("Help"));
}

void PackageDetailDockWidget::fillItems(QTreeWidgetItem *widget, Shaman::AbstractPackage::List pkgList)
{
    QStringList slist;
    QTreeWidgetItem *treeItem;

    widget->takeChildren();
    foreach (Shaman::AbstractPackage *apkg, pkgList) {
        if (!apkg) {
            continue;
        }
        slist.clear();
        slist.append(apkg->name());
        slist.append(apkg->version());
        slist.append("");
        treeItem = new QTreeWidgetItem(widget, slist);
        switch (apkg->state()) {
            case Shaman::AbstractPackage::Installed:
                treeItem->setIcon(0, KIcon("user-online"));
                break;
            case Shaman::AbstractPackage::NotInstalled:
                treeItem->setIcon(0, KIcon("user-offline"));
                break;
            case Shaman::AbstractPackage::Upgradeable:
                treeItem->setIcon(0, KIcon("user-invisible"));
                break;
        }
        switch (apkg->action()) {
            case Shaman::AbstractPackage::Install:
                treeItem->setIcon(2, KIcon("list-add"));
                break;
            case Shaman::AbstractPackage::Uninstall:
                treeItem->setIcon(2, KIcon("list-remove"));
                break;
            case Shaman::AbstractPackage::Upgrade:
                treeItem->setIcon(2, KIcon("user-invisible"));
                break;
            case Shaman::AbstractPackage::NoAction:
                break;
        }
    }
}

void PackageDetailDockWidget::backendStateChanged(Shaman::Backend::State state)
{
    switch (state) {
        case Shaman::Backend::Searching:
            helpIconLabel->setPixmap(KIcon("system-help").pixmap(64));
            helpLabel->setText(QString());
            helpInfoLabel->setText(QString());
            break;
        case Shaman::Backend::Reloading:
            helpIconLabel->setPixmap(KIcon("system-software-update").pixmap(64));
            helpLabel->setText(i18n("Loading information"));
            helpInfoLabel->setText(i18n("Shaman is currently querying %1, the package management system, "
                                        "to cache the necessary information needed for it to work.\n"
                                        "This should take just a few seconds, you can see more details "
                                        "about what is happening in the status bar." ,m_backend->properties()->name()));
            slotCurrentPackageChanged(0);
            break;
        case Shaman::Backend::WaitingForLock:
            helpIconLabel->setPixmap(KIcon("dialog-warning").pixmap(64));
            helpLabel->setText(i18n("Shaman is waiting for an exclusive lock on the package management system"));
            helpInfoLabel->setText(i18n("To operate correctly, Shaman must be the only application acting on %1, the package "
                                        "management system.\nShaman detected that another running application, such as another "
                                        "frontend or a terminal program, is holding a lock on it. Please close it to resume "
                                        "normal operation", m_backend->properties()->name()));
            slotCurrentPackageChanged(0);
            break;
        default:
            helpIconLabel->setPixmap(KIcon("system-help").pixmap(64));
            helpLabel->setText(i18n("Choose a package to view detailed information about it here"));
            helpInfoLabel->setText(i18n("Shaman is able to provide extended information about a package, such as packages "
                                        "requiring it, its dependencies, a list of the files contained in it, and more. Just "
                                        "click on a package to make its details be shown here"));
            break;
    }
}

