/* This file is part of the KDE project

   Copyright (C) 2008 - 2010 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackagesModel.h"

#include <shamanpackagegroup.h>
#include <shamanrepository.h>

#include <KLocale>
#include <KDebug>
#include <KIcon>

using namespace Shaman;

bool packageItemGreaterThan(PackageItem * a, PackageItem *b)
{
    if (a->package()->name() == "abs")
        kDebug() << "soooooooooo" << (a->package()->state() != Shaman::AbstractPackage::Installed && b->package()->state() == Shaman::AbstractPackage::Installed);
    if ((a->package()->state() != Shaman::AbstractPackage::Installed && b->package()->state() == Shaman::AbstractPackage::Installed) ||
        ((int)a->package()->type() > (int)b->package()->type()) ||
        (a->package()->type() == b->package()->type() && a->package()->version() < b->package()->version()))
        return false;
    return true;
}

Item::Item(MetaPackage * package)
  : AbstractItem(),
    m_package(package)
{
}

Item::~Item()
{
    qDeleteAll(m_children);
}

Shaman::MetaPackage * Item::metaPackage()
{
    return m_package;
}

void Item::append(PackageItem * package)
{
    m_children.append(package);
}

QList<PackageItem*> Item::children()
{
    return m_children;
}

int Item::count()
{
    return m_children.count();
}

void Item::sort()
{
    //kDebug() << "Sort with count" << m_children.count();
    qSort(m_children.begin(), m_children.end(), packageItemGreaterThan);
}

PackageItem::PackageItem(Shaman::AbstractPackage * package, Item * parent)
  : AbstractItem(),
    m_package(package),
    m_parent(parent)
{
}

bool PackageItem::isPackageItem()
{
    return true;
}

Shaman::AbstractPackage * PackageItem::package()
{
    return m_package;
}

Item * PackageItem::parent()
{
    return m_parent;
}

PackagesModel::PackagesModel(Shaman::MetaPackage::List packages, QObject *parent)
  : QAbstractItemModel(parent)
{
    setPackages(packages);
}

PackagesModel::~PackagesModel()
{
    qDeleteAll(m_packages);
}

int PackagesModel::columnCount(const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    return 6;//TODO: What columns do we really want?
}

QVariant PackagesModel::data(const QModelIndex & index, int role) const
{
    AbstractPackage *package = 0;
    if (!index.parent().isValid())
        package = m_packages[index.row()]->metaPackage();
    else
        package = m_packages[index.parent().row()]->children().at(index.row())->package();

    if (role == Qt::DisplayRole)
    {
        switch (index.column())
        {
            //case 0:

            case 1:
                return package->name();
            //case 2:
            //    return package->actionText();
            case 3:
                return package->version();
            case 4:
            {
                QString repositories;
                foreach (Repository * repo, package->repositories())
                {
                    if (repo == package->repositories().last())
                        repositories.append(repo->name());
                    else
                        repositories.append(repo->name() + ", ");
                }
                return repositories;
            }
            case 5:
            {
                QString packageGroups;
                foreach (PackageGroup * group, package->packageGroups())
                {
                    if (group == package->packageGroups().last())
                        packageGroups.append(group->name());
                    else
                        packageGroups.append(group->name() + ", ");
                }
                return packageGroups;
            }
            default:
                return QVariant();
        }
    }
    if (role == Qt::DecorationRole)
    {
        switch (index.column())
        {
            case 0:
            {
                Shaman::AbstractPackage::State state = package->state();
                if (state == Shaman::AbstractPackage::Installed) {
                    return KIcon("user-online").pixmap(16, 16);
                }
                else if (state == Shaman::AbstractPackage::NotInstalled) {
                    return KIcon("user-offline").pixmap(16, 16);
                }
                else if (state == Shaman::AbstractPackage::Upgradeable) {
                    return KIcon("user-invisible").pixmap(16, 16);
                }
            }
            case 2:
            {
                Shaman::AbstractPackage::Action action = package->action();
                if (action == Shaman::AbstractPackage::Install) {
                    return KIcon("list-add").pixmap(16, 16);
                }
                else if (action == Shaman::AbstractPackage::Uninstall) {
                    return KIcon("list-remove").pixmap(16, 16);
                }
                else if (action == Shaman::AbstractPackage::Upgrade) {
                    return KIcon("user-invisible").pixmap(16, 16);
                }
            }
        }
    }
    return QVariant();
}

QVariant PackagesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    Q_UNUSED(orientation);
    if (role == Qt::DisplayRole)
    {
        switch (section)
        {
            //case 0:
            //    return i18n("Status");
            case 1:
                return i18nc("The name of the Package", "Name");
            //case 2:
            //    return i18n("Action");
            case 3:
                return i18n("Version");
            case 4:
                return i18n("Repositories");
            case 5:
                return i18n("Groups");
        }
    }
    else if (role == Qt::DecorationRole)
    {
        switch (section)
        {
            case 0:
                return KIcon("user-invisible").pixmap(32, 32);
            case 2:
                return KIcon("applications-development").pixmap(32, 32);
            case 4:
                return KIcon("network-server-database").pixmap(32, 32);
            case 5:
                return KIcon("bookmark-new-list").pixmap(32, 32);
        }
    }
    return QVariant();
}

QModelIndex PackagesModel::index(int row, int column, const QModelIndex & parent) const
{
    Q_UNUSED(parent);
    if (row < 0)
        return QModelIndex();
    if (!parent.isValid() && row < m_packages.size())
        return createIndex(row, column, m_packages.at(row));
    else if (parent.isValid() && row < m_packages.at(parent.row())->count()) {
        return createIndex(row, column, m_packages.at(parent.row())->children().at(row));
    }

    return QModelIndex();
}

QModelIndex PackagesModel::parent(const QModelIndex & idx) const
{
    AbstractItem * item = static_cast<AbstractItem*>(idx.internalPointer());
    if (!item->isPackageItem()) { //For the first item in the list this maybe won't work... FIXME: unhack it!
        return QModelIndex();
    }
    return index(m_packages.indexOf(static_cast<PackageItem*>(idx.internalPointer())->parent()), 0, QModelIndex());
}

int PackagesModel::rowCount(const QModelIndex & parent) const
{
    //kDebug() << parent.data(Qt::DisplayRole) << parent.parent().isValid() << parent.isValid();
    if (!parent.isValid())
        return m_packages.count();
    else if (parent.isValid() && !parent.parent().isValid() && m_packages[parent.row()]->count() > 1) {
        return m_packages[parent.row()]->count();
    }
    else
        return 0;
}

Shaman::AbstractPackage * PackagesModel::packageFromModelIndex(const QModelIndex &index)
{
    if (index.row() >= 0 && index.row() < m_packages.count() && !index.parent().isValid())
        return m_packages.at(index.row())->children().first()->package();
    else if (index.row() >= 0 && index.row() < m_packages.at(index.parent().row())->count() && index.parent().isValid())
        return m_packages.at(index.parent().row())->children().at(index.row())->package();
    return 0;
}

Shaman::MetaPackage * PackagesModel::metaPackageFromModelIndex(const QModelIndex &index)
{
    if (index.row() >= 0 && index.row() < m_packages.count() && !index.parent().isValid())
        return m_packages.at(index.row())->metaPackage();
    else if (index.row() >= 0 && index.row() < m_packages.at(index.parent().row())->count() && index.parent().isValid())
        return m_packages.at(index.parent().row())->metaPackage();
    return 0;
}

void PackagesModel::setPackages(const Shaman::MetaPackage::List &pkgList)
{
    kDebug() << "Adding" << pkgList.count() << "metapackages";
    Shaman::AbstractPackage::List pkgs;
    //qDeleteAll(m_packages);
    m_packages.clear();

    foreach (Shaman::MetaPackage* package, pkgList) {
        Item * rootItem = new Item(package);
        m_packages.append(rootItem);

        // Now iterate the all the children
        foreach (Shaman::AbstractPackage *childPackage, package->packages()) {
            rootItem->append(new PackageItem(childPackage, rootItem));
        }
    }
    foreach (Item * item, m_packages) {
        item->sort();
    }
    reset();
}

void PackagesModel::watch(Shaman::AbstractPackage::List &pkgList)
{
    kDebug();
    foreach (QPointer<Shaman::PackageWatcher> p, m_watchers) {
        p.data()->deleteLater();
        delete p;
    }
    
    foreach (Shaman::AbstractPackage * p, pkgList) {
            // Create a watcher
        QPointer<Shaman::PackageWatcher> watcher = new Shaman::PackageWatcher(p, this);

        connect(watcher.data(), SIGNAL(actionChanged(Shaman::AbstractPackage::Action)), SLOT(reloadPackage()));
        connect(watcher.data(), SIGNAL(stateChanged(Shaman::AbstractPackage::State)), SLOT(reloadPackage()));
        connect(watcher.data(), SIGNAL(valueCached(Shaman::AbstractPackage::CachedValue)), this, SLOT(reloadPackage()));
        if (!(p->cachedValues() & Shaman::AbstractPackage::CustomValues)) {
            // If it's not cached, cache them now!
            p->cacheValue(Shaman::AbstractPackage::CustomValues);
        }
        m_watchers << watcher;
    }
}

void PackagesModel::reloadPackage()
{
    kDebug();
    Shaman::AbstractPackage * package = qobject_cast<Shaman::PackageWatcher*>(QObject::sender())->package();
    Item * itm = 0;
    foreach (Item * item, m_packages) {
        foreach (PackageItem *i, item->children()) {
            if (i->package() == package) {
                itm = item;
                break;
            }
        }
        if (itm)
            break;
    }
    if (!itm)
        return;
    QModelIndex idx = index(m_packages.indexOf(itm), 0, QModelIndex());
    QModelIndex end = index(itm->count(), columnCount(), index(m_packages.indexOf(itm), 0, QModelIndex()));
    emit dataChanged(idx, end);//TODO: Make the updating of data work
}
