/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackagesSortFilterProxyModel.h"
#include "PackagesModel.h"

#include <shamanrepository.h>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <KDebug>
#include <shamanquery.h>

using namespace Shaman;

PackagesSortFilterProxyModel::PackagesSortFilterProxyModel(QObject *parent)
  : QSortFilterProxyModel(parent),
    m_repo(0),
    m_group(0),
    m_state(PackageStateModel::AllPackages),
    m_searchText(QString()),
    m_searchType(SearchModel::Basic)
{
    m_searchCapabilities = Base::instance()->core()->backend()->capabilities()->searchCapabilities();
}

PackagesSortFilterProxyModel::~PackagesSortFilterProxyModel()
{
}

void PackagesSortFilterProxyModel::setPackageState(PackageStateModel::State state)
{
    m_state = state;
    invalidateFilter();
}

void PackagesSortFilterProxyModel::setRepositoryFilter(Repository * repo)
{
    m_repo = repo;
    invalidateFilter();
}

void PackagesSortFilterProxyModel::setPackageGroupFilter(PackageGroup * group)
{
    m_group = group;
    invalidateFilter();
}

void PackagesSortFilterProxyModel::setSearchType(SearchModel::Type searchType)
{
    m_searchType = searchType;
    if ((m_searchCapabilities & BackendCapabilities::BasicSearchFromBackendCapability && searchType == SearchModel::Basic) ||
        (m_searchCapabilities & BackendCapabilities::ExtendedSearchFromBackendCapability && searchType == SearchModel::Extended))
        setSearchText(m_searchText);
    else
        invalidateFilter();
}

void PackagesSortFilterProxyModel::setSearchText(const QString searchText)
{
    m_searchText = searchText;
    if (m_searchText.isEmpty()) {
        m_packages.clear();
        invalidateFilter();
        emit textSearchCompleted();
        return;
    }
    Query *query = 0;
    if (m_searchCapabilities & BackendCapabilities::BasicSearchFromBackendCapability && m_searchType == SearchModel::Basic) {
        query = new Query(QRegExp(searchText, Qt::CaseInsensitive));
    } else if (m_searchCapabilities & BackendCapabilities::ExtendedSearchFromBackendCapability && m_searchType == SearchModel::Extended) {
        query = new Query(QRegExp(searchText, Qt::CaseInsensitive));
    } else {
        invalidateFilter();
        emit textSearchCompleted();
    }

    if (query) {
        connect(query, SIGNAL(completed(const Shaman::MetaPackage::List&)), this, SLOT(searchCompleted(const Shaman::MetaPackage::List&)));
    }
}

void PackagesSortFilterProxyModel::searchCompleted(const Shaman::MetaPackage::List &packages)
{
    m_packages = packages;
    invalidateFilter();
    emit textSearchCompleted();
}

bool PackagesSortFilterProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
{
    //Our "main"-method
    AbstractPackage *package = static_cast<PackagesModel*>(sourceModel())->packageFromModelIndex(sourceModel()->index(sourceRow, 0, sourceParent));
    //We have a package as internal pointer
    if (!package)
        return false;
    
    bool subitem = false;
    if (!sourceParent.isValid() && sourceModel()->hasChildren(sourceParent)) {
        const QModelIndex &index = sourceModel()->index(sourceRow, 0, sourceParent);
        int i = 0;
        while (index.child(i, 0).isValid()) {
            if ((m_repo && m_repo->contains(static_cast<PackagesModel*>(sourceModel())->packageFromModelIndex(index.child(i, 0)))) 
               || (m_group && m_group->contains(static_cast<PackagesModel*>(sourceModel())->packageFromModelIndex(index.child(i, 0))))) {
                subitem = true;
                break;
            }
            i++;
        }
    }

    bool result = (!m_group || m_group->contains(package) || subitem) && //Look if it's in the group
                  (!m_repo || m_repo->contains(package) || subitem) && //Look if it's in the repository
                  ((m_state == PackageStateModel::AllPackages) ||
                  (package->state() == AbstractPackage::Installed && m_state == PackageStateModel::InstalledPackages) ||
                  (package->state() == AbstractPackage::NotInstalled && m_state == PackageStateModel::NotInstalledPackages) ||
                  (package->state() == AbstractPackage::Upgradeable && m_state == PackageStateModel::UpgradeablePackages) ||
                  (package->action() != AbstractPackage::NoAction && m_state == PackageStateModel::QueuedPackages));
                  

    if ((m_searchCapabilities & BackendCapabilities::BasicSearchFromViewCapability && m_searchType == SearchModel::Basic) ||
               (m_searchCapabilities & BackendCapabilities::ExtendedSearchFromViewCapability && m_searchType == SearchModel::Extended)) {
        return result && (m_searchText.isEmpty() ||
                (m_searchType == SearchModel::Basic && package->name().contains(m_searchText, Qt::CaseInsensitive)) ||
                (m_searchType == SearchModel::Extended && package->description().contains(m_searchText, Qt::CaseInsensitive)));
    } else if ((m_searchCapabilities & BackendCapabilities::BasicSearchFromBackendCapability && m_searchType == SearchModel::Basic) ||
        (m_searchCapabilities & BackendCapabilities::ExtendedSearchFromBackendCapability && m_searchType == SearchModel::Extended)) {
        return result && (m_searchText.isEmpty() || m_packages.isEmpty() || 
                          m_packages.contains(static_cast<PackagesModel*>(sourceModel())->metaPackageFromModelIndex(sourceModel()->index(sourceRow, 1, sourceParent))));
    }

    // Please the compiler
    return false;
}
