/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "PackagesView.h"
#include "PackagesModel.h"
#include "PackagesSortFilterProxyModel.h"
#include <shamanabstractpackage.h>

#include <QPainter>
#include <QPaintEvent>

#include <KAction>
#include <KLocale>
#include <KIcon>
#include <KMenu>
#include <KDebug>
#include <QTimeLine>
#include <shamanbase.h>
#include <shamancoreinterface.h>

PackagesView::PackagesView(QWidget * parent)
  : QTreeView(parent)
  , m_retainPaintEvent(false)
  , m_fadeTimeLine(new QTimeLine(1200, this))
  , m_spinnerTimeLine(new QTimeLine(2000, this))
#if KDE_IS_VERSION(4,3,67)
  , m_sequence("process-working", KIconLoader::SizeSmallMedium)
#endif
  , m_backgroundPixmap(KIconLoader::global()->loadIcon("document-preview-archive", KIconLoader::Desktop,
                       KIconLoader::SizeEnormous))
  , m_cachedState(Shaman::Backend::Idling)
  , m_addingToQueue(false)
{
    setSortingEnabled(true);
    sortByColumn(1, Qt::AscendingOrder);
    //setRootIsDecorated(false);

    // Setup animations
#if KDE_IS_VERSION(4,3,67)
    m_spinnerTimeLine->setFrameRange(0, m_sequence.frameCount());
#else
    loadSpinnerFrames();
    m_spinnerTimeLine->setFrameRange(0, m_spinner.size());
#endif
    m_spinnerTimeLine->setLoopCount(0);
    m_spinnerTimeLine->setCurveShape(QTimeLine::LinearCurve);
    connect(m_spinnerTimeLine, SIGNAL(frameChanged(int)), viewport(), SLOT(repaint()));

    m_fadeTimeLine->setFrameRange(0, 100);
    m_fadeTimeLine->setCurveShape(QTimeLine::EaseOutCurve);
    connect(m_fadeTimeLine, SIGNAL(frameChanged(int)), viewport(), SLOT(repaint()));
    connect(m_fadeTimeLine, SIGNAL(finished()), this, SLOT(stopRetainingPaintEvent()));

    connect(Shaman::Base::instance()->core()->backend(), SIGNAL(stateChanged(Shaman::Backend::State)),
            this, SLOT(backendStateChanged(Shaman::Backend::State)));
    backendStateChanged(Shaman::Base::instance()->core()->backend()->state());

    m_contextMenu = new KMenu(i18n("Actions"), this);

    Shaman::BackendCapabilities::TransactionCapabilities tcap =
                                        Shaman::Base::instance()->core()->backend()->capabilities()->transactionCapabilities();

    if (tcap & Shaman::BackendCapabilities::InstallPackagesCapability) {
        m_installAction = new KAction(this);
        m_installAction.data()->setObjectName("install_selected_packages");
        m_installAction.data()->setText(i18n("Install selected Packages"));
        m_installAction.data()->setIcon(KIcon("list-add"));
        m_installAction.data()->setShortcut(Qt::CTRL + Qt::Key_I);
        connect(m_installAction.data(), SIGNAL(triggered()), SLOT(installCurrentPackages()));
        m_contextMenu->addAction(m_installAction.data());
    }

    if (tcap & Shaman::BackendCapabilities::UpgradeSystemCapability) {
        m_updateAction = new KAction(this);
        m_updateAction.data()->setObjectName("update_selected_packages");
        m_updateAction.data()->setText(i18n("Update selected Packages"));
        m_updateAction.data()->setIcon(KIcon("system-software-update"));
        m_updateAction.data()->setShortcut(Qt::CTRL + Qt::Key_U);
        connect(m_updateAction.data(), SIGNAL(triggered()), SLOT(updateCurrentPackages()));
        m_contextMenu->addAction(m_updateAction.data());
    }

    if (tcap & Shaman::BackendCapabilities::RemovePackagesCapability) {
        m_uninstallAction = new KAction(this);
        m_uninstallAction.data()->setObjectName("uninstall_selected_packages");
        m_uninstallAction.data()->setText(i18n("Uninstall selected Packages"));
        m_uninstallAction.data()->setIcon(KIcon("list-remove"));
        m_uninstallAction.data()->setShortcut(Qt::CTRL + Qt::Key_R);
        connect(m_uninstallAction.data(), SIGNAL(triggered()), SLOT(uninstallCurrentPackages()));
        m_contextMenu->addAction(m_uninstallAction.data());
    }

    m_cancelAction = new KAction(this);
    m_cancelAction->setObjectName("cancel_selected_packages");
    m_cancelAction->setText(i18n("Cancel selected Packages"));
    m_cancelAction->setIcon(KIcon("dialog-cancel"));
    m_cancelAction->setShortcut(Qt::CTRL + Qt::Key_C);
    connect(m_cancelAction, SIGNAL(triggered()), SLOT(cancelCurrentPackages()));
    m_contextMenu->addAction(m_cancelAction);
}

PackagesView::~PackagesView()
{
}

void PackagesView::init()
{
    connect(selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)), 
                              SLOT(currentRowChanged(const QModelIndex &, const QModelIndex &)));
}

void PackagesView::currentRowChanged(const QModelIndex &current, const QModelIndex &previous)
{
    Q_UNUSED(previous);
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    emit currentPackageChanged(packagesModel->packageFromModelIndex(proxyModel->mapToSource(current)));
}

void PackagesView::contextMenuEvent(QContextMenuEvent * e)
{
    Q_UNUSED(e);
    bool cancelEnabled = false;
    bool removeEnabled = true;
    bool installEnabled = true;
    bool updateEnabled = true;
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    //TODO: I know that cast sucks, probably find a better sollution TODO: Sometimes disable Uninstall and sometimes rename install action to reinstall
    Shaman::AbstractPackage::List packages;
    foreach (const QModelIndex &index, selectionModel()->selectedRows())
    {
        if (packagesModel->packageFromModelIndex(proxyModel->mapToSource(index))->action() != Shaman::AbstractPackage::NoAction) {
            cancelEnabled = true;
        }
        Shaman::AbstractPackage * p = packagesModel->packageFromModelIndex(proxyModel->mapToSource(index));
        packages << p;
        Shaman::AbstractPackage::State state = p->state();
        if (state != Shaman::AbstractPackage::Installed && state != Shaman::AbstractPackage::Upgradeable) {
            removeEnabled = false;
        }
        if (state != Shaman::AbstractPackage::Upgradeable) {
            updateEnabled = false;
        }
        if (state != Shaman::AbstractPackage::NotInstalled) {
            installEnabled = false;
        }
    }

    m_cancelAction->setEnabled(cancelEnabled);
    if (!m_installAction.isNull()) {
        m_installAction.data()->setEnabled(installEnabled);
    }
    if (!m_uninstallAction.isNull()) {
        m_uninstallAction.data()->setEnabled(removeEnabled);
    }
    if (!m_updateAction.isNull()) {
        m_updateAction.data()->setEnabled(updateEnabled);
    }
    
    packagesModel->watch(packages);

    m_contextMenu->exec(QCursor::pos());
}

void PackagesView::installCurrentPackages()
{
    m_addingToQueue = true;
    backendStateChanged(Shaman::Base::instance()->core()->backend()->state());
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    //TODO: I know that cast sucks, probably find a better sollution
    foreach (const QModelIndex &index, selectionModel()->selectedRows())
    {
        packagesModel->packageFromModelIndex(proxyModel->mapToSource(index))->setAction(Shaman::AbstractPackage::Install);
    }
    m_addingToQueue = false;
    backendStateChanged(Shaman::Base::instance()->core()->backend()->state());
}

void PackagesView::updateCurrentPackages()
{
    m_addingToQueue = true;
    backendStateChanged(Shaman::Base::instance()->core()->backend()->state());
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    //TODO: I know that cast sucks, probably find a better sollution
    foreach (const QModelIndex &index, selectionModel()->selectedRows())
    {
        packagesModel->packageFromModelIndex(proxyModel->mapToSource(index))->setAction(Shaman::AbstractPackage::Upgrade);
    }
    m_addingToQueue = false;
    backendStateChanged(Shaman::Base::instance()->core()->backend()->state());
}

void PackagesView::uninstallCurrentPackages()
{
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    //TODO: I know that cast sucks, probably find a better sollution
    foreach (const QModelIndex &index, selectionModel()->selectedRows())
    {
        packagesModel->packageFromModelIndex(proxyModel->mapToSource(index))->setAction(Shaman::AbstractPackage::Uninstall);
    }
}

void PackagesView::cancelCurrentPackages()
{
    PackagesSortFilterProxyModel * proxyModel = qobject_cast<PackagesSortFilterProxyModel*>(model());
    PackagesModel * packagesModel = qobject_cast<PackagesModel*>(proxyModel->sourceModel());
    //TODO: I know that cast sucks, probably find a better sollution
    foreach (const QModelIndex &index, selectionModel()->selectedRows())
    {
        packagesModel->packageFromModelIndex(proxyModel->mapToSource(index))->setAction(Shaman::AbstractPackage::NoAction);
    }
}

QList<KAction*> PackagesView::actions()
{
    QList<KAction*> list;
    list << m_cancelAction;
    if (!m_installAction.isNull()) {
        list << m_installAction.data();
    }
    if (!m_uninstallAction.isNull()) {
        list << m_uninstallAction.data();
    }
    if (!m_updateAction.isNull()) {
        list << m_updateAction.data();
    }
    return list;
}

void PackagesView::paintEvent(QPaintEvent* event)
{
    if (isEnabled() || m_addingToQueue) {
        QTreeView::paintEvent(event);
    }

    if (!isEnabled() || m_retainPaintEvent) {
        QPainter painter(viewport());

        QRect spinnerRect(QPoint(0, 0), QSize((int)KIconLoader::SizeSmallMedium, (int)KIconLoader::SizeSmallMedium));
        spinnerRect.moveCenter(QPoint(size().width()/2, size().height()/2  + 32));

        QRect textRect(QPoint(0, 0), QSize(size().width(),64));
        textRect.moveCenter(QPoint(size().width()/2, size().height()/2));

        QRect iconRect(QPoint(0, 0), QSize((int)KIconLoader::SizeEnormous, (int)KIconLoader::SizeEnormous));
        iconRect.moveCenter(QPoint(size().width()/2, size().height()/2));

        painter.setOpacity(m_fadeTimeLine->currentFrame() * 0.0035);
        painter.drawPixmap(iconRect, m_backgroundPixmap);
        painter.setOpacity(m_fadeTimeLine->currentFrame() * 0.01);
#if KDE_IS_VERSION(4,3,67)
        painter.drawPixmap(spinnerRect, m_sequence.frameAt(m_spinnerTimeLine->currentFrame()));
#else
        painter.drawPixmap(spinnerRect, m_spinner.at(m_spinnerTimeLine->currentFrame()));
#endif

        painter.setPen(Qt::SolidLine);
        Qt::Alignment align(Qt::AlignVCenter | Qt::AlignHCenter);
        painter.drawText(textRect, m_message, QTextOption(align));
    }
}

void PackagesView::setEnabled(bool enabled)
{
    if (enabled == isEnabled()) {
        return;
    }

    QTreeView::setEnabled(enabled);

    // Ok: now
    if (enabled) {
        m_fadeTimeLine->setDirection(QTimeLine::Backward);
        m_fadeTimeLine->start();
        m_spinnerTimeLine->stop();
        m_retainPaintEvent = true;
        viewport()->update();
    } else {
        m_fadeTimeLine->setDirection(QTimeLine::Forward);
        m_fadeTimeLine->start();
        m_spinnerTimeLine->start();
        viewport()->update();
    }
}

void PackagesView::stopRetainingPaintEvent()
{
    m_retainPaintEvent = false;
    viewport()->update();
}

void PackagesView::backendStateChanged(Shaman::Backend::State state)
{
    if (m_cachedState == Shaman::Backend::WaitingForLock && state != m_cachedState) {
        // Restore the original pixmap
        m_backgroundPixmap = KIconLoader::global()->loadIcon("document-preview-archive", KIconLoader::Desktop,
                                                             KIconLoader::SizeEnormous);
    }

    if (m_addingToQueue) {
        // Special case
        m_message = i18n("Adding targets to queue...");
        setEnabled(false);
        return;
    }

    switch (state) {
        case Shaman::Backend::Reloading:
            if (m_cachedState != Shaman::Backend::Reloading) {
                m_message = i18n("Loading...");
                setEnabled(false);
            }
            break;
        case Shaman::Backend::Searching:
            if (m_cachedState != Shaman::Backend::Searching) {
                m_message = i18n("Searching packages...");
                setEnabled(false);
            }
            break;
        case Shaman::Backend::WaitingForLock:
            if (m_cachedState != Shaman::Backend::WaitingForLock) {
                m_backgroundPixmap = KIconLoader::global()->loadIcon("object-locked-finished", KIconLoader::Desktop,
                                                                     KIconLoader::SizeEnormous);
                m_message = i18n("Waiting for an exclusive lock...");
                setEnabled(false);
            }
            break;
        default:
            if (m_cachedState == Shaman::Backend::Reloading || m_cachedState == Shaman::Backend::Searching ||
                m_cachedState == Shaman::Backend::WaitingForLock) {
                m_message = i18n("Operation completed");
            }
            setEnabled(true);
            break;
    }
    m_cachedState = state;
}

#if KDE_IS_VERSION(4,3,67)
#else
void PackagesView::loadSpinnerFrames()
{
    QPixmap bigPixmap = QPixmap(KIconLoader::global()->iconPath("process-working", -(int)KIconLoader::SizeSmallMedium));
    QSize size((int)KIconLoader::SizeSmallMedium, (int)KIconLoader::SizeSmallMedium);

    m_spinner.resize((bigPixmap.height()/size.height()) * (bigPixmap.width()/size.width()));

    int pos = 0;
    for (int row = 0; row < bigPixmap.height()/size.height(); ++row) {
        for (int col = 0; col < bigPixmap.width()/size.width(); ++col) {
            QPixmap pix = QPixmap(size);
            pix.fill(Qt::transparent);
            QPainter painter(&pix);
            painter.drawPixmap(QPoint(0, 0), bigPixmap, QRect(col*size.width(), row*size.height(), size.width(), size.height()));
            painter.end();
            m_spinner[pos++] = pix;
        }
    }
}
#endif

#include "PackagesView.moc" 
