/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>
   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "PackagesViewDockView.h"
#include <QTimeLine>
#include <QPainter>
#include <QSortFilterProxyModel>

#include <klocalizedstring.h>
#include <KIconLoader>

PackagesViewDockView::PackagesViewDockView(QWidget* parent)
    : QListView(parent)
    , m_retainPaintEvent(false)
    , m_fadeTimeLine(new QTimeLine(1200, this))
    , m_spinnerTimeLine(new QTimeLine(2000, this))
#if KDE_IS_VERSION(4,3,67)
    , m_sequence("process-working", KIconLoader::SizeSmallMedium)
#endif
    , m_backgroundPixmap(KIconLoader::global()->loadIcon("document-preview-archive", KIconLoader::Desktop,
                       KIconLoader::SizeEnormous))
{
    // Setup animations
#if KDE_IS_VERSION(4,3,67)
    m_spinnerTimeLine->setFrameRange(0, m_sequence.frameCount());
#else
    loadSpinnerFrames();
    m_spinnerTimeLine->setFrameRange(0, m_spinner.size());
#endif
    m_spinnerTimeLine->setLoopCount(0);
    m_spinnerTimeLine->setCurveShape(QTimeLine::LinearCurve);
    connect(m_spinnerTimeLine, SIGNAL(frameChanged(int)), viewport(), SLOT(repaint()));

    m_fadeTimeLine->setFrameRange(0, 100);
    m_fadeTimeLine->setCurveShape(QTimeLine::EaseOutCurve);
    connect(m_fadeTimeLine, SIGNAL(frameChanged(int)), viewport(), SLOT(repaint()));
    connect(m_fadeTimeLine, SIGNAL(finished()), this, SLOT(stopRetainingPaintEvent()));
    m_message = i18n("Loading...");
}

PackagesViewDockView::~PackagesViewDockView()
{

}

void PackagesViewDockView::paintEvent(QPaintEvent* e)
{
    if (isEnabled()) {
        QListView::paintEvent(e);
    }

    if (!isEnabled() || m_retainPaintEvent) {
        QPainter painter(viewport());

        QRect spinnerRect(QPoint(0, 0), QSize((int)KIconLoader::SizeSmallMedium, (int)KIconLoader::SizeSmallMedium));
        spinnerRect.moveCenter(QPoint(size().width()/2, size().height()/2  + 32));

        QRect textRect(QPoint(0, 0), QSize(size().width(),64));
        textRect.moveCenter(QPoint(size().width()/2, size().height()/2));

        QRect iconRect(QPoint(0, 0), QSize((int)KIconLoader::SizeEnormous, (int)KIconLoader::SizeEnormous));
        iconRect.moveCenter(QPoint(size().width()/2, size().height()/2));

        painter.setOpacity(m_fadeTimeLine->currentFrame() * 0.0035);
        painter.drawPixmap(iconRect, m_backgroundPixmap);
        painter.setOpacity(m_fadeTimeLine->currentFrame() * 0.01);
#if KDE_IS_VERSION(4,3,67)
        painter.drawPixmap(spinnerRect, m_sequence.frameAt(m_spinnerTimeLine->currentFrame()));
#else
        painter.drawPixmap(spinnerRect, m_spinner.at(m_spinnerTimeLine->currentFrame()));
#endif

        painter.setPen(Qt::SolidLine);
        Qt::Alignment align(Qt::AlignVCenter | Qt::AlignHCenter);
        painter.drawText(textRect, m_message, QTextOption(align));
    }
}

void PackagesViewDockView::slotRowsInserted()
{
    if (m_viewType == PackageGroupsView || m_viewType == RepositoriesView) {
        QSortFilterProxyModel *smodel = qobject_cast<QSortFilterProxyModel *>(model());
        if (smodel->sourceModel()->rowCount() > 1) {
            setEnabled(true);
        } else {
            setEnabled(false);
        }
    }
}

void PackagesViewDockView::stopRetainingPaintEvent()
{
    m_retainPaintEvent = false;
    viewport()->update();
}

void PackagesViewDockView::setViewType(PackagesViewDockView::ViewType type)
{
    m_viewType = type;
    slotRowsInserted();
}

void PackagesViewDockView::setEnabled(bool enabled)
{
    if (enabled == isEnabled()) {
        return;
    }

    QListView::setEnabled(enabled);

    // Ok: now
    if (enabled) {
        m_fadeTimeLine->setDirection(QTimeLine::Backward);
        m_fadeTimeLine->start();
        m_spinnerTimeLine->stop();
        m_retainPaintEvent = true;
        viewport()->update();
    } else {
        m_fadeTimeLine->setDirection(QTimeLine::Forward);
        m_fadeTimeLine->start();
        m_spinnerTimeLine->start();
        viewport()->update();
    }
}

#if KDE_IS_VERSION(4,3,67)
#else
void PackagesViewDockView::loadSpinnerFrames()
{
    QPixmap bigPixmap = QPixmap(KIconLoader::global()->iconPath("process-working", -(int)KIconLoader::SizeSmallMedium));
    QSize size((int)KIconLoader::SizeSmallMedium, (int)KIconLoader::SizeSmallMedium);

    m_spinner.resize((bigPixmap.height()/size.height()) * (bigPixmap.width()/size.width()));

    int pos = 0;
    for (int row = 0; row < bigPixmap.height()/size.height(); ++row) {
        for (int col = 0; col < bigPixmap.width()/size.width(); ++col) {
            QPixmap pix = QPixmap(size);
            pix.fill(Qt::transparent);
            QPainter painter(&pix);
            painter.drawPixmap(QPoint(0, 0), bigPixmap, QRect(col*size.width(), row*size.height(), size.width(), size.height()));
            painter.end();
            m_spinner[pos++] = pix;
        }
    }
}
#endif

#include "PackagesViewDockView.moc"
