/* This file is part of the KDE project

   Copyright (C) 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "PackagesViewDockWidget.h"

#include "RepositoryModel.h"
#include "PackageGroupModel.h"
#include "FavouriteSearchModel.h"
#include "PackagesViewPlugin.h"

#include <shamanbackend.h>

#include <QSortFilterProxyModel>

#include <KIcon>
#include <KDebug>

PackagesViewDockWidget::PackagesViewDockWidget(Shaman::Backend * backend, PackagesViewPlugin *plugin, QWidget * parent)
  : QDockWidget(parent),
    m_backend(backend),
    m_repoModel(0),
    m_groupModel(0),
    m_plugin(plugin)
{
    setupUi(this);
    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByRepositoryCapability) {
        repositoriesButton->setIcon(KIcon("network-server-database"));
    } else {
        repositoriesButton->setVisible(false);
        repositoriesButton->setChecked(false);
        repositoriesButton->setEnabled(false);
    }
    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByPackageGroupsCapability) {
        packageGroupsButton->setIcon(KIcon("bookmark-new-list"));
    } else {
        packageGroupsButton->setVisible(false);
        packageGroupsButton->setEnabled(false);
        packageGroupsButton->setChecked(false);
    }
    favouritesButton->setIcon(KIcon("rating"));

    m_proxyModel = new QSortFilterProxyModel(this);
    m_proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    listView->setModel(m_proxyModel);

    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByRepositoryCapability) {
        m_repoModel = new RepositoryModel(m_backend->repositories(), m_backend->packages(), this);
        connect(m_repoModel, SIGNAL(rowsInserted(QModelIndex,int,int)), listView, SLOT(slotRowsInserted()));
        connect(m_repoModel, SIGNAL(rowsRemoved(QModelIndex,int,int)), listView, SLOT(slotRowsInserted()));
    }

    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByPackageGroupsCapability) {
        m_groupModel = new PackageGroupModel(m_backend->packageGroups(), m_backend->packages(), this);
        connect(m_groupModel, SIGNAL(rowsInserted(QModelIndex,int,int)), listView, SLOT(slotRowsInserted()));
        connect(m_groupModel, SIGNAL(rowsRemoved(QModelIndex,int,int)), listView, SLOT(slotRowsInserted()));
    }

    m_favouriteSearchModel = new FavouriteSearchModel(m_plugin->config().group("FavouriteSearches"), this);

    if (m_repoModel) {
        m_proxyModel->setSourceModel(m_repoModel);
        repositoriesButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::RepositoriesView);
    } else if (m_groupModel) {
        m_proxyModel->setSourceModel(m_groupModel);
        packageGroupsButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::PackageGroupsView);
    } else {
        m_proxyModel->setSourceModel(m_favouriteSearchModel);
        favouritesButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::FavoriteSearchesView);
    }

    connect(repositoriesButton, SIGNAL(clicked()), SLOT(changeModel()));
    connect(packageGroupsButton, SIGNAL(clicked()), SLOT(changeModel()));
    connect(favouritesButton, SIGNAL(clicked()), SLOT(changeModel()));
    connect(searchLine, SIGNAL(textChanged(const QString &)), m_proxyModel, SLOT(setFilterFixedString(const QString &)));
    connect(listView->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)),
                                        SLOT(currentRowChanged(const QModelIndex &, const QModelIndex &)));
    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByRepositoryCapability) {
        connect(backend, SIGNAL(repositoriesChanged(Shaman::Repository::List)), m_repoModel, SLOT(setRepositories(Shaman::Repository::List)));
        connect(backend, SIGNAL(repositoriesChanged(Shaman::Repository::List)), listView, SLOT(slotRowsInserted()));
        connect(backend, SIGNAL(packagesChanged(Shaman::AbstractPackage::List)),
                m_repoModel, SLOT(setDefaultRepository(Shaman::AbstractPackage::List)));
    }

    if (m_backend->capabilities()->packageGroupingCapabilities() & BackendCapabilities::GroupPackagesByPackageGroupsCapability) {
        connect(backend, SIGNAL(packageGroupsChanged(Shaman::PackageGroup::List)), 
                m_groupModel, SLOT(setPackageGroups(Shaman::PackageGroup::List)));
        connect(backend, SIGNAL(packageGroupsChanged(Shaman::PackageGroup::List)), listView, SLOT(slotRowsInserted()));
        connect(backend, SIGNAL(packagesChanged(Shaman::AbstractPackage::List)),
                m_groupModel, SLOT(setDefaultPackageGroup(Shaman::AbstractPackage::List)));
    }
}

PackagesViewDockWidget::~PackagesViewDockWidget()
{
}

void PackagesViewDockWidget::changeModel()
{
    QPushButton * sender = qobject_cast<QPushButton*>(QObject::sender());
    if (sender == repositoriesButton) {
        m_proxyModel->setSourceModel(m_repoModel);
        setWindowTitle(i18n("Repositories"));
        packageGroupsButton->setChecked(false);
        favouritesButton->setChecked(false);
        repositoriesButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::RepositoriesView);
    }
    else if (sender == packageGroupsButton) {
        m_proxyModel->setSourceModel(m_groupModel);
        setWindowTitle(i18n("Package Groups"));
        repositoriesButton->setChecked(false);
        favouritesButton->setChecked(false);
        packageGroupsButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::PackageGroupsView);
    }
    else if (sender == favouritesButton)
    {
        m_proxyModel->setSourceModel(m_favouriteSearchModel);
        setWindowTitle(i18n("Favorite Searches"));
        repositoriesButton->setChecked(false);
        packageGroupsButton->setChecked(false);
        favouritesButton->setChecked(true);
        listView->setViewType(PackagesViewDockView::FavoriteSearchesView);
    }
    connect(listView->selectionModel(), SIGNAL(currentRowChanged(const QModelIndex &, const QModelIndex &)),
                                        SLOT(currentRowChanged(const QModelIndex &, const QModelIndex &)));
}

void PackagesViewDockWidget::currentRowChanged(const QModelIndex &current, const QModelIndex &previous)
{
    Q_UNUSED(previous);
    if (repositoriesButton->isChecked()) {
        emit packageGroupFilterChanged(0);//TODO: Optimize to not do the job twice
        emit repositoryFilterChanged(m_repoModel->repositoryFromModelIndex(current));
    }
    else if (packageGroupsButton->isChecked()) {
        emit repositoryFilterChanged(0);
        emit packageGroupFilterChanged(m_groupModel->packageGroupFromModelIndex(current));
    }
    else if (favouritesButton->isChecked()) {
        FavouriteSearchModel::Search search = m_favouriteSearchModel->searchFromModelIndex(current);
        emit changeSearch(search.search, search.type);
    }
}

void PackagesViewDockWidget::addFavouriteSearch(const QString &search, SearchModel::Type type)
{
    m_favouriteSearchModel->addFavouriteSearch(search, type);
}
