/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#ifndef SHAMANSTATUSBARINTERFACE_H
#define SHAMANSTATUSBARINTERFACE_H

#include "shamanmacros.h"

#include <QObject>
#include <QPixmap>
#include <QString>

class QStatusBar;

namespace Shaman {
/** \class StatusBarInterface shamanstatusbarinterface.h
 * \brief Shaman's StatusBarInterface-class.
 *
 * This class is the connection from the GuiPlugin to the program's StatusBar.
 * It has a Standard-Message which gets displayed in idling state and the showMessage()-function,
 * which is used to display a Message for a specific time (mostly a few seconds) only. 
 * The StatusBar also has a Progress-Bar which can be used with setProgress().
 * To set the default values, simply call reset().
 *
 * \code
 * statusBar()->setStandardMessage(i18n("Hello World!"), KIcon("list-add").pixmap(22, 22));
 * statusBar()->setProgress(40);
 * \endcode
 *
 * This sets the Standard-Message of the StatusBar to "Hello World!" with the Icon "list-add" as pixmap and sets 
 * the progress to 40.
 */
class SHAMAN_EXPORT StatusBarInterface : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(StatusBarInterface)
    public:
        StatusBarInterface(QStatusBar * statusBar, QObject * parent);
        ~StatusBarInterface();

    public Q_SLOTS:
        /**
         * Shows a Message on the StatusBar
         * @param text The text to show
         * @param pixmap The pixmap to show
         * @param timeout the seconds before the standard message will get displayed again. A negative value
         *                will hold the message indefinitely
         */
        void showMessage(const QString &text, const QPixmap &pixmap, int timeout = 5);

        /**
         * Sets the Standard-Message of the StatusBar
         * @param text The text to show
         * @param pixmap The pixmap to show
         */
        void setStandardMessage(const QString &text, const QPixmap &pixmap);

        /**
         * Sets the progress of the StatusBar's ProgressBar
         * @param percent the new Progress
         */
        void setProgress(int percent);

        /**
         * Resets the StatusBar
         */
        void reset();

    private:
        class Private;
        Private *d;
};
}

#endif 
