/* This file is part of the KDE project

   Copyright (C) 2008 Dario Freddi <drf@kde.org>
                 2008 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "MainWindow.h"

#include "settings.h"

#include <shamanbase.h>

#include <KActionCollection>
#include <KDebug>
#include <KStatusBar>
#include <KToggleAction>
#include <KMenuBar>
#include <KNotifyConfigWidget>
#include <KShortcutsDialog>
#include <KEditToolBar>
#include <KAboutApplicationDialog>

#include <QAction>
#include <QDockWidget>

#include <shamancoreinterface.h>
#include "processqueue/ProcessQueueHandler.h"
#include "updatedatabase/UpdateDatabaseHandler.h"
#include <QLabel>
#include <KAboutData>
#include "QueueSerializer.h"
#include <KApplication>
#include <KNotification>
#include <KCMultiDialog>
#include "shamanguiinterface.h"

MainWindow::MainWindow(KAboutData *data)
  : KXmlGuiWindow(),
    m_shaman(0),
    m_menubarAction(0),
    m_trickWindow(new QMainWindow(0)),
    m_isReloading(false),
    m_aboutData(data),
    m_dockWidgetIndex(0),
    m_isFirstReload(true)
{
    setCentralWidget(m_trickWindow);
    m_trickWindow->setWindowFlags(Qt::Widget);

    // local - Destroys all sub-windows and exits
    KAction *quitAction = KStandardAction::quit(this, SLOT(slotQuit()), actionCollection());
    actionCollection()->addAction("quit", quitAction);
    // local - Standard configure actions
    KAction *preferencesAction = KStandardAction::preferences(this, SLOT(slotPreferences()), actionCollection());
    actionCollection()->addAction("preferences", preferencesAction);
    KAction *configToolbarAction = KStandardAction::configureToolbars(this, SLOT(slotConfigureToolbars()), actionCollection());
    actionCollection()->addAction("configure_toolbars", configToolbarAction);
    KAction *keyBindingsAction = KStandardAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection());
    actionCollection()->addAction("configure_keys", keyBindingsAction);
    KAction *cinfigNotifyAction = KStandardAction::configureNotifications(this, SLOT(slotConfigureNotifications()), actionCollection());
    actionCollection()->addAction("configure_notifications", cinfigNotifyAction);

    m_menubarAction = KStandardAction::showMenubar(this, SLOT(slotShowMenubar()), actionCollection());
    m_menubarAction->setChecked(!menuBar()->isHidden());
    actionCollection()->addAction("settings_showmenubar", m_menubarAction);

    m_shaman = Shaman::Base::instance();
    m_shaman->setConfigurationFile(KGlobal::config());
    m_shamanGui = GuiInterface::instance();
    m_shamanGui->setStatusBar(statusBar());

    connect(m_shamanGui, SIGNAL(createMainWidget(QWidget *)), SLOT(slotCreateMainWidget(QWidget *)));
    connect(m_shamanGui, SIGNAL(createDockWidget(Qt::DockWidgetArea, QDockWidget *)),
            SLOT(slotCreateDockWidget(Qt::DockWidgetArea, QDockWidget *)));
    connect(m_shamanGui, SIGNAL(createAction(QAction *)), SLOT(slotCreateAction(QAction *)));
    connect(m_shaman, SIGNAL(initCompleted()), SLOT(setUpGui()));

    connect(KApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(aboutToQuit()));

    m_shaman->init(Shaman::Base::CachedMode);
}

MainWindow::~MainWindow()
{
    slotSaveMyself();
}

void MainWindow::aboutToQuit()
{
    // eventually the queue...
    if (Settings::saveQueueOnExit()) {
        kDebug() << "Saving last queue";
        KConfigGroup queue(KGlobal::config(), "LastQueue");
        QueueSerializer::saveQueueToConfigGroup(&queue);
        KGlobal::config().data()->sync();
    }
}

void MainWindow::setUpGui()
{
    // Ok, first of all update the about data
    Backend *backend = m_shaman->core()->backend();
    m_aboutData->setOtherText(ki18n("Using %1, %2.").subs(backend->properties()->name())
                                                    .subs(backend->properties()->description()));
    foreach (const KAboutPerson &person, backend->properties()->authors()) {
        m_aboutData->addAuthor(ki18n("%1").subs(person.name()), ki18n("%1 backend - %2").subs(backend->properties()->name())
                                                                                        .subs(person.task()),
                               person.emailAddress().toUtf8(), person.webAddress().toUtf8());
    }

    setComponentData(KComponentData(m_aboutData));

    kDebug();

    if (backend->capabilities()->transactionCapabilities() & Shaman::BackendCapabilities::InstallPackagesCapability ||
        backend->capabilities()->transactionCapabilities() & Shaman::BackendCapabilities::UpgradePackagesCapability ||
        backend->capabilities()->transactionCapabilities() & Shaman::BackendCapabilities::RemovePackagesCapability) {
        ProcessQueueHandler *queueHandler = new ProcessQueueHandler(this);
        KAction *processQueueAction = new KAction(KIcon("dialog-ok"), i18n("Process Queue"), this);
        processQueueAction->setShortcut(Qt::CTRL + Qt::Key_P);
        actionCollection()->addAction("process_queue", processQueueAction);
        connect(processQueueAction, SIGNAL(triggered()), queueHandler, SLOT(processQueue()));
        m_queueActions.append(processQueueAction);
    }

    if (backend->capabilities()->transactionCapabilities() & Shaman::BackendCapabilities::UpdateRepositoriesCapability) {
        UpdateDatabaseHandler *updateRepositoriesHandler = new UpdateDatabaseHandler(this);
        KAction *updateRepositoriesAction = new KAction(KIcon("view-refresh"), i18n("Update Database"), this);
        updateRepositoriesAction->setShortcut(Qt::CTRL + Qt::Key_U);
        actionCollection()->addAction("update_database", updateRepositoriesAction);
        connect(updateRepositoriesAction, SIGNAL(triggered()), updateRepositoriesHandler, SLOT(updateRepositories()));
        m_queueActions.append(updateRepositoriesAction);
    }

    if (backend->capabilities()->transactionCapabilities() & Shaman::BackendCapabilities::UpgradePackagesCapability) {
        KAction *processQueueAction = new KAction(KIcon("system-software-update"), i18n("Update system"), this);
        processQueueAction->setShortcut(Qt::CTRL + Qt::Key_S);
        actionCollection()->addAction("update_system", processQueueAction);
        connect(processQueueAction, SIGNAL(triggered()), this, SLOT(slotUpdateSystem()));
        m_queueActions.append(processQueueAction);
        // Check whether to enable it or not
        processQueueAction->setEnabled(m_shaman->core()->backend()->upgradeablePackages().count() > 0);
        connect(m_shaman->core()->backend(), SIGNAL(upgradeablePackagesChanged(Shaman::AbstractPackage::List)),
                this, SLOT(slotUpgradeablePackagesChanged(Shaman::AbstractPackage::List)));
    }

    createGUI();

    // restore position, size and visibility
    move(Settings::mainPosition());
    setAutoSaveSettings();
    show();

    // Ok, now attach to backend signals
    connect(m_shaman->core()->backend(), SIGNAL(stateChanged(Shaman::Backend::State)),
            this, SLOT(backendStateChanged(Shaman::Backend::State)));
    backendStateChanged(m_shaman->core()->backend()->state());
    connect(m_shaman->core()->transactionInterface(), SIGNAL(newTransactionQuestion(Shaman::TransactionQuestion*)),
            this, SLOT(newTransactionQuestion(Shaman::TransactionQuestion*)));
}

void MainWindow::slotCreateMainWidget(QWidget * widget)
{
    if (centralWidget()) {
        kDebug() << "There's already a central widget :(";
    }
    m_trickWindow->setCentralWidget(widget);
}

void MainWindow::slotCreateDockWidget(Qt::DockWidgetArea area, QDockWidget * dockwidget)
{
    ++m_dockWidgetIndex;
    if (area == Qt::TopDockWidgetArea || area == Qt::BottomDockWidgetArea) {
        m_trickWindow->addDockWidget(area, dockwidget);
    } else {
        addDockWidget(area, dockwidget);
    }
    connect(dockwidget, SIGNAL(allowedAreasChanged(Qt::DockWidgetAreas)), SLOT(slotAllowedAreasChanged(Qt::DockWidgetAreas)));

    QAction *dockAction = dockwidget->toggleViewAction();
    actionCollection()->addAction(QString("show_dockwidget%1").arg(m_dockWidgetIndex), dockAction);
}

void MainWindow::slotCreateAction(QAction * action)
{
    kDebug() << "Add action " + action->objectName();
    actionCollection()->addAction(action->objectName(), action);//TODO: Probably set a custom name
}

void MainWindow::slotQuit()
{
    //Settings::self()->writeConfig();
    KApplication::instance()->quit();
}

void MainWindow::slotPreferences()
{
    m_configModules.clear();
    m_saved = 0;
    KPageWidgetItem *item;
    KCMultiDialog *dialog = new KCMultiDialog(this);
    item = dialog->addModule("kcm_shaman_general");
    if (item) {
        m_configModules.insert(item->name(), GeneralSettingsType);
    }

    foreach (const QString &module, m_shaman->core()->backend()->properties()->configurationModules()) {
        item = dialog->addModule(module);
        if (item) {
            m_configModules.insert(item->name(), BackendSettingsType);
        }
    }
    connect(dialog, SIGNAL(configCommitted()), this, SLOT(slotConfigCommitted()));
    connect(dialog, SIGNAL(finished()), this, SLOT(slotConfigDialogFinished()));
    dialog->show();
}

void MainWindow::slotConfigCommitted()
{
    KCMultiDialog *dialog = qobject_cast<KCMultiDialog*>(sender());
    kDebug() << "Config committed for" << dialog->currentPage()->name();
    if (m_configModules.contains(dialog->currentPage()->name())) {
        m_saved |= m_configModules[dialog->currentPage()->name()];
    }
}

void MainWindow::slotConfigDialogFinished()
{
    // Act accordingly
    kDebug() << "Config dialog has finished running";
    if (m_saved & GeneralSettingsType) {
        kDebug() << "General settings have been modified";
        // Nothing to do yet (save on exit automagically reloads)
    }
    if (m_saved & BackendSettingsType) {
        kDebug() << "Backend settings have been modified";
    }
}

void MainWindow::slotConfigureNotifications()
{
    KNotifyConfigWidget::configure(this);
}

void MainWindow::slotConfigureKeys()
{
    KShortcutsDialog::configure(actionCollection());
}

void MainWindow::slotConfigureToolbars()
{
    KEditToolBar edit(actionCollection());
    connect(&edit, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
    edit.exec();
}

void MainWindow::slotShowMenubar()
{
    if (m_menubarAction->isChecked()) {
        menuBar()->show();
    } else {
        menuBar()->hide();
    }
}

void MainWindow::slotSaveMyself()
{
    kDebug() << "Saving before closing";
    // save last parameters ..
    Settings::setMainPosition(pos());
    // .. and write config to disk
    Settings::self()->writeConfig();
}

void MainWindow::slotNewToolbarConfig()
{
    createGUI();
}

void MainWindow::backendStateChanged(Shaman::Backend::State state)
{
    if (state == Shaman::Backend::Reloading) {
        // Trigger reloading
        m_isReloading = true;
        m_reenableActions.clear();
        foreach (QAction *action, m_queueActions) {
            if (action->isEnabled()) {
                action->setEnabled(false);
                m_reenableActions.append(action);
            }
        }
    } else if (m_isReloading) {
        foreach (QAction *action, m_reenableActions) {
            action->setEnabled(true);
        }
        if (m_isFirstReload) {
            // Eventually load the last queue
            if (Settings::saveQueueOnExit()) {
                KConfigGroup queue(KGlobal::config(), "LastQueue");
                QueueSerializer::loadQueueFromConfigGroup(queue);
                // Now erase the old queue
                queue.deleteGroup();
                queue.config()->sync();
            }
            m_isFirstReload = false;
        }
        m_isReloading = false;
    }
}

void MainWindow::slotAllowedAreasChanged(Qt::DockWidgetAreas area)
{
    QDockWidget *w = qobject_cast<QDockWidget *>(sender());
    Q_ASSERT(w);
    removeDockWidget(w);
    if (area & Qt::LeftDockWidgetArea) {
        addDockWidget(Qt::LeftDockWidgetArea, w);
    } else if (area & Qt::RightDockWidgetArea) {
        addDockWidget(Qt::RightDockWidgetArea, w);
    } else if (area & Qt::TopDockWidgetArea) {
        addDockWidget(Qt::TopDockWidgetArea, w);
    } else if (area & Qt::BottomDockWidgetArea) {
        addDockWidget(Qt::BottomDockWidgetArea, w);
    }
}

void MainWindow::newTransactionQuestion(Shaman::TransactionQuestion *question)
{
    Shaman::StringTransactionQuestion *strq = dynamic_cast<Shaman::StringTransactionQuestion*>(question);

    if (!strq) {
        return;
    }
    if (strq->modal()) {
        return;
    }

    KNotification *notification = KNotification::event("transactionquestion", strq->text(), strq->icon().pixmap(32),
                                                       0, KNotification::Persistent, componentData());
    QStringList actions;

    if (question->possibleAnswers() & TransactionQuestion::AnswerYes) {
        actions << i18n("Yes");
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerNo) {
        actions << i18n("No");
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerCancel) {
        actions << i18n("Cancel");
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser1) {
        actions << question->user1Text();
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser2) {
        actions << question->user2Text();
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser3) {
        actions << question->user3Text();
    }
    notification->setActions(actions);
    notification->setProperty("__Shaman_Question", QVariant::fromValue(question));

    connect(notification, SIGNAL(activated(uint)), this, SLOT(slotActionActivated(uint)));
}

void MainWindow::slotActionActivated(uint action)
{
    TransactionQuestion *question = sender()->property("__Shaman_Question").value<Shaman::TransactionQuestion*>();
    TransactionQuestion::Answer answer;
    uint raction = 0;

    if (question->possibleAnswers() & TransactionQuestion::AnswerYes) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerYes;
        }
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerNo) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerNo;
        }
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerCancel) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerCancel;
        }
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser1) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerUser1;
        }
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser2) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerUser2;
        }
    }
    if (question->possibleAnswers() & TransactionQuestion::AnswerUser3) {
        ++raction;
        if (action == raction) {
            answer = TransactionQuestion::AnswerUser3;
        }
    }

    question->setAnswer(answer);
}

void MainWindow::slotUpdateSystem()
{
    foreach (Shaman::AbstractPackage *package, m_shaman->core()->backend()->upgradeablePackages()) {
        package->setAction(Shaman::AbstractPackage::Upgrade);
    }

    actionCollection()->action("process_queue")->trigger();
}

void MainWindow::slotUpgradeablePackagesChanged(Shaman::AbstractPackage::List packages)
{
    actionCollection()->action("update_system")->setEnabled(packages.count() > 0);
}

#include "MainWindow.moc"
