/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "ErrorQuestionHandler.h"
#include <KMessageBox>
#include <klocalizedstring.h>
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <shamanwidgetfactory.h>

ErrorQuestionHandler::ErrorQuestionHandler(QWidget* parent)
    : KDialog(parent)
{
    connect(Shaman::Base::instance()->core()->transactionInterface(),
            SIGNAL(errorTriggered(Shaman::TransactionInterface::Error,QString)),
            this,
            SLOT(errorTriggered(Shaman::TransactionInterface::Error,QString)));

    connect(Shaman::Base::instance()->core()->transactionInterface(),
            SIGNAL(newTransactionQuestion(Shaman::TransactionQuestion*)),
            this,
            SLOT(newTransactionQuestion(Shaman::TransactionQuestion*)));
}

ErrorQuestionHandler::~ErrorQuestionHandler()
{

}

void ErrorQuestionHandler::errorTriggered(Shaman::TransactionInterface::Error code, const QString &details)
{
    switch (code) {
        case Shaman::TransactionInterface::DownloadFailedError:
            KMessageBox::detailedError(this, i18n("There has been an error while downloading packages."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::ProcessingFailedError:
            KMessageBox::detailedError(this, i18n("There has been an error while processing the queue."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::CleanupFailedError:
            KMessageBox::detailedError(this, i18n("There has been an error while cleaning up."),
                                       details, i18n("Cleanup failed"));
            break;
        case Shaman::TransactionInterface::FileConflictError:
            KMessageBox::detailedError(this, i18n("A file conflict was detected in the current queue."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::DependencyConflictError:
            KMessageBox::detailedError(this, i18n("A dependency conflict was detected in the current queue."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::DependenciesUnsatisfiedError:
            KMessageBox::detailedError(this, i18n("A dependency could not to be found."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::BackendError:
            KMessageBox::detailedError(this, i18n("There has been an error in the backend."),
                                       details, i18n("Queue processing failed"));
            break;
        case Shaman::TransactionInterface::HookError:
            KMessageBox::detailedError(this, i18n("A hook requested to abort the transaction."),
                                       details, i18n("Queue processing failed"));
            break;
        default:
            KMessageBox::detailedError(this, i18n("There has been an error while performing the transaction."),
                                       details, i18n("Queue processing failed"));
            break;
    }
}

void ErrorQuestionHandler::newTransactionQuestion(Shaman::TransactionQuestion* question)
{
    // Handle only modal questions here
    Shaman::StringTransactionQuestion *strq = dynamic_cast<Shaman::StringTransactionQuestion*>(question);

    if (strq) {
        if (!strq->modal()) {
            return;
        }
    }

    KDialog * dialog = new KDialog(this);
    dialog->setMainWidget(question->questionWidgetFactory()->createInstance());

    KDialog::ButtonCodes codes = 0;
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerYes) {
        codes |= Yes;
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerNo) {
        codes |= No;
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerCancel) {
        codes |= Cancel;
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser1) {
        codes |= User1;
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser2) {
        codes |= User2;
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser3) {
        codes |= User3;
    }
    dialog->setButtons(codes);

    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser1) {
        KGuiItem item;
        item.setIcon(KIcon(question->user1Icon()));
        item.setText(question->user1Text());
        dialog->setButtonGuiItem(KDialog::User1, item);
        //dialog->enableButton(KDialog::User1, true);
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser2) {
        KGuiItem item;
        item.setIcon(KIcon(question->user2Icon()));
        item.setText(question->user2Text());
        dialog->setButtonGuiItem(KDialog::User2, item);
        //dialog->enableButton(KDialog::User2, true);
    }
    if (question->possibleAnswers() & Shaman::TransactionQuestion::AnswerUser3) {
        KGuiItem item;
        item.setIcon(KIcon(question->user3Icon()));
        item.setText(question->user3Text());
        dialog->setButtonGuiItem(KDialog::User3, item);
        //dialog->enableButton(KDialog::User3, true);
    }

    int result = dialog->exec();

    Shaman::TransactionQuestion::Answer answer = Shaman::TransactionQuestion::UnknownAnswer;
    switch (result) {
    case KDialog::Yes:
        answer = Shaman::TransactionQuestion::AnswerYes;
        break;
    case KDialog::No:
        answer = Shaman::TransactionQuestion::AnswerNo;
        break;
    case KDialog::Cancel:
        answer = Shaman::TransactionQuestion::AnswerCancel;
        break;
    case KDialog::User1:
        answer = Shaman::TransactionQuestion::AnswerUser1;
        break;
    case KDialog::User2:
        answer = Shaman::TransactionQuestion::AnswerUser2;
        break;
    case KDialog::User3:
        answer = Shaman::TransactionQuestion::AnswerUser3;
        break;
    default:
        break;
    }

    dialog->hide();
    QWidget *w = dialog->mainWidget();
    w->setParent(0);
    dialog->setMainWidget(0);
    dialog->deleteLater();

    question->setAnswerWidget(w);
    question->setAnswer(answer);
}

