/* This file is part of the KDE project

   Copyright (C) 2008 - 2009 Lukas Appelhans <l.appelhans@gmx.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/
#include "ProcessQueueDialog.h"

#include "DownloadView.h"
#include "ProcessView.h"

#include "ui_ProcessQueueDialog.h"

#include <shamanbackend.h>
#include <shamantransactioninterface.h>
#include <shamanbase.h>
#include <shamancoreinterface.h>

#include <KLocale>
#include <KIcon>
#include <KMessageBox>
#include <KDebug>
#include <QMovie>
#include <KPushButton>
#include <QCloseEvent>
#include <QScrollArea>

using namespace Shaman;

ProcessQueueDialog::ProcessQueueDialog(Backend * backend, QWidget * parent)
  : ErrorQuestionHandler(parent)
  , m_backend(backend)
  , m_running(true)
{
    setAttribute(Qt::WA_DeleteOnClose);
    m_transactionInterface = Shaman::Base::instance()->core()->transactionInterface();
    setCaption(i18n("Process Queue"));
    setButtons(KDialog::Cancel);
    connect(button(KDialog::Cancel), SIGNAL(clicked(bool)), m_backend, SLOT(cancelQueueProcessing()));
    setModal(true);

    Ui::ProcessQueueDialog ui;
    QWidget * widget = new QWidget(this);
    ui.setupUi(widget);
    m_containerWidget = ui.containerWidget;
    m_containerWidget->hide();
    QVBoxLayout *lay = new QVBoxLayout;
    m_containerWidget->setLayout(lay);

    m_preHooksLabel = ui.preHooksLabel;
    m_preHooksLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_downloadLabel = ui.downloadLabel;
    m_downloadLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_installLabel = ui.installLabel;
    m_installLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_cleanupLabel = ui.cleanupLabel;
    m_cleanupLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_postHooksLabel = ui.postHooksLabel;
    m_postHooksLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_progressBar = ui.progressBar;
    setMainWidget(widget);

    connect(m_transactionInterface, SIGNAL(stateChanged(Shaman::TransactionInterface::State)), SLOT(stateChanged()));
    connect(m_transactionInterface, SIGNAL(transactionProgressChanged(int)), m_progressBar, SLOT(setValue(int)));
    connect(m_transactionInterface, SIGNAL(downloadProgressChanged(qint64,qint64)),
            this, SLOT(downloadProgressChanged(qint64,qint64)));
    connect(m_transactionInterface, SIGNAL(downloadProgressReliabilityChanged(bool)),
            this, SLOT(downloadProgressReliabilityChanged(bool)));
    connect(m_transactionInterface, SIGNAL(errorTriggered(Shaman::TransactionInterface::Error,QString)),
            this, SLOT(errorTriggered(Shaman::TransactionInterface::Error,QString)));
}

ProcessQueueDialog::~ProcessQueueDialog()
{
}

void ProcessQueueDialog::closeEvent(QCloseEvent* e)
{
    if (m_running) {
        // We really can't close right now
        if (KMessageBox::warningYesNo(this, i18n("A transaction is currently in progress. Do you want to abort it?")) ==
            KMessageBox::Yes) {
            m_backend->cancelQueueProcessing();
        }

        e->ignore();
    } else {
        KDialog::closeEvent(e);
    }
}

void ProcessQueueDialog::processQueue(const Shaman::ProcessQueueOption::List &options)
{
    while (m_containerWidget->layout()->count() > 0) {
        QLayoutItem * item = m_containerWidget->layout()->takeAt(0);
        if (item->widget()) {
            item->widget()->deleteLater();
        }
    }
    show();
    m_backend->processQueue(options);
    m_running = true;
}

void ProcessQueueDialog::stateChanged()
{
    Shaman::TransactionInterface::State state = m_transactionInterface->state();
    kDebug() << state;
    QMovie *movie;
    switch (state) {
        case TransactionInterface::RunningPreHooks:
            setDetailsWidget(ProcessViewWidget);
            movie = KIconLoader::global()->loadMovie("shaman-misc-processing", KIconLoader::NoGroup, -32, this);
            m_preHooksLabel->setMovie(movie);
            movie->start();
            m_progressBar->setRange(0,0);
            break;
        case TransactionInterface::Downloading:
            setDetailsWidget(DownloadViewWidget);
            m_preHooksLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-downloading", KIconLoader::NoGroup, -32, this);
            m_downloadLabel->setMovie(movie);
            movie->start();
            break;
        case TransactionInterface::Processing:
            setDetailsWidget(ProcessViewWidget);
            m_downloadLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-installing", KIconLoader::NoGroup, -32, this);
            m_installLabel->setMovie(movie);
            movie->start();
            m_progressBar->setRange(0, 100);
            m_progressBar->setValue(0);
            m_progressBar->setFormat("%p%");
            break;
        case TransactionInterface::Cleaning:
            setDetailsWidget(ProcessViewWidget);
            m_progressBar->setRange(0,0);
            m_installLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-cleanup", KIconLoader::NoGroup, -32, this);
            m_cleanupLabel->setMovie(movie);
            movie->start();
            break;
        case TransactionInterface::RunningPostHooks:
            setDetailsWidget(ProcessViewWidget);
            m_progressBar->setRange(0,0);
            m_installLabel->setPixmap(KIcon("dialog-ok").pixmap(32));//Some backends may not use the Cleaning state
            m_cleanupLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-misc-processing", KIconLoader::NoGroup, -32, this);
            m_postHooksLabel->setMovie(movie);
            movie->start();
            break;
        case TransactionInterface::NoTransaction:
            setDetailsWidget(ProcessViewWidget);
            m_postHooksLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            m_progressBar->setRange(0,100);
            if (m_transactionInterface->lastError() == TransactionInterface::NoError) {
                m_progressBar->setValue(100);
                m_progressBar->setFormat(i18n("Transaction completed"));
            } else {
                m_progressBar->setValue(0);
                m_progressBar->setFormat(i18n("Transaction failed"));
            }
            setButtons(KDialog::Close);
            m_running = false;
            break;
    }
}

void ProcessQueueDialog::errorTriggered(TransactionInterface::Error , QString )
{
    switch (m_transactionInterface->state()) {
        case Shaman::TransactionInterface::RunningPreHooks:
            m_preHooksLabel->setPixmap(KIcon("dialog-error").pixmap(32));
            break;
        case Shaman::TransactionInterface::Downloading:
            m_downloadLabel->setPixmap(KIcon("dialog-error").pixmap(32));
            break;
        case Shaman::TransactionInterface::Processing:
            m_installLabel->setPixmap(KIcon("dialog-error").pixmap(32));
            break;
        case Shaman::TransactionInterface::Cleaning:
            m_cleanupLabel->setPixmap(KIcon("dialog-error").pixmap(32));
            break;
        case Shaman::TransactionInterface::RunningPostHooks:
            m_postHooksLabel->setPixmap(KIcon("dialog-error").pixmap(32));
            break;
        default:
            break;
    }
    setButtons(KDialog::Close);
}

void ProcessQueueDialog::downloadProgressChanged(qint64 bytesDownloaded, qint64 bytesTotal)
{
    m_progressBar->setRange(0, bytesTotal);
    m_progressBar->setValue(bytesDownloaded);
    m_progressBar->setFormat(i18n("%1 / %2 (%3 / s)", KIO::convertSize(bytesDownloaded), KIO::convertSize(bytesTotal), 
                             KIO::convertSize(m_transactionInterface->downloadSpeed())));
}

void ProcessQueueDialog::downloadProgressReliabilityChanged(bool reliable)
{
    if (!reliable && m_transactionInterface->state()) {
        // Set the progress bar to a different appearance
        m_progressBar->setRange(0, 100);
        m_progressBar->setValue(m_transactionInterface->transactionProgress());
        m_progressBar->setFormat("%p%");
    }
}

void ProcessQueueDialog::setDetailsWidget(ProcessQueueDialog::DetailsWidget widget)
{
    // First clean up the layout
    while (m_containerWidget->layout()->count() > 0) {
        QLayoutItem * item = m_containerWidget->layout()->takeAt(0);
        if (item->widget()) {
            // Do not delete ProcessView
            if (!qobject_cast< ProcessView* >(item->widget())) {
                item->widget()->deleteLater();
            } else {
                // Hide it out instead
                item->widget()->hide();
            }
        }
    }

    switch (widget) {
        case ProcessViewWidget:
            if (m_processView.isNull()) {
                m_processView = new ProcessView;
            }
            m_containerWidget->layout()->addWidget(m_processView.data());
            m_processView.data()->show();
            break;
        case DownloadViewWidget:
            m_containerWidget->layout()->addWidget(new DownloadView);
            break;
    }
}

#include "ProcessQueueDialog.moc"
