/* This file is part of the KDE project

   Copyright (C) 2009 Dario Freddi <drf@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "UpdateDatabaseDialog.h"

#include "ui_UpdateDatabaseDialog.h"
#include <shamanbase.h>
#include <shamancoreinterface.h>
#include <QTimer>
#include <QFormLayout>
#include <QMovie>
#include <KDebug>
#include <KMessageBox>
#include <shamanbackend.h>
#include <QCloseEvent>
#include <shamanguiinterface.h>
#include <shamanstatusbarinterface.h>

using namespace Shaman;

UpdateDatabaseDialog::UpdateDatabaseDialog(Shaman::Repository::List repos, QWidget* parent, Qt::WFlags flags)
    : KDialog(parent, flags)
    , m_repositories(repos)
    , m_handleProgress(true)
    , m_running(true)
{
    setAttribute(Qt::WA_DeleteOnClose);
    m_ui = new Ui::UpdateDatabaseDialog();
    QWidget * widget = new QWidget(this);
    m_ui->setupUi(widget);
    m_ui->scrollArea->hide();

    m_transactionInterface = Shaman::Base::instance()->core()->transactionInterface();
    setCaption(i18n("Update Database"));
    setButtons(KDialog::Cancel);
    setModal(true);

    m_ui->preHooksLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_ui->updateLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));
    m_ui->postHooksLabel->setPixmap(KIcon("edit-clear-history").pixmap(32));

    setMainWidget(widget);
    loadRepositoriesWidget();

    connect(m_transactionInterface, SIGNAL(stateChanged(Shaman::TransactionInterface::State)),
            this, SLOT(stateChanged(Shaman::TransactionInterface::State)));
    connect(m_transactionInterface, SIGNAL(errorTriggered(Shaman::TransactionInterface::Error, QString)),
            this, SLOT(errorTriggered(Shaman::TransactionInterface::Error, QString)));
    connect(m_transactionInterface, SIGNAL(transactionProgressChanged(int)),
            this, SLOT(setTransactionProgress(int)));
}

UpdateDatabaseDialog::~UpdateDatabaseDialog()
{
}

void UpdateDatabaseDialog::closeEvent(QCloseEvent* e)
{
    if (m_running) {
        // We really can't close right now
        if (KMessageBox::warningYesNo(this, i18n("A transaction is currently in progress. Do you want to abort it?")) ==
            KMessageBox::Yes) {
            // TODO: Abort
        }

        e->ignore();
    } else {
        KDialog::closeEvent(e);
    }
}

void UpdateDatabaseDialog::errorTriggered(Shaman::TransactionInterface::Error , const QString& )
{

}

void UpdateDatabaseDialog::stateChanged(Shaman::TransactionInterface::State state)
{
    QMovie *movie;
    switch (state) {
        case TransactionInterface::RunningPreHooks:
            movie = KIconLoader::global()->loadMovie("shaman-misc-processing", KIconLoader::NoGroup, -32, this);
            m_ui->preHooksLabel->setMovie(movie);
            movie->start();
            m_ui->progressBar->setRange(0,0);
            break;
        case TransactionInterface::Downloading:
        case TransactionInterface::Processing:
        case TransactionInterface::Cleaning:
            m_ui->preHooksLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-refresh-cache", KIconLoader::NoGroup, -32, this);
            m_ui->updateLabel->setMovie(movie);
            movie->start();
            showRepositoriesWidget();
            m_ui->progressBar->setRange(0, m_repositories.count() * 2);
            m_ui->progressBar->setFormat("%p%");
            break;
        case TransactionInterface::RunningPostHooks:
            m_ui->progressBar->setRange(0,0);
            m_ui->updateLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            movie = KIconLoader::global()->loadMovie("shaman-misc-processing", KIconLoader::NoGroup, -32, this);
            m_ui->postHooksLabel->setMovie(movie);
            movie->start();
            break;
        case TransactionInterface::NoTransaction:
            m_ui->postHooksLabel->setPixmap(KIcon("dialog-ok").pixmap(32));
            m_ui->progressBar->setRange(0,100);
            m_ui->progressBar->setValue(100);
            m_ui->progressBar->setFormat(i18n("Package list refreshed"));
            setButtons(0);
            m_running = false;
            QTimer::singleShot(2500, this, SLOT(deleteLater()));
            break;
    }
}

void UpdateDatabaseDialog::loadRepositoriesWidget()
{
    m_layout = new QFormLayout(this);
    m_processed.clear();

    foreach (Repository *repo, m_repositories) {
        kDebug() << "Add repo" << repo->name();
        QHBoxLayout *layout = new QHBoxLayout(this);
        QLabel *textLabel = new QLabel(repo->name(), this);
        layout->addWidget(textLabel);
        QLabel *pixmapLabel = new QLabel(this);
        layout->addWidget(pixmapLabel);
        pixmapLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        pixmapLabel->setPixmap(KIcon("edit-clear-history").pixmap(22, 22));
        connect(repo, SIGNAL(stateChanged(Repository::State)), SLOT(updateRepoState(Repository::State)));
        m_repositoriesToLabel[repo] = pixmapLabel;
        m_layout->addRow(textLabel, pixmapLabel);
    }

    m_repositoriesWidget = new QWidget(); // Not parenting since it will be shown later!!
    m_repositoriesWidget->setLayout(m_layout);
}

void UpdateDatabaseDialog::showRepositoriesWidget()
{
    m_ui->scrollArea->setWidget(m_repositoriesWidget);
}

void UpdateDatabaseDialog::updateRepoState(Repository::State state)
{
    Repository *repo = qobject_cast< Repository* >(sender());
    if (state == Shaman::Repository::UpdatingFinished) {
        QPixmap icon = KIcon("dialog-ok-apply").pixmap(22, 22);
        m_repositoriesToLabel[repo]->setPixmap(icon);
        GuiInterface::instance()->statusBar()->showMessage(i18n("Finished updating %1.", repo->name()), icon);
        if (!m_processed.contains(repo) && m_handleProgress) {
            m_processed.append(repo);
            m_ui->progressBar->setValue(m_processed.count() * 2);
        }
    } else if (state == Shaman::Repository::Downloading) {
        QPixmap icon = KIcon("download").pixmap(22, 22);
        m_repositoriesToLabel[repo]->setPixmap(icon);
        GuiInterface::instance()->statusBar()->showMessage(i18n("Downloading %1...", repo->name()), icon);
        if (m_handleProgress) {
            m_ui->progressBar->setValue(m_processed.count() * 2 + 1);
        }
    } else if (state == Shaman::Repository::Updating) {
        QPixmap icon = KIcon("edit-redo").pixmap(22, 22);
        m_repositoriesToLabel[repo]->setPixmap(icon);
        GuiInterface::instance()->statusBar()->showMessage(i18n("Updating %1...", repo->name()), icon);
        if (m_handleProgress) {
            m_ui->progressBar->setValue(m_processed.count() * 2 + 1);
        }
    } else if (state == Shaman::Repository::Error) {
        QPixmap icon = KIcon("dialog-cancel").pixmap(22, 22);
        m_repositoriesToLabel[repo]->setPixmap(icon);
        GuiInterface::instance()->statusBar()->showMessage(i18n("Updating %1...", repo->name()), icon);
        if (m_handleProgress) {
            m_ui->progressBar->setValue(m_processed.count() * 2 + 1);
        }
    }
}

void UpdateDatabaseDialog::setTransactionProgress(int progress)
{
    m_handleProgress = false;
    m_ui->progressBar->setRange(0,100);
    m_ui->progressBar->setValue(progress);
}
